/*
	Virtual VCR
    Copyright (C) 2002  Shaun Faulds

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Acknowledgments:
	This application and associated filters are based on the examples
	from the Microsoft DirectX DirectShow SDK.
*/

#include <dshow.h>
#include "CFilterDescription.h"
#include <string>
using namespace std;

BOOL CFilterDescription::createInstance()
{

	TCHAR tempDesc[1024];
    WCHAR filterDesc[1024];
	wsprintf(tempDesc, TEXT("%s"), filterID.c_str());

#ifndef UNICODE
    MultiByteToWideChar(CP_ACP, 0, tempDesc, -1, filterDesc, NUMELMS(filterDesc));
#else
    wcscpy(tempDesc, T2W(filterDesc));
#endif

    IBindCtx *lpBC;
    HRESULT hr = CreateBindCtx(0, &lpBC);
	IMoniker *pmFilter = 0;
    if(SUCCEEDED(hr))
	{
        DWORD dwEaten;
        hr = MkParseDisplayName(lpBC, filterDesc, &dwEaten, &pmFilter);
        lpBC->Release();
    }

	if(pmFilter)
	{
		pmFilter->BindToObject(0, 0, IID_IBaseFilter, (void**)&filterPointer);
		pmFilter->Release();
	}
	else
	{
		loadErrMsg(TEXT("Could Not Load Filter: %s"), filterName);
	}

	return true;
}

int CFilterDescription::loadErrMsg(LPTSTR sz,...) 
{
    static TCHAR tach[2000];
    va_list va;

    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);
    MessageBox(NULL, tach, NULL, MB_OK|MB_ICONEXCLAMATION|MB_TASKMODAL);
    return FALSE;
}

BOOL CFilterDescription::releaseInstance()
{
	if(filterPointer)
		filterPointer->Release();
	filterPointer = NULL;

	return true;
}

CFilterDescription::CFilterDescription(string id, string name, string prop, BOOL mine)
{
	filterName = name;
	filterID = id;
	filterProperties = prop;
	filterPointer = NULL;
	isMine = mine;
}

CFilterDescription::~CFilterDescription()
{
	releaseInstance();
}

BOOL CFilterDescription::getIsMine()
{
	return isMine;
}

string CFilterDescription::getName()
{
	return filterName;
}

string CFilterDescription::getID()
{
	return filterID;
}

string CFilterDescription::getProp()
{
	return filterProperties;
}

CFilterDescription::setID(string id)
{
	filterID = id;
}

CFilterDescription::setProp(string prop)
{
	filterProperties = prop;
}

CFilterDescription::setIsMine(BOOL mine)
{
	isMine = mine;
}

IBaseFilter* CFilterDescription::getPointer()
{
	return filterPointer;
}