// CFilterGraphManager.h: interface for the CFilterGraphManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CFILTERGRAPHMANAGER_H__4E01907A_F2B2_4A64_AA7B_4F9F569227FA__INCLUDED_)
#define AFX_CFILTERGRAPHMANAGER_H__4E01907A_F2B2_4A64_AA7B_4F9F569227FA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <streams.h>
#include "CCrossbar.h"
#include "CAudioMixer.h"
#include "CFilterList.h"
#include "CFilterDescription.h"

class CFilterGraphManager  
{
public:
	CFilterGraphManager();
	virtual ~CFilterGraphManager();

	// Helper methods
	int setVideoCaptureFilter(IBaseFilter*);
	int setAudioCaptureFilter(IBaseFilter*);
	int setVideoCompressor(IBaseFilter*);
	int setAudioCompressor(IBaseFilter*);
	int setFilterList(CFilterList *list);

	// Filter Graph methods
	int buildCaptureGraph(BOOL);
	int initCaptureBuilder();

	int setCrossbarIndex(int inputNumber);
	int setCaptureOptions(RECT*, GUID, int);
	int setAudioInput();
	int setAudioFormat(int channles, int samplesPerSec, int avgBytesPerSec, int blockAlign, int bitsPerSample, int formatTag);
	int setVideoAmpSettings(int video_Brit, int video_Cont, int video_Hue, int video_Sat, int video_Colour);
	int setTunerChannel(int channel);
	int setTunerOption(int tunerInputType, int tunerCountryCode, int tunerMode);

	int addSmartTeeToGraph();

private:

	// Creation Methods
	int makeCaptureGraphBuilder();
	int makeFilterGraph();
	int addFilter(IBaseFilter*, WCHAR*);
	int addToROT();
	int addCrossBarHelper();
	IBaseFilter* addFileWriterSection(WCHAR *outputName, BOOL compatIndex, int masterStream);

	// Priavte data
	IBaseFilter *videoCapFilter;			// Video Capture Filter
	IBaseFilter *audioCapFilter;			// Audio Capture Filter

	IBaseFilter *videoCompressor;			// Video Compression Codec
	IBaseFilter *audioCompressor;			// Audio Compression Codec

	ICaptureGraphBuilder2 *capGraphBuilder;	// Capture Graph Builder
	IGraphBuilder *filterGraph;				// Graph builder

	DWORD graphRegister;					// Register graph with ROT

	CCrossbar *crossbar;

	int audioInputIndex;
	double audioInputLevel;
	BOOL usePreFilters;

	CFilterList *filterList;
};

#endif // !defined(AFX_CFILTERGRAPHMANAGER_H__4E01907A_F2B2_4A64_AA7B_4F9F569227FA__INCLUDED_)
