/*
	Virtual VCR
    Copyright (C) 2002  Shaun Faulds

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Acknowledgments:
	This application and associated filters are based on the examples
	from the Microsoft DirectX DirectShow SDK.
*/


#include <dshow.h>
#include "CFilterDescription.h"
#include "CFilterList.h"
#include <string>
#include <streams.h>
using namespace std;

//Include the common filter properties interface

#include "../Common/Filter/iFilterProperties.h"


CFilterList::CFilterList()
{

	memset(allFilters , 0, sizeof(allFilters[0]) * 5000);
	memset(selectedFilters , 0, sizeof(selectedFilters[0]) * 5000);
	
	getFilterList();
}

CFilterList::~CFilterList()
{
	for(int x = 0; x < 5000; x++)
	{
		// Delete the objects from the full list
		// becasue both the full list and the selected
		// list contain pointer to the same objects 
		// I only have to delete all the objects from the
		// full list, I just have to temember that the selected
		// list is not usable now.
		if(allFilters[x])
			delete allFilters[x];
	}
}

BOOL CFilterList::addFilterToSelected(int x)
{
	if(allFilters[x])
	{
		// Get instance of the filter
		allFilters[x]->createInstance();
		// Now add it to the first available
		// slot in the selected filter list
		for(int y = 0; y < 5000; y++)
		{
			if(!selectedFilters[y])
			{
				selectedFilters[y] = allFilters[x];
				break;
			}
		}
	}

	return true;
}

IBaseFilter* CFilterList::getFilterPointer(int offSet)
{
	if(selectedFilters[offSet])
	{
		return selectedFilters[offSet]->getPointer();
	}
	else
		return NULL;
}

BOOL CFilterList::isFilterMine(int x)
{
	if(allFilters[x])
	{	
		return allFilters[x]->getIsMine();
	}
	else
		return false;
	
}

BOOL CFilterList::isFilterInSelected(int index)
{
	if(!allFilters[index]) return false;

	CFilterDescription *source = allFilters[index];

	for(int x = 0; x < 5000; x++)
	{
		if(selectedFilters[x])
		{
			if(selectedFilters[x] == source)
				return true;
		}
		else
			break;
	}

	return false;
}

BOOL  CFilterList::moveFilterUP(int x)
{
	CFilterDescription *tempPointer;

		if(x > 0 && selectedFilters[x])
		{
			tempPointer = selectedFilters[x];
			selectedFilters[x] = selectedFilters[x-1];
			selectedFilters[x-1] = tempPointer;
		}

	return true;
}

BOOL  CFilterList::moveFilterDown(int x)
{
	CFilterDescription *tempPointer;

	if(x < 5000 && selectedFilters[x] && selectedFilters[x+1])
	{
		tempPointer = selectedFilters[x];
		selectedFilters[x] = selectedFilters[x+1];
		selectedFilters[x+1] = tempPointer;
	}

	return true;
}

BOOL CFilterList::removeAllFilterFromSelected()
{
	for(int y = 0; y < 5000; y++)
		selectedFilters[y] = 0;

	return true;
}

BOOL CFilterList::removeFilter(int x)
{
	CFilterDescription *tempPointer;

	while(selectedFilters[x])
	{
		tempPointer = selectedFilters[x+1];
		selectedFilters[x++] = tempPointer;
	}

	return true;
}

BOOL CFilterList::showPropBox(HWND hwnd, int x)
{
	if(x > -1 && x < 5000 && selectedFilters[x]->getPointer())
	{
		ISpecifyPropertyPages *pProp;
		HRESULT hr = selectedFilters[x]->getPointer()->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pProp);
		if (SUCCEEDED(hr)) 
		{
			// Get the filter's name and IUnknown pointer.
			IUnknown *pFilterUnk = NULL;
			selectedFilters[x]->getPointer()->QueryInterface(IID_IUnknown, (void **)&pFilterUnk);

			// Show the page. 
			CAUUID caGUID;
			pProp->GetPages(&caGUID);
			pProp->Release();

			OleCreatePropertyFrame(
				hwnd,                   // Parent window
				0, 0,                   // (Reserved)
				L"Filter",				// Caption for the dialog box
				1,                      // Number of objects (just the filter)
				&pFilterUnk,            // Array of object pointers. 
				caGUID.cElems,          // Number of property pages
				caGUID.pElems,          // Array of property page CLSIDs
				0,                      // Locale identifier
				0, NULL                 // Reserved
			);

			// Clean up.
			pFilterUnk->Release();
			CoTaskMemFree(caGUID.pElems);
		}
	}
	return true;
}

BOOL CFilterList::populateListBoxWithSelected(HWND handle, int item)
{
	SendDlgItemMessage(handle, item, LB_RESETCONTENT, 0, 0);
	for(int x = 0; x < 5000; x++)
	{
		if(selectedFilters[x])
			SendDlgItemMessage(handle, item, LB_ADDSTRING, 0, (LPARAM) selectedFilters[x]->getName().c_str());
		else
			break;
	}
	return true;
}

string CFilterList::getNameFromAllList(int offSet)
{
	if(allFilters[offSet])
		return allFilters[offSet]->getName();
	else
		return "end";
}

BOOL CFilterList::checkFilter(IBaseFilter* filter)
{
	int numberOfInput = 0;
	int numberOfOutput = 0;
	BOOL mediaGood = true;

	IEnumPins *pinEnum = NULL;
	filter->EnumPins(&pinEnum);
	IPin *pPin = NULL;

	if(pinEnum)
	{
		while(pinEnum->Next(1, &pPin, 0) == S_OK)
		{
			PIN_DIRECTION PinDirThis;
			pPin->QueryDirection(&PinDirThis);
			if (PINDIR_INPUT == PinDirThis)
				numberOfInput++;
			else
				numberOfOutput++;
/*
			AM_MEDIA_TYPE media;
			media.majortype = MEDIATYPE_Video;
			media.subtype = MEDIASUBTYPE_RGB24;
			
			try
			{
				HRESULT accept = pPin->QueryAccept(&media);
				if(accept == S_OK)
					mediaGood = true;
				else
					mediaGood = false;
			}
			catch(...)
			{
				loadErrMsg(TEXT("ERROR"));
				mediaGood = false;
			}


			IEnumMediaTypes *EnumMedia = NULL;
			pPin->EnumMediaTypes(&EnumMedia);

			if(EnumMedia)
			{
				AM_MEDIA_TYPE *mediaType = NULL;
				while(EnumMedia->Next(1, &mediaType, NULL) == S_OK)
				{
				//	if(!mediaType) break;
					if(mediaType->majortype == MEDIATYPE_Video)// && mediaType->subtype == MEDIASUBTYPE_RGB24)
						mediaGood = true;
					else
						mediaGood = false;

					DeleteMediaType(mediaType);
				}
			}
			if(EnumMedia)
				EnumMedia->Release();
			EnumMedia = NULL;
*/
			if(pPin)
				pPin->Release();
			pPin = NULL;
		}
		pinEnum->Release();
	}

	if(numberOfInput == 1 && numberOfOutput == 1 && mediaGood)
		return TRUE;
	else
		return FALSE;
}

/*
CFilterList::getFilterList2()
{
	try
	{

		HRESULT hr;
		ICreateDevEnum *pSysDevEnum = NULL;

		hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER,
			IID_ICreateDevEnum, (void **)&pSysDevEnum);

		// Obtain a class enumerator for the video compressor category.
		IEnumMoniker *pEnum = NULL;

		//CLSID_LegacyAmFilterCategory
		//CLSID_VideoCompressorCategory

		hr = pSysDevEnum->CreateClassEnumerator(
			CLSID_LegacyAmFilterCategory, &pEnum, 0);

		IMoniker *pMoniker = NULL;
		ULONG cFetched = NULL;
		IBaseFilter *pFilter = NULL;
		
		filterCount = 0;
	
		char name[256];

		while(pEnum->Next(1, &pMoniker, &cFetched) == S_OK)
		{
			IPropertyBag *pPropBag;
			pFilter = NULL;

			pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);

			if(pPropBag)
			{
				VARIANT varName;
				VariantInit(&varName);
				varName.vt = VT_BSTR;
				pPropBag->Read(L"FriendlyName", &varName, 0);
				WideCharToMultiByte(CP_ACP, 0, varName.bstrVal, -1, name, 128, NULL, NULL);
				VariantClear(&varName);

//loadErrMsg(TEXT("%s"), name);

				if(true)//strcmp(name, "Smart Tee") != 0)
				{
					pMoniker->BindToObject(NULL, NULL, IID_IBaseFilter, (void**)&pFilter);
					
					if(pFilter)
					{
						if(checkFilter(pFilter))
						{
							IFilterProperties *setupd;
							pFilter->QueryInterface(IID_IFilterProperties, (void **)&setupd);
							// If the filter has the IID_IFilterProperties interface
							// then set the isMine property to true as this filter
							// is one of mine.
							
							BOOL mine = false;
							
							if(setupd)
							{
								mine = true;
								setupd->Release();		
							}
							
							CLSID filterCLSID;
							pFilter->GetClassID(&filterCLSID);
							allFilters[filterCount++] =
								new FilterDescription(filterCLSID, name, "none", mine);
						}
					}
									

				}
				else
				{
					loadErrMsg(TEXT("%s"), name);
				}
			}

			if(pPropBag)
				pPropBag->Release();
			pPropBag = NULL;

			if(pFilter)
			{
				pFilter->Release();
			}
			pFilter = NULL;

			if(pMoniker)
				pMoniker->Release();
			pMoniker = NULL;
		}
		if(pEnum)
			pEnum->Release();
		pEnum = NULL;
	}
	catch(...)
	{
		loadErrMsg(TEXT("Exception Thrown While Getting Filter List"));
	}

//	loadErrMsg(TEXT("Filters Loaded %d"),filterCount);
}
*/

/*
CFilterList::getFilterList()
{
	int totalCreated = 0;
	try
	{
		IFilterMapper2 *pMapper = NULL;

		IEnumMoniker *pEnum = NULL;
		
		CoCreateInstance(CLSID_FilterMapper2, 
			NULL, CLSCTX_INPROC, IID_IFilterMapper2, 
			(void **) &pMapper);

		GUID arrayInTypes[2];
		arrayInTypes[0] = MEDIATYPE_Video;
		arrayInTypes[1] = GUID_NULL;

		pMapper->EnumMatchingFilters(
			&pEnum,
			0,                  // Reserved.
			TRUE,				// Use exact match?
			MERIT_DO_NOT_USE,	// Minimum merit.
			TRUE,              // At least one input pin?
			1,                  // Number of major type/subtype pairs for input.
			arrayInTypes,       // Array of major type/subtype pairs for input.
			NULL,               // Input medium.
			NULL,               // Input pin category.
			FALSE,				// Must be a renderer?
			TRUE,               // At least one output pin?
			1,                  // Number of major type/subtype pairs for output.
			arrayInTypes,       // Array of major type/subtype pairs for output.
			NULL,               // Output medium.
			NULL);              // Output pin category.

		IMoniker *pMoniker = NULL;
		ULONG cFetched = NULL;
		IBaseFilter *pFilter = NULL;
		
		filterCount = 0;
	
		char name[256];
		while(pEnum->Next(1, &pMoniker, &cFetched) == S_OK)
		{
			IPropertyBag *pPropBag;
			pFilter = NULL;

			pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);

			if(pPropBag)
			{
				VARIANT varName;
				VariantInit(&varName);
				varName.vt = VT_BSTR;
				pPropBag->Read(L"FriendlyName", &varName, 0);
				WideCharToMultiByte(CP_ACP, 0, varName.bstrVal, -1, name, 128, NULL, NULL);
				VariantClear(&varName);

				if(checkFilterName(name))
				{
					pMoniker->BindToObject(NULL, NULL, IID_IBaseFilter, (void**)&pFilter);
					if(pFilter)
					{
						if(checkFilter(pFilter))
						{
							IFilterProperties *setupd;
							pFilter->QueryInterface(IID_IFilterProperties, (void **)&setupd);
							// If the filter has the IID_IFilterProperties interface
							// then set the isMine property to true as this filter
							// is one of mine.
							
							BOOL mine = false;
							
							if(setupd)
							{
								mine = true;
								setupd->Release();		
							}
							//setupd = NULL;
							
							CLSID filterCLSID;
							pFilter->GetClassID(&filterCLSID);
							allFilters[filterCount++] =
								new FilterDescription(filterCLSID, name, "none", mine);
						}
					}
				}
				else
				{
					//loadErrMsg(TEXT("NOT USED %s"), name);
				}
				
			}

			if(pPropBag)
				pPropBag->Release();
			pPropBag = NULL;

			if(pFilter)
			{
				totalCreated ++;
				pFilter->Release();
			}
			pFilter = NULL;

			if(pMoniker)
				pMoniker->Release();
			pMoniker = NULL;
		}
		if(pEnum)
			pEnum->Release();
		pEnum = NULL;
	}
	catch(...)
	{
		loadErrMsg(TEXT("Exception Thrown While Getting Filter List"));
	}
	//loadErrMsg(TEXT("Filter Created %d Filters Used %d"), totalCreated, filterCount);
}
*/

CFilterList::getFilterList()
{
	int totalCreated = 0;
	try
	{
		IFilterMapper2 *pMapper = NULL;
		IEnumMoniker *pEnum = NULL;
		
		CoCreateInstance(CLSID_FilterMapper2, 
			NULL, CLSCTX_INPROC, IID_IFilterMapper2, 
			(void **) &pMapper);

		GUID arrayInTypes[2];
		arrayInTypes[0] = MEDIATYPE_Video;
		arrayInTypes[1] = GUID_NULL;

		HRESULT hr = pMapper->EnumMatchingFilters(
			&pEnum,
			0,                  // Reserved.
			TRUE,				// Use exact match?
			MERIT_DO_NOT_USE,	// Minimum merit.
			TRUE,	            // At least one input pin?
			1,                  // Number of major type/subtype pairs for input.
			arrayInTypes,       // Array of major type/subtype pairs for input.
			NULL,               // Input medium.
			NULL,               // Input pin category.
			FALSE,				// Must be a renderer?
			TRUE,               // At least one output pin?
			1,                  // Number of major type/subtype pairs for output.
			arrayInTypes,       // Array of major type/subtype pairs for output.
			NULL,               // Output medium.
			NULL);              // Output pin category.

		IMoniker *pMoniker;
		ULONG cFetched;
		
		filterCount = 0;
	
		char name[256];
		TCHAR *tach = NULL;
		WCHAR *szDisplayName = NULL;

		while(pEnum->Next(1, &pMoniker, &cFetched) == S_OK)
		{
			IPropertyBag *pPropBag;
			pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);

			if(pPropBag)
			{
				VARIANT varName;
				VariantInit(&varName);
				varName.vt = VT_BSTR;
				pPropBag->Read(L"FriendlyName", &varName, 0);
				WideCharToMultiByte(CP_ACP, 0, varName.bstrVal, -1, name, 128, NULL, NULL);
				VariantClear(&varName);

				szDisplayName = 0;
				pMoniker->GetDisplayName(0, 0, &szDisplayName);
				
				if(pMoniker)
				{
					if(SUCCEEDED(pMoniker->GetDisplayName(0, 0, &szDisplayName)))
					{   
						if(szDisplayName)
						{
							tach = new TCHAR [wcslen(szDisplayName)+1];
							wsprintf(tach, TEXT("%S"), szDisplayName);
							CoTaskMemFree(szDisplayName);
						}

					}
				}
				BOOL isMine = checkFilterName(name);
				allFilters[filterCount++] =
					new CFilterDescription(tach, name, "none", isMine);
			}

			delete(tach);
			tach = NULL;

			if(pPropBag)
				pPropBag->Release();
			pPropBag = NULL;

			if(pMoniker)
				pMoniker->Release();
			pMoniker = NULL;
		}
		if(pEnum)
			pEnum->Release();
		pEnum = NULL;


	}
	catch(...)
	{
		loadErrMsg(TEXT("Exception Thrown While Getting Filter List"));
	}
	//loadErrMsg(TEXT("Filter Created %d Filters Used %d"), totalCreated, filterCount);
}


BOOL CFilterList::checkFilterName(char *name)
{
	char *names[256];
	ZeroMemory(names, 256 * sizeof(names[0]));
	names[0] = "Video Crop Filter";
	names[1] = "Colour Histogram";

	for(int x=0; x < 256; x++)
	{
		if(names[x] == 0) return false;
		if(strcmp(name, names[x]) == 0)
			return true;
	}

	return false;
}

int CFilterList::loadErrMsg(LPTSTR sz,...) 
{
    static TCHAR tach[2000];
    va_list va;

    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);
    MessageBox(NULL, tach, NULL, MB_OK|MB_ICONEXCLAMATION|MB_TASKMODAL);
    return FALSE;
}

string CFilterList::getIDstring(int x)
{
	return selectedFilters[x]->getID();
}

void CFilterList::selectFilterByIDString(string idSting, char* prop)
{
	for(int x = 0; x < 5000; x++)
	{
		if(allFilters[x])
		{
			if(allFilters[x]->getID() == idSting)
			{
				// Get instance of the filter
				allFilters[x]->createInstance();
				// Now add it to the first available
				// slot in the selected filter list
				
				for(int y = 0; y < 5000; y++)
				{
					if(!selectedFilters[y])
					{
						selectedFilters[y] = allFilters[x];
						setFilterPropertyString(y, prop);
						break;
					}
				}
				
				break;
			}
		}
		else
			break;
	}

}

char* CFilterList::getFilterPropertyString(int x)
{

	char *filterData = new char[1024];
	wsprintf(filterData, "none");

	if(selectedFilters[x])
	{
		IFilterProperties *setupd;
		selectedFilters[x]->getPointer()->QueryInterface(IID_IFilterProperties, (void **)&setupd);
		if(setupd)
		{
			char *newData = NULL;
			setupd->get_Prop(*&newData);
			wsprintf(filterData, "%s", newData);
			setupd->Release();
		}
	}

	return filterData;
}

BOOL CFilterList::setFilterPropertyString(int x, char* prop)
{
	if(selectedFilters[x])
	{
		IFilterProperties *setupd;
		selectedFilters[x]->getPointer()->QueryInterface(IID_IFilterProperties, (void **)&setupd);
		if(setupd)
		{
			setupd->set_Prop(prop);
			setupd->Release();
		}
	}

	return true;
}