/*
	Virtual VCR
    Copyright (C) 2002  Shaun Faulds

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Acknowledgments:
	This application and associated filters are based on the examples
	from the Microsoft DirectX DirectShow SDK.
*/

#if !defined(VirtualVCRMAIN_Header)
#define VirtualVCRMAIN_Header

//------------------------------------------------------------------------------
// Funciton Prototypes
//------------------------------------------------------------------------------
typedef LONG(PASCAL *LPWNDPROC)(HWND, UINT, WPARAM, LPARAM); // pointer to a window procedure
LONG WINAPI AppWndProc(HWND hwnd, UINT uiMessage, WPARAM wParam, LPARAM lParam);
LONG PASCAL AppCommand(HWND hwnd, unsigned msg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
int ErrMsg(LPTSTR sz,...);
BOOL SetCaptureFile(HWND hWnd);
int DoDialog(HWND hwndParent, int DialogID, DLGPROC fnDialog, long lParam);
int FAR PASCAL CaptureStatsProc(HWND hDlg, UINT Message, UINT wParam, LONG lParam);

void TearDownGraph(void);
BOOL BuildCaptureGraph();
BOOL BuildPreviewGraph();
void UpdateStatus(bool lastUpdate);
void ScanForDevices();
void ChooseDevices(TCHAR *szVideo, TCHAR *szAudio);
void ChooseDevices(IMoniker *pmVideo, IMoniker *pmAudio);
void ChooseFrameRate();
BOOL InitCapFilters();
void FreeCapFilters();
BOOL StopPreview();
BOOL StartPreview();
BOOL StopCapture();
BOOL StartCapture();
DWORDLONG GetSize(LPCTSTR tach);
void OnClose();
RECT GetVideoSize(IPin *pCapPin);

// Adds/removes a DirectShow filter graph from the Running Object Table,
// allowing GraphEdit to "spy" on a remote filter graph if enabled.
HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister);
void RemoveGraphFromRot(DWORD pdwRegister);
void ResizeWindow();
int FAR PASCAL StopConditionsProc(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
static long GetFreeDiskSpaceInKB(LPTSTR pFile);
BOOL FormatNumber(LONGLONG number, TCHAR* buff);
BOOL ShowCapStats(BOOL show);
BOOL TryToStartCapture();
BOOL AddSmartTeeToGraph();
void NukeAllUnconnectedFilters();

int FAR PASCAL SetFileNameProc(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
BOOL ParseCommand(LPSTR szCmdLine, char *value, char *param);
void changeChannel(int channel);
HWND DoDialogModeless(HWND hwndParent, int DialogID, DLGPROC fnDialog, long lParam);
void DoKeyPressInput(int key);
void ProcessNumInput(int number, bool change);
BOOL OpenFileDialog(HWND hWnd, LPTSTR pszName, int cb);
BOOL OpenSaveDialog(HWND hWnd, LPTSTR pszName, int cb);
BOOL OpenLoadDialog(HWND hWnd, LPTSTR pszName, int cb);
BOOL ShowLoaderDialog(BOOL show);
int MsgBox(LPTSTR sz,...);
void getTunerSettings();

BOOL GetAudioFormat();
BOOL SetAudioFormat();
int GetVersionInfo(char *returnString, bool longVersion);

void ChooseInitDevices(int indexVideo, int indexAudio);
int FAR PASCAL DroppedFrameProc(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
bool RollCapFileOver();

// Properties pages
VOID ShowPropPages(HWND hwndOwner);
int FAR PASCAL PROP_ABOUT_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_VIDEO_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_FILTERS_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_FILE_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_STOP_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_VIEW_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_AUDIO_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_DEVICES_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_TUNER_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);
int FAR PASCAL PROP_AVSYNC_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam);

int FAR PASCAL ScanProgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
int FAR PASCAL HelpDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

BOOL PopulateDeviceDropDownBox(HWND hwndCombo);
BOOL DoPropertiesDialog(int index);

void logData(LONG, double, double, double, double, LONG, LONG, BOOL);
void logAction(LPTSTR sz,...);
void toggleFullScreen();

void getVideoAmpSettings();
void setVideoAmpSettings();

int SetVideoFormat(int pintype);
int hrToText(HRESULT hr, CHAR *error);
int getFilterVersion(char *ver, GUID filterGUID);

HWND CreateToolBar(HWND mainWindow); 
void scannForTVChannels( void *mode );
int ChannelChange(int amount);

//------------------------------------------------------------------------------

#define ID_APP      1000

/* Menu Items */
#define MENU_START_CAP		7
#define MENU_STOP_CAP		8
#define MENU_PREVIEW		15


// Dialogs
#define IDD_ABOUT           	    600

// defines for dialogs
#define IDC_FRAMERATE		    402
#define IDC_CAPFILENAME		    403
#define IDC_USETIMELIMIT	    405

// window messages
#define WM_FGNOTIFY	WM_USER+1
#define WM_IGNORE WM_USER+2

#define ID_TOOLBAR				66666

#endif //VirtualVCRMAIN_Header
