/*
	Virtual VCR
    Copyright (C) 2002  Shaun Faulds

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Acknowledgments:
	This application and associated filters are based on the examples
	from the Microsoft DirectX DirectShow SDK.
*/

#include <windows.h>
#include <dbt.h>
#include <streams.h>
#include <mmreg.h>
#include <msacm.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include <commdlg.h>
#include <atlbase.h>
#include <Commctrl.h>
#include <time.h>
#include <Prsht.h>
#include <fstream.h>
#include <iostream.h>
#include <Dxerr8.h>
#include <process.h>

#include "stdafx.h"
#include "VirtualVCR.h"
#include "status.h"
#include "CCrossbar.h"
#include "CCodec.h"
#include "resource.h"
#include "CFilterDescription.h"
#include "CFilterList.h"
#include "CAppPropData.h"
#include "CAudioMixer.h"
#include "LimitSingleInstance.h"
#include "CFilterGraphManager.h"

#include <initguid.h>

// Include the Flow Meter CLSID's and interface defenitions
#include "../FlowMeter/iFlowMeter.h"
#include "../FlowMeter/FlowMeterUIDs.h"

// Include the Flow Meter CLSID's and interface defenitions
#include "../AudioResample/iAudioResample.h"
#include "../AudioResample/AudioResampleUIDs.h"

// Include the common filter properties interface
#include "../Common/Filter/iFilterProperties.h"

// Include the Audio Scope GUID file
#include "../Scope/scopeGUID.h"

// Include the VideoCrop GUID
#include "../VideoCrop/VideoCropUIDs.h"

// Inlcude the Histogram GUID
#include "../Histogram/HistogramUIDs.h"

// you can never have too many parentheses!
#define ABS(x) (((x) > 0) ? (x) : -(x))

// An application can advertise the existence of its filter graph
// by registering the graph with a global Running Object Table (ROT).
// The GraphEdit application can detect and remotely view the running
// filter graph, allowing you to 'spy' on the graph with GraphEdit.
//
// To enable registration in this sample, define REGISTER_FILTERGRAPH.
//
#ifdef  DEBUG
#define REGISTER_FILTERGRAPH
#endif

#define REGISTER_FILTERGRAPH

//------------------------------------------------------------------------------
// Global data
//------------------------------------------------------------------------------
HINSTANCE ghInstApp;
HACCEL ghAccel;
HFONT  ghfontApp;
TEXTMETRIC gtm;
TCHAR gszAppName[]=TEXT("Virtual VCR");
HWND ghwndApp, ghwndStatus;
DWORD g_dwGraphRegister=0;

HWND hwndSTATUS = NULL; // stats dialog 
HWND hwndLOADER = NULL; // loader Dialog 
CAppPropData *appData; // persistant app data object

CLimitSingleInstance g_SingleInstanceObj(TEXT("Virtual VCR Application"));


struct _capstuff {

	char *cmdLine;

	::ofstream logFile; // log file stream
	::ofstream actionLog; //action log file
	BOOL log;

	IMoniker *pmVideo;
    IMoniker *pmAudio;

    ICaptureGraphBuilder2 *pBuilder;
    IVideoWindow *pVW;
    IMediaEventEx *pME;
    IAMDroppedFrames *pDF;
//    IAMVideoCompression *pVC;
    IAMVfwCaptureDialogs *pDlg;
//    IAMStreamConfig *pASC;      // for audio cap
//    IAMStreamConfig *pVSC;      // for video cap

    IBaseFilter *pVCap, *pACap;	// Video and Audio capture devices

	IBaseFilter *SmartTeeFilter;
	IBaseFilter *VideoRender;  //  Video Render

	boolean bAutoStartCapture;	// This is set from the cammand line to start the capture imediatly 

	// Flow Meter Pointers
	IBaseFilter *FlowMeterVideo;
	IFlowMeter *FlowMeterVideoInterface;
	IBaseFilter *FlowMeterAudio;
	IFlowMeter *FlowMeterAudioInterface;

	// Audio Resample Sync Filter
	IBaseFilter *AudioResampleFilter;
	IAudioResample *AudioResampleInterface;

	IBaseFilter *audioScope;
	IBaseFilter *colourHistogram;
	IBaseFilter *videoRender;

    IGraphBuilder *pFg;
    IFileSinkFilter *pSink;
    IConfigAviMux *pConfigAviMux;
	IBaseFilter *pFileWriter;
	IBaseFilter *pAVIMux;

    BOOL fCaptureGraphBuilt;
    BOOL fPreviewGraphBuilt;
    BOOL fCapturing;
    BOOL fPreviewing;

    BOOL fCapAudioIsRelevant;
    bool fDeviceMenuPopulated;
    IMoniker *rgpmVideoMenu[10];
    IMoniker *rgpmAudioMenu[10];

    long lCapStartTime;
    long lCapStopTime;
	long lTimeHold;
	__int64 lFreeSpaceHold;
	DWORDLONG StartFreeSpace;
    WCHAR wachFriendlyName[120];

    long lDroppedBase;
    long lNotBase;
    CCrossbar *pCrossbar;
    LONG NumberOfVideoInputs;
    int iNumVCapDevices;
	int iNumACapDevices;

	//Tuner Channel selection stuff.
	LONG lastKeysPressed; // Time stamp of last key press
	int channelBuilder;
	BOOL tunerSet;

	// Shows the current loading status
	char *status;
	int filterCounter;

	// Profile Loading from cammand line
	char profileToLoad[1024];
	BOOL profileFound;

	double AVsync; // AV sync value
	__int64 freeHDatStart; // Free HD space when cap started
	__int64 *droppedList; //Dropped Frame List

	HWND aboutHWND;

	int capFileCount;

	CAudioMixer *audioMix;			// Audio input mixer
	int toolBarHeight;
	HWND toolBarHWND;

	BOOL screenSaverState;
	BOOL channelScanning;
	HWND channelScanningStatus;

	TCHAR currentFileName[256];

} gcap;


//------------------------------------------------------------------------------
// Name: SetAppCaption()
// Desc: Set the caption to be the application name followed by the capture file
//------------------------------------------------------------------------------
void SetAppCaption() 
{
    TCHAR tach[_MAX_PATH + 80];
    lstrcpy(tach, gszAppName);
    if(appData->szCaptureFile[0] != 0)
	{
        lstrcat(tach, TEXT(" - "));
		
		if(appData->appendStampToName)
		{
			TCHAR fileName[256];
			// copy accross the file name minus the extension
			int nameLength = 0;
			for(int x=0; x < 256; x++)
			{
				if(appData->szCaptureFile[x] == 0)
				{
					nameLength = x;
					break;
				}
			}
			strncpy(fileName, appData->szCaptureFile, nameLength-4);
			fileName[nameLength-4] = 0;
			
			lstrcat(tach, fileName);
			lstrcat(tach, TEXT(" (YYYY-MM-DD at HH-MM-SS).avi"));
		}
		else
			lstrcat(tach, appData->szCaptureFile);
    }
    SetWindowText(ghwndApp, tach);
}


/*----------------------------------------------------------------------------*\
|   AppInit( hInst, hPrev)                                                     |
|                                                                              |
|   Description:                                                               |
|       This is called when the application is first loaded into               |
|       memory.  It performs all initialization that doesn't need to be done   |
|       once per instance.                                                     |
|                                                                              |
|   Arguments:                                                                 |
|       hInstance       instance handle of current instance                    |
|       hPrev           instance handle of previous instance                   |
|                                                                              |
|   Returns:                                                                   |
|       TRUE if successful, FALSE if not                                       |
|                                                                              |
\*----------------------------------------------------------------------------*/
BOOL AppInit(HINSTANCE hInst, HINSTANCE hPrev, int sw) 
{

	gcap.audioScope = NULL;

    WNDCLASS    cls;
    HDC         hdc;

    const DWORD  dwExStyle = 0;

	CoInitialize(NULL);
    DbgInitialise(hInst);

    /* Save instance handle for DialogBoxs */
    ghInstApp = hInst;

    ghAccel = LoadAccelerators(hInst, MAKEINTATOM(ID_APP));

    if(!hPrev)
	{
        /*
        *  Register a class for the main application window
        */
        cls.hCursor        = LoadCursor(NULL,IDC_ARROW);
        cls.hIcon          = LoadIcon(hInst, TEXT("IDI_FILM"));
        cls.lpszMenuName   = NULL;//MAKEINTATOM(ID_APP); // Menu
        cls.lpszClassName  = MAKEINTATOM(ID_APP);
        cls.hbrBackground  = (HBRUSH)(COLOR_WINDOW + 1);
        cls.hInstance      = hInst;
        cls.style          = CS_BYTEALIGNCLIENT | CS_VREDRAW | CS_HREDRAW | CS_DBLCLKS;
        cls.lpfnWndProc    = (WNDPROC) AppWndProc;
        cls.cbWndExtra     = 0;
        cls.cbClsExtra     = 0;

        if(!RegisterClass(&cls))
            return FALSE;
    }

	// Is this necessary?
    ghfontApp = (HFONT)GetStockObject(ANSI_VAR_FONT);
    hdc = GetDC(NULL);
    SelectObject(hdc, ghfontApp);
    GetTextMetrics(hdc, &gtm);
    ReleaseDC(NULL, hdc);

	// Show the loader dialog
	ShowLoaderDialog(true);
	if(gcap.aboutHWND != NULL)
		SetDlgItemText(gcap.aboutHWND, IDC_STATUS, "Loading Application Data");

	// Create the persistant data object and load all the saved settings
	appData = new CAppPropData();
	BOOL loaded = appData->loadData();
	if(!loaded)
		ErrMsg(TEXT("Could not load the VirtualVCR.ini file"));
	// End of load settings

	// Use new Profile File is asked to
	if(gcap.profileFound)
	{
		loaded = appData->loadProfileData(gcap.profileToLoad, false);
		if(!loaded)
			ErrMsg(TEXT("Error loading the profile file: %s"), gcap.profileToLoad);
	}

	// Over ride the channel setting
	char channel[256];
	BOOL channelFound = ParseCommand(gcap.cmdLine, channel, "-channel");
	if(channelFound)
	{
		char *end;
		int newChannel = strtol(channel, &end, 10);
		
		if(newChannel > 0 && newChannel < 999)
		{
			gcap.channelBuilder = newChannel;
			ProcessNumInput(0, true);
		}
	}
	
	// Over ride the stop condition
	char stopTime[256];
	BOOL stopFound = ParseCommand(gcap.cmdLine, stopTime, "-runfor");
	if(stopFound)
	{
		char *end;
		appData->dwTimeLimit = strtol(stopTime, &end, 10);
		appData->fUseTimeLimit = true;
	}

	// Over ride the capture file name
	//TCHAR szCaptureFile
	TCHAR newOutput[256];
	BOOL outputFound = ParseCommand(gcap.cmdLine, newOutput, "-output");
	if(outputFound)
	{
		strcpy(appData->szCaptureFile, newOutput);
	}

	// Do we want to log activity
	gcap.log = ParseCommand(gcap.cmdLine, NULL, "-logging");

    ghwndApp=CreateWindowEx(dwExStyle,
        MAKEINTATOM(ID_APP),    // Class name
        gszAppName,             // Caption
        // Style bits
        WS_SYSMENU | WS_MINIMIZEBOX | WS_CLIPCHILDREN,//WS_OVERLAPPEDWINDOW,
		appData->iApplicationWindowX,	//X Window location
		appData->iApplicationWindowY,	//Y Window location
        //CW_USEDEFAULT, 0,       // Position
        320,300,                // Size
        (HWND)NULL,             // Parent window (no parent)
        (HMENU)NULL,            // use class menu
        hInst,                  // handle to window instance
        (LPSTR)NULL             // no params to pass on
        );

    // create the status bar
    statusInit(hInst, hPrev);
    ghwndStatus = CreateWindowEx(0,
        szStatusClass,
        NULL,
        WS_CHILD|WS_BORDER|WS_VISIBLE|WS_CLIPSIBLINGS,
        0, 0,
        0, 0,
        ghwndApp,
        NULL,
        hInst,
        NULL);
    if(ghwndStatus == NULL)
        return(FALSE);

	if(gcap.aboutHWND != NULL)
		SetDlgItemText(gcap.aboutHWND, IDC_STATUS, "Scanning For Devices");

//////////////////////////////////////////////////////////////////////////////

    // get which devices to use
    ZeroMemory(gcap.rgpmAudioMenu, sizeof(gcap.rgpmAudioMenu));
    ZeroMemory(gcap.rgpmVideoMenu, sizeof(gcap.rgpmVideoMenu));
    gcap.pmVideo = 0;
    gcap.pmAudio = 0;

    gcap.fDeviceMenuPopulated = false;
    ScanForDevices();

	if(gcap.aboutHWND != NULL)
		SetDlgItemText(gcap.aboutHWND, IDC_STATUS, "Selecting Devices");

	ChooseInitDevices(appData->VideoDeviceIndex, appData->AudioDeviceIndex);

///////////////////////////////////////////////////////////////////////////////

	// Now show the main Window
    ShowWindow(ghwndApp, sw);

	// Add the Toolbar to the main window
	gcap.toolBarHWND = NULL;
	gcap.toolBarHWND = CreateToolBar(ghwndApp);
	
	// Show Cap Stats
	ShowCapStats(appData->fWantCaptureStats);

    SetAppCaption();

	ShowLoaderDialog(false);

	// This delay may help some people with slow tuner cards
	Sleep (1000);

	// If auto capture is set in the command line start the capture
	if(gcap.bAutoStartCapture)
	{
		if(gcap.fPreviewing)
			StopPreview();

		if(gcap.fPreviewGraphBuilt)
			TearDownGraph();

		BuildCaptureGraph();
		StartCapture();
	}
	else if(appData->fWantPreview)
		StartPreview();

    return TRUE;
}

void IMonRelease(IMoniker *&pm) 
{
    if(pm)
	{
        pm->Release();
        pm = 0;
    }
}

/*----------------------------------------------------------------------------*\
|   WinMain( hInst, hPrev, lpszCmdLine, cmdShow )                              |
|                                                                              |
|   Description:                                                               |
|       The main procedure for the App.  After initializing, it just goes      |
|       into a message-processing loop until it gets a WM_QUIT message         |
|       (meaning the app was closed).                                          |
|                                                                              |
|   Arguments:                                                                 |
|       hInst           instance handle of this instance of the app            |
|       hPrev           instance handle of previous instance, NULL if first    |
|       szCmdLine       ->null-terminated command line                         |
|       cmdShow         specifies how the window is initially displayed        |
|                                                                              |
|   Returns:                                                                   |
|       The exit code as specified in the WM_QUIT message.                     |
|                                                                              |
\*----------------------------------------------------------------------------*/
int PASCAL WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw) 
{
	// Check to see if we are already running
	// and if so return to quit.
	
    if (g_SingleInstanceObj.IsAnotherInstanceRunning())
	{
		MsgBox(TEXT("Virtual VCR is already running."));
		return FALSE;
	}
	
    MSG msg;
	
	gcap.cmdLine = new char[1024];
	strcpy(gcap.cmdLine, szCmdLine);
	
	//process command line arguments.
	char location[256];
	BOOL found = ParseCommand(szCmdLine, location, "-capture");
	gcap.bAutoStartCapture = found;

	// Do we want to use a custom profile
	gcap.profileFound = ParseCommand(szCmdLine, gcap.profileToLoad, "-profile");
	
    /*
	Call initialization procedure
	*/
    if(!AppInit(hInst,hPrev,sw))
        return FALSE;
	
		/*
		Polling messages from event queue
    */
    for(;;) {
        while(PeekMessage(&msg, NULL, 0, 0,PM_REMOVE)) {
            if(msg.message == WM_QUIT)
                return ((int) msg.wParam);
			
            if(TranslateAccelerator(ghwndApp, ghAccel, &msg))
                continue;
			
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
		
        WaitMessage();
    }
	
    // not reached
    return ((int) msg.wParam);
}


/*----------------------------------------------------------------------------*\
|   AppWndProc( hwnd, uiMessage, wParam, lParam )                              |
|                                                                              |
|   Description:                                                               |
|       The window proc for the app's main (tiled) window.  This processes all |
|       of the parent window's messages.                                       |
|                                                                              |
|   Arguments:                                                                 |
|       hwnd            window handle for the window                           |
|       msg             message number                                         |
|       wParam          message-dependent                                      |
|       lParam          message-dependent                                      |
|                                                                              |
|   Returns:                                                                   |
|       0 if processed, nonzero if ignored                                     |
|                                                                              |
\*----------------------------------------------------------------------------*/
LONG WINAPI AppWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) 
{
    PAINTSTRUCT ps;
    HDC hdc;
    RECT rc;
    int cxBorder, cyBorder, cy;

    switch(msg) {
        case WM_CREATE:
            break;

        case WM_COMMAND:
            return AppCommand(hwnd,msg,wParam,lParam);

        case WM_INITMENU:

            // we can start capture if not capturing already
            EnableMenuItem((HMENU)wParam, MENU_START_CAP, 
                (!gcap.fCapturing) ? MF_ENABLED : MF_GRAYED);
            // we can stop capture if it's currently capturing
            EnableMenuItem((HMENU)wParam, MENU_STOP_CAP, 
                (gcap.fCapturing) ? MF_ENABLED : MF_GRAYED);

            // do we want preview?
            CheckMenuItem((HMENU)wParam, MENU_PREVIEW, 
                (appData->fWantPreview) ? MF_CHECKED : MF_UNCHECKED);
            // can toggle preview if not capturing
            EnableMenuItem((HMENU)wParam, MENU_PREVIEW,
                (gcap.fCapturing && !gcap.fPreviewing) ? MF_GRAYED : MF_ENABLED);

			//Check the Show Stats Menu Item if stats are showing
            CheckMenuItem((HMENU)wParam, MENU_SHOWSTATS, 
                (appData->fWantCaptureStats) ? MF_CHECKED : MF_UNCHECKED);

			// Disable the Profile Save / Load while capturing
            EnableMenuItem((HMENU)wParam, MENU_SAVESETTINGS,
                !gcap.fCapturing ? MF_ENABLED : MF_GRAYED);
            EnableMenuItem((HMENU)wParam, MENU_SAVETOFILE,
                !gcap.fCapturing ? MF_ENABLED : MF_GRAYED);
            EnableMenuItem((HMENU)wParam, MENU_LOADFROMFILE,
                !gcap.fCapturing ? MF_ENABLED : MF_GRAYED);

			// Grey Show drpped frames while capturing
			EnableMenuItem((HMENU)wParam, MENU_SHOW_DROPPED_FRAMES, 
				gcap.fCapturing ? MF_GRAYED : MF_ENABLED);

            // can show settings if not capturing
            EnableMenuItem((HMENU)wParam, MENU_SETTINGS,
                gcap.fCapturing ? MF_GRAYED : MF_ENABLED);

            break;


        case WM_DESTROY:
            DbgTerminate();

            IMonRelease(gcap.pmVideo);
            IMonRelease(gcap.pmAudio);

			{
				for(int i = 0; i < NUMELMS(gcap.rgpmVideoMenu); i++)
				{
					IMonRelease(gcap.rgpmVideoMenu[i]);
				}
				for(i = 0; i < NUMELMS(gcap.rgpmAudioMenu); i++)
				{
					IMonRelease(gcap.rgpmAudioMenu[i]);
				}
            }

            CoUninitialize();
            PostQuitMessage(0);
            break;


        case WM_CLOSE:
			if(gcap.log) logAction(TEXT("WM_CLOSE OnClose() called"));
            OnClose();
			if(gcap.log) logAction(TEXT("WM_CLOSE - exiting"));
            break;

        case WM_ENDSESSION:
            if(wParam || (lParam & ENDSESSION_LOGOFF))
			{
				if(gcap.log) logAction(TEXT("WM_ENDSESSION OnClose() called"));
                OnClose();
            }
            break;

        case WM_ERASEBKGND:
            break;

        // Process Key Input
        case WM_KEYDOWN:
			DoKeyPressInput(wParam);
            return 0;

        case WM_PAINT:
            hdc = BeginPaint(hwnd,&ps);

            // nothing to do
            EndPaint(hwnd,&ps);
            break;

        case WM_TIMER:
            // update our status bar and capture stats dialog
            if(gcap.fCapturing)
                UpdateStatus(false);

            // is our time limit up?
            if(appData->fUseTimeLimit)
			{
				if((timeGetTime() - gcap.lCapStartTime) / 1000 >= appData->dwTimeLimit)
				{
					StopCapture();

					// Quit App if applicable
					if(appData->fQuitWhenDone)
					{
            			PostMessage(hwnd,WM_CLOSE,0,0L);
					}

                    if(appData->fWantPreview)
					{
//                         BuildPreviewGraph();
                         StartPreview();
                    }
                }
            }
			if(appData->fUseFreeSpace)
			{
				long freespace = GetFreeDiskSpaceInKB(appData->szCaptureFile) / 1024;
				if(freespace < appData->dwFreeSpaceLimit)
				{
					StopCapture();
                    if(appData->fWantPreview)
					{
//                        BuildPreviewGraph();
                         StartPreview();
                    }
				}
			}
            break;

        case WM_SIZE:
            // make the preview window fit inside our window, taking up
            // all of our client area except for the status window at the
            // bottom (gcap.toolBarHeight)

			// Resize Toolbar
			SendMessage(gcap.toolBarHWND, TB_AUTOSIZE, 0, 0);

            GetClientRect(ghwndApp, &rc);
            cxBorder = GetSystemMetrics(SM_CXBORDER);
            cyBorder = GetSystemMetrics(SM_CYBORDER);
            cy = 0;

			if(!appData->HideBars)
			{
				cy = statusGetHeight() + cyBorder;

				MoveWindow(ghwndStatus, -cxBorder, rc.bottom - cy,
					rc.right + (2 * cxBorder), cy + cyBorder, TRUE);
			}

            rc.bottom -= cy;

            if(gcap.pVW)
			{
				if(appData->HideBars)
				{
					gcap.pVW->SetWindowPosition(0, 0, rc.right, rc.bottom);
				}
				else
				{
					gcap.pVW->SetWindowPosition(0, gcap.toolBarHeight, rc.right, rc.bottom-gcap.toolBarHeight);
				}

			}

			//MsgBox(TEXT("WM_SIZE %d %d"), rc.right, rc.bottom);
            break;

		case WM_MOVE:
		{

			RECT rcW;
			GetWindowRect(hwnd, &rcW);
			int newX = rcW.left;
			int newY = rcW.top;

			if(newX < 0 || newX > 5000)
				newX = 0;

			if(newY < 0 || newY > 5000)
				newY = 0;

			appData->iApplicationWindowX = newX;
			appData->iApplicationWindowY = newY;

			// If we have docked the stats dialog move the stats dialog 
			// at the same time
			if(appData->CapStatsRelativePos)
			{
				int newXlocation = newX + appData->iCaptureStatsWindowX;
				int newYlocation = newY + appData->iCaptureStatsWindowY;
				if(newXlocation < 0 || newXlocation > 5000) newXlocation = 0;
				if(newYlocation < 0 || newYlocation > 5000) newYlocation = 0;

				SetWindowPos(
					hwndSTATUS, // handle to window
					HWND_TOP,  // placement-order handle
					newXlocation, // horizontal position
					newYlocation, // vertical position
					NULL, // width
					NULL, // height
					SWP_NOSIZE | SWP_NOACTIVATE // window-positioning options
					);
			}
			return 0;
		}

        case WM_FGNOTIFY:
            // uh-oh, something went wrong while capturing - the filtergraph
            // will send us events like EC_COMPLETE, EC_USERABORT and the one
            // we care about, EC_ERRORABORT. but only while we are capturing
            if(gcap.pME)
			{
				LONG event, l1, l2;
				BOOL bAbort = FALSE;
				while(gcap.pME->GetEvent(&event, (LONG_PTR *) &l1, (LONG_PTR *) &l2, 0) == S_OK)
				{
					gcap.pME->FreeEventParams(event, l1, l2);
					if(event == EC_ERRORABORT && gcap.fCapturing)
					{
						StopCapture();
						bAbort = TRUE;
						continue;
					}
				}
				if(bAbort)
				{
					if(appData->fWantPreview)
					{
//						BuildPreviewGraph();
						StartPreview();
					}
					ErrMsg(TEXT("ERROR during capture\nError code=%08x and %08x"), l1, l2);
				}
            }
            break;
    }

    return (LONG) DefWindowProc(hwnd,msg,wParam,lParam);
}


// Make a graph builder object we can use for capture graph building
//
BOOL MakeBuilder() 
{
	if(gcap.log) logAction(TEXT("MakeBuilder()"));

    // we have one already
    if(gcap.pBuilder)
        return TRUE;

    HRESULT hr = CoCreateInstance((REFCLSID)CLSID_CaptureGraphBuilder2,
        NULL, CLSCTX_INPROC, (REFIID)IID_ICaptureGraphBuilder2,
        (void **)&gcap.pBuilder);

    return (hr == NOERROR) ? TRUE : FALSE;
}


// Make a graph object we can use for capture graph building
//
BOOL MakeGraph() 
{
	if(gcap.log) logAction(TEXT("MakeGraph()"));

    // we have one already
    if(gcap.pFg)
        return TRUE;

    HRESULT hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC,
        IID_IGraphBuilder, (LPVOID *)&gcap.pFg);

    return (hr == NOERROR) ? TRUE : FALSE;
}


// make sure the preview window inside our window is as big as the
// dimensions of captured video, or some capture cards won't show a preview.
// (Also, it helps people tell what size video they're capturing)
// We will resize our app's window big enough so that once the status bar
// is positioned at the bottom there will be enough room for the preview
// window to be w x h
//
int gnRecurse = 0;


void ResizeWindow(int w, int h) 
{
	if(gcap.log) logAction(TEXT("ResizeWindow(int w, int h)"));

	if(appData->HideBars)
	{
		ShowWindow(ghwndStatus, SW_HIDE);
		ShowWindow(gcap.toolBarHWND, SW_HIDE);
	}
	else
	{
		ShowWindow(ghwndStatus, SW_SHOW);
		ShowWindow(gcap.toolBarHWND, SW_SHOW);
	}
	
    RECT rcW, rcC;
    int xExtra, yExtra;
    int cyBorder = GetSystemMetrics(SM_CYBORDER);

    gnRecurse++;

    GetWindowRect(ghwndApp, &rcW);
    GetClientRect(ghwndApp, &rcC);
    xExtra = rcW.right - rcW.left - rcC.right;

	if(appData->HideBars)
	{
		yExtra = rcW.bottom - rcW.top - rcC.bottom + cyBorder;
	}
	else
	{
		yExtra = rcW.bottom - rcW.top - rcC.bottom + cyBorder
			+ statusGetHeight() + gcap.toolBarHeight;
	}

    rcC.right = w;
    rcC.bottom = h;

	if(appData->AlwaysOnTop)
	{
		SetWindowPos(ghwndApp, HWND_TOPMOST, 0, 0, rcC.right + xExtra,
			rcC.bottom + yExtra, SWP_NOMOVE);
	}
	else
	{
		SetWindowPos(ghwndApp, HWND_BOTTOM, 0, 0, rcC.right + xExtra,
			rcC.bottom + yExtra, SWP_NOMOVE);
		BringWindowToTop(ghwndApp);
	}

    // we may need to recurse once.  But more than that means the window cannot
    // be made the size we want, trying will just stack fault.
    //
    if(gnRecurse == 1 && ((rcC.right + xExtra != rcW.right - rcW.left && w > GetSystemMetrics(SM_CXMIN)) ||
        (rcC.bottom + yExtra != rcW.bottom - rcW.top)))
        ResizeWindow(w,h);

    gnRecurse--;
}

void ResizeWindow()
{
	if(gcap.log) logAction(TEXT("ResizeWindow()"));

	HRESULT hr = E_FAIL;
    // Resize our window to be the same size that we're capturing

	//if(appData->UseSmartTee && gcap.SmartTeeFilter && gcap.pVCap)
	if(gcap.SmartTeeFilter)
	{
		IPin *pCapPin = NULL;
	
		hr =  gcap.SmartTeeFilter->FindPin(L"Capture", &pCapPin);
		if(hr != S_OK)
			ErrMsg(TEXT("Error getting Smart Tee Capture Pin for resize"));

		if(pCapPin)
		{
			RECT dim = GetVideoSize(pCapPin);
			//MsgBox(TEXT("%d %d %d %d"), dim.left, dim.right, dim.top, dim.bottom);
			if(dim.right > 1 && dim.bottom > 1)
			{
				ResizeWindow(dim.right, dim.bottom);
			}

			pCapPin->Release();
		}
	}

}

//Get the video stream size
RECT GetVideoSize(IPin *pCapPin)
{
	if(gcap.log) logAction(TEXT("GetVideoSize(IPin *pCapPin)"));

	HRESULT hr = E_FAIL;

	RECT dim;
	dim.bottom = 0;
	dim.top = 0;
	dim.left = 0;
	dim.right = 0;

	int width = 0;
	int height = 0;

	if(pCapPin)
	{
		AM_MEDIA_TYPE pmt01;
		
		hr = pCapPin->ConnectionMediaType(&pmt01);
		if(hr == NOERROR)
		{
			if(pmt01.formattype == FORMAT_VideoInfo)
			{
				width = HEADER(pmt01.pbFormat)->biWidth;
				height = abs(HEADER(pmt01.pbFormat)->biHeight);

				dim.right = width;
				dim.bottom = height;

				TCHAR statusTEXT[32];
				wsprintf(statusTEXT, TEXT("%d x %d"), dim.right, dim.bottom);
				statusUpdateStatus(ghwndStatus, statusTEXT, 2);	

				appData->ColourFormatName(pmt01.subtype, statusTEXT);
				statusUpdateStatus(ghwndStatus, statusTEXT, 3);	
			}
			FreeMediaType(pmt01);
		}
	}

	return dim;
}


// This should iterate through the filters that are in the 
// filterGraph and check to see if they not the
// video capture filter or the audio capture filter
// and then remove them.
void NukeAllUnconnectedFilters() 
{
	return; // disable this for now

	if(gcap.log) logAction(TEXT("NukeAllUnconnectedFilters()"));

	if(gcap.pFg == NULL)
		return;

	DbgLog((LOG_TRACE,1,TEXT("Nuking All Unconnected Filters")));

	HRESULT hr;
	HRESULT hr2;

	IBaseFilter *pVideo = NULL;
	IBaseFilter *pAudio = NULL;
	gcap.pFg->FindFilterByName(L"Video Capture Filter", &pVideo);
	gcap.pFg->FindFilterByName(L"Audio Capture Filter", &pAudio);

	IEnumFilters *ppEnum = NULL;
	ULONG u = 0;
	ULONG u2;
	IPin *pP = NULL;
	IPin *pT = NULL;
	IEnumPins *pins = NULL;
	BOOL removeFilter = TRUE;
	FILTER_INFO fi;

	hr = gcap.pFg->EnumFilters(&ppEnum);
	ppEnum->Reset();

	IBaseFilter *filterList[256];
//	memset(filterList, 0, (sizeof(filterList[0]) * 256));
	hr = ppEnum->Next(255, filterList, &u);

	if(u > 0)
	{
		for(int x = 0; x < u; x++)
		{
			if(filterList[x] == NULL) break;

			removeFilter = TRUE;

			filterList[x]->QueryFilterInfo(&fi);
			if(gcap.log) logAction(TEXT("Checking Filter %d of %d: %S"), x+1, u, fi.achName);

			if(fi.pGraph)
				fi.pGraph->Release();

			hr2 = filterList[x]->EnumPins(&pins);
			pins->Reset();

		    while(hr2 == NOERROR)
			{
				hr2 = pins->Next(1, &pP, &u2);
				if(hr2 == S_OK && pP)
				{
					pP->ConnectedTo(&pT);
					if(	pT ||
						(pVideo == filterList[x] && filterList[x] != NULL) || 
						(pAudio == filterList[x] && filterList[x] != NULL)
					)
					{	
						removeFilter = FALSE;
					}

					if(pT)
						pT->Release();
					pT = NULL;

					pP->Release();
					pP = NULL;
				}
			}

			pins->Release();

			if(removeFilter)
			{
				if(gcap.log) logAction(TEXT("Remove Filter : %S"), fi.achName);
				gcap.pFg->RemoveFilter(filterList[x]);
			}
			else
			{
				if(gcap.log) logAction(TEXT("Leaving Filter : %S"), fi.achName);
			}


			int refCount = 0;
/*
			if(filterList[x])
			{
				refCount = filterList[x]->Release();
				if(gcap.log) logAction(TEXT("Release Filter : %S - new refCount=%d"), fi.achName, refCount);
			}
*/
			filterList[x] = NULL;

		}
	}

    if(ppEnum)
        ppEnum->Release();

	if(pVideo) pVideo->Release();
	if(pAudio) pVideo->Release();
}

// Tear down everything downstream of a given filter
void NukeDownstream(IBaseFilter *pf) 
{
	FILTER_INFO fi;
//	pf->QueryFilterInfo(&fi);
//	if(gcap.log) logAction(TEXT("NukeDownstream(%S)"), fi.achName);
//	if(fi.pGraph)
//		fi.pGraph->Release();

    DbgLog((LOG_TRACE,1,TEXT("Nuking...")));

    IPin *pP, *pTo;
    ULONG u;
    IEnumPins *pins = NULL;
    PIN_INFO pininfo;
    HRESULT hr = pf->EnumPins(&pins);
    pins->Reset();

    while(hr == NOERROR)
	{
        hr = pins->Next(1, &pP, &u);
        if(hr == S_OK && pP)
		{
            pP->ConnectedTo(&pTo);
            if(pTo)
			{
                hr = pTo->QueryPinInfo(&pininfo);
                if(hr == NOERROR)
				{
					pininfo.pFilter->QueryFilterInfo(&fi);

                    if(pininfo.dir == PINDIR_INPUT)
					{
                        NukeDownstream(pininfo.pFilter);
                        gcap.pFg->Disconnect(pTo);
						gcap.pFg->Disconnect(pP);

						if(gcap.log)
							logAction(TEXT("NukeDownstream() Removing Filter : %S"), fi.achName);

                        gcap.pFg->RemoveFilter(pininfo.pFilter);
                    }

					int refCount = pininfo.pFilter->Release();
					if(gcap.log) logAction(TEXT("NukeDownstream() Release - %S : New RefCount=%d"), fi.achName, refCount);

					if(fi.pGraph)
						fi.pGraph->Release();
                }
                pTo->Release();
            }
            pP->Release();
        }
    }
    if(pins)
        pins->Release();
}


// Tear down everything downstream of the capture filters, so we can build
// a different capture graph.  Notice that we never destroy the capture filters
// and WDM filters upstream of them, because then all the capture settings
// we've set would be lost.
//
void TearDownGraph() 
{
	if(gcap.log) logAction(TEXT("TearDownGraph()"));

    if(gcap.pSink)
        gcap.pSink->Release();
    gcap.pSink = NULL;

    if(gcap.pConfigAviMux)
        gcap.pConfigAviMux->Release();
    gcap.pConfigAviMux = NULL;

    if(gcap.pVW)
	{
        // stop drawing in our window, or we may get wierd repaint effects
        gcap.pVW->put_Owner(NULL);
        gcap.pVW->put_Visible(OAFALSE);
        gcap.pVW->Release();
    }
    gcap.pVW = NULL;

    if(gcap.pME)
	{
		//gcap.pME->SetNotifyWindow(NULL, 0, 0);
        gcap.pME->Release();
		gcap.pME = NULL;
	}

    if(gcap.pDF)
        gcap.pDF->Release();
    gcap.pDF = NULL;

	// Releases all the filters I have created
	if(gcap.VideoRender)
		gcap.VideoRender->Release();
	gcap.VideoRender = NULL;

	if(gcap.SmartTeeFilter)
		gcap.SmartTeeFilter->Release();
	gcap.SmartTeeFilter = NULL;

	if(gcap.pFileWriter)
		gcap.pFileWriter->Release();
	gcap.pFileWriter = NULL;

	if(gcap.pAVIMux)
		gcap.pAVIMux->Release();
	gcap.pAVIMux = NULL;

	//Release the Flow Meter Filters and Interfaces
//	if(gcap.FlowMeterVideo)
//		gcap.FlowMeterVideo->Release();
//	gcap.FlowMeterVideo = NULL;
//	if(gcap.FlowMeterAudio)
//		gcap.FlowMeterAudio->Release();
//	gcap.FlowMeterAudio = NULL;

    if(gcap.FlowMeterVideoInterface)
        gcap.FlowMeterVideoInterface->Release();
    gcap.FlowMeterVideoInterface = NULL;

    if(gcap.FlowMeterAudioInterface)
        gcap.FlowMeterAudioInterface->Release();
    gcap.FlowMeterAudioInterface = NULL;

	// Release the Audio Resample Interface
	if(gcap.AudioResampleInterface)
		gcap.AudioResampleInterface->Release();
	gcap.AudioResampleInterface = NULL;

	// Release the Colour Histogram Filter
	if(gcap.colourHistogram)
		gcap.colourHistogram->Release();
	gcap.colourHistogram = NULL;

	// Release the Video Render filter
	if(gcap.videoRender)
		gcap.videoRender->Release();
	gcap.videoRender = NULL;

	if(gcap.audioScope)
	{
		int ref = gcap.audioScope->Release();
		if(gcap.log) logAction(TEXT("TearDownGraph() - freeing scope %d"), ref);
		gcap.audioScope = NULL;
	}

    // destroy the graph downstream of our capture filters
    if(gcap.pVCap)
        NukeDownstream(gcap.pVCap);
    if(gcap.pACap)
        NukeDownstream(gcap.pACap);

	//if(gcap.pVCap && gcap.pACap)
		NukeAllUnconnectedFilters();

    // potential debug output - what the graph looks like
    if (gcap.pFg) DumpGraph(gcap.pFg, 0);

#ifdef REGISTER_FILTERGRAPH
    // Remove filter graph from the running object table   
    if (g_dwGraphRegister)
    {
        RemoveGraphFromRot(g_dwGraphRegister);
        g_dwGraphRegister = 0;
    }
#endif

    gcap.fCaptureGraphBuilt = FALSE;
    gcap.fPreviewGraphBuilt = FALSE;
}


// create the capture filters of the graph.  We need to keep them loaded from
// the beginning, so we can set parameters on them and have them remembered
//
BOOL InitCapFilters()
{
	if(gcap.log) logAction(TEXT("InitCapFilters()"));

    HRESULT hr=S_OK;
    BOOL f;
	int result = 0;

    f = MakeBuilder();
    if(!f)
	{
        ErrMsg(TEXT("Could Not Create Graph Builder"));
        return FALSE;
    }

	IAMBufferNegotiation *audioBuffer = NULL;
	IAMBufferNegotiation *videoBuffer = NULL;

    //
    // First, we need a Video Capture filter, and some interfaces
    //

    gcap.pVCap = NULL;
    if(gcap.pmVideo != 0)
	{
		IPropertyBag *pBag;
        gcap.wachFriendlyName[0] = 0;
        hr = gcap.pmVideo->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pBag);
        if(SUCCEEDED(hr))
		{
            VARIANT var;
            var.vt = VT_BSTR;
            hr = pBag->Read(L"FriendlyName", &var, NULL);
            if(hr == NOERROR)
			{
                lstrcpyW(gcap.wachFriendlyName, var.bstrVal);
                SysFreeString(var.bstrVal);
            }
            pBag->Release();
        }
        hr = gcap.pmVideo->BindToObject(0, 0, IID_IBaseFilter, (void**)&gcap.pVCap);
    }

    if(gcap.pVCap == NULL)
	{
        ErrMsg(TEXT("Could Not Create Video Capture Filter"));
        goto InitCapFiltersFail;
    }

//TEST
/*
{
	CFilterGraphManager *capGraph = new CFilterGraphManager();
	capGraph->initCaptureBuilder();

	capGraph->setVideoCaptureFilter(gcap.pVCap);

	hr = gcap.pmAudio->BindToObject(0, 0, IID_IBaseFilter, (void**)&gcap.pACap);
	capGraph->setAudioCaptureFilter(gcap.pACap);

	capGraph->setFilterList(appData->filterList);

	capGraph->buildCaptureGraph(TRUE);

	return true;
}
*/
//TEST

//TEST
/*
{
	HMODULE captureEngine = LoadLibrary("CaptureEngine.dll");
	if(captureEngine == NULL)
		ErrMsg(TEXT("Capture Engine DLL not Found"));
	else
	{
		FARPROC testFunctions =
			(FARPROC) GetProcAddress(captureEngine, "fnCaptureEngine");

		if(testFunctions == NULL)
			ErrMsg(TEXT("Function entry point not found"));
		else
		{
			int testNum = testFunctions();
			MsgBox(TEXT("%d"), testNum);

		}
	}

	if(captureEngine !=  NULL)
	{
		FreeLibrary(captureEngine);
		captureEngine = NULL;
	}


}
*/
//TEST


    //
    // make a filtergraph, give it to the graph builder and put the video
    // capture filter in the graph
    //

    f = MakeGraph();
    if(!f)
	{
        ErrMsg(TEXT("Could Not Create Filter Graph"));
        goto InitCapFiltersFail;
    }
    hr = gcap.pBuilder->SetFiltergraph(gcap.pFg);
    if(hr != NOERROR)
	{
        ErrMsg(TEXT("Could Not Set Filter Graph"));
        goto InitCapFiltersFail;
    }
    hr = gcap.pFg->AddFilter(gcap.pVCap, L"Video Capture Filter");
    if(hr != NOERROR)
	{
        ErrMsg(TEXT("Could Not Add Video Capture Filter to the Filter Graph"));
        goto InitCapFiltersFail;
    }

    // Calling FindInterface below will result in building the upstream
    // section of the capture graph (any WDM TVTuners or Crossbars we might
    // need).

    // we use this interface to get the name of the driver
    // Don't worry if it doesn't work:  This interface may not be available
    // until the pin is connected, or it may not be available at all.
    // (eg: interface may not be available for some DV capture)


//    hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
//        &MEDIATYPE_Interleaved,
//        gcap.pVCap, IID_IAMVideoCompression, (void **)&gcap.pVC);

//    if(hr != S_OK)
//	{
//        hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
//            &MEDIATYPE_Video,
//           gcap.pVCap, IID_IAMVideoCompression, (void **)&gcap.pVC);
//    }

    // !!! What if this interface isn't supported?
    // we use this interface to set the frame rate and get the capture size
//    hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
//        &MEDIATYPE_Interleaved,
//        gcap.pVCap, IID_IAMStreamConfig, (void **)&gcap.pVSC);

//    if(hr != NOERROR)
//	{



//        hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
//            &MEDIATYPE_Video, gcap.pVCap,
//            IID_IAMStreamConfig, (void **)&gcap.pVSC);

//        if(hr != NOERROR)
//		{
//            // this means we can't set frame rate (non-DV only)
//            ErrMsg(TEXT("Could Not get the IAMStreamConfig interface from the Video Capture Filter"));
//       }




//    }

    gcap.fCapAudioIsRelevant = TRUE;
/*
    AM_MEDIA_TYPE *pmt;

    // default capture format
	// this is where we set the custom capture format
	//
    if(gcap.pVSC && gcap.pVSC->GetFormat(&pmt) == S_OK)
	{
        // DV capture does not use a VIDEOINFOHEADER
        if(pmt->formattype == FORMAT_VideoInfo)
		{
			if(gcap.pVSC && appData->fUseCustomFormat)
			{
				VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *)pmt->pbFormat;
				pvi->AvgTimePerFrame = (LONGLONG)(((double)10000000) / appData->FrameRate);
				pvi->bmiHeader.biWidth = appData->CustomWidth;
				pvi->bmiHeader.biHeight = appData->CustomHeight;

				// Set the video sub type to the last used.
				pmt->subtype = appData->ColourFormat;

				hr = gcap.pVSC->SetFormat(pmt);
			}
        }
        if(pmt->majortype != MEDIATYPE_Video)
		{
            // This capture filter captures something other that pure video.
            // Maybe it's DV or something?  Anyway, chances are we shouldn't
            // allow capturing audio separately, since our video capture    
            // filter may have audio combined in it already!
            gcap.fCapAudioIsRelevant = FALSE;
            appData->fCapAudio = FALSE;
        }
        DeleteMediaType(pmt);
    }
*/
	result = SetVideoFormat(0);
	if(result == -4)
	{
		gcap.fCapAudioIsRelevant = FALSE;
		appData->fCapAudio = FALSE;
	}

    // we use this interface to bring up the 3 dialogs
    // NOTE:  Only the VfW capture filter supports this.  This app only brings
    // up dialogs for legacy VfW capture drivers, since only those have dialogs
    hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
        &MEDIATYPE_Video, gcap.pVCap,
        IID_IAMVfwCaptureDialogs, (void **)&gcap.pDlg);

    // Use the crossbar class to help us sort out all the possible video inputs
    // The class needs to be given the capture filters ANALOGVIDEO input pin
    {
        IPin        *pP = 0;
        IEnumPins   *pins;
        ULONG        n;
        PIN_INFO     pinInfo;
        BOOL         Found = FALSE;
        IKsPropertySet *pKs;
        GUID guid;
        DWORD dw;
        BOOL fMatch = FALSE;

        gcap.pCrossbar = NULL;

        if(SUCCEEDED(gcap.pVCap->EnumPins(&pins)))
		{            
            while(!Found && (S_OK == pins->Next(1, &pP, &n)))
			{
                if(S_OK == pP->QueryPinInfo(&pinInfo))
				{
                    if(pinInfo.dir == PINDIR_INPUT)
					{
                        // is this pin an ANALOGVIDEOIN input pin?
                        if(pP->QueryInterface(IID_IKsPropertySet,
                            (void **)&pKs) == S_OK)
						{
                            if(pKs->Get(AMPROPSETID_Pin,
                                AMPROPERTY_PIN_CATEGORY, NULL, 0,
                                &guid, sizeof(GUID), &dw) == S_OK)
							{
                                if(guid == PIN_CATEGORY_ANALOGVIDEOIN)
                                    fMatch = TRUE;
                            }
                            pKs->Release();
                        }

                        if(fMatch)
						{
                            gcap.pCrossbar = new CCrossbar(pP);
                            hr = gcap.pCrossbar->GetInputCount
                                (&gcap.NumberOfVideoInputs);
                            Found = TRUE;
                        }
                    }
                    pinInfo.pFilter->Release();
                }
                pP->Release();
            }
            pins->Release();
        }

    }

	// there's no point making an audio capture filter
    if(gcap.fCapAudioIsRelevant == FALSE)
        goto SkipAudio;

    // create the audio capture filter, even if we are not capturing audio right
    // now, so we have all the filters around all the time.

    //
    // We want an audio capture filter and some interfaces
    //

    if(gcap.pmAudio == 0)
	{
        // there are no audio capture devices. We'll only allow video capture
        appData->fCapAudio = FALSE;
		MsgBox(TEXT("No Audio Device Found. Skiping Audio Setup"));
        goto SkipAudio;
    }
    gcap.pACap = NULL;

    hr = gcap.pmAudio->BindToObject(0, 0, IID_IBaseFilter, (void**)&gcap.pACap);

    if(gcap.pACap == NULL)
	{
        // there are no audio capture devices. We'll only allow video capture
        appData->fCapAudio = FALSE;
        ErrMsg(TEXT("Cannot create audio capture filter"));
        goto SkipAudio;
    }

	// Set the audio input and mixer level
	{
		if(appData->audioInputIndex > -1)
		{
			CAudioMixer *mix = new CAudioMixer(gcap.pACap);

			mix->setInput(appData->audioInputIndex);
			mix->setLevel(appData->audioInputLevel, appData->audioInputIndex);

			//int inputIndex = mix->getInput();
			//double level = mix->getLevel(appData->audioInputIndex);
			//MsgBox(TEXT("%d %d"), inputIndex, (int)(level*1000));

			delete mix;
		}
	}

    //
    // put the audio capture filter in the graph
    //

    // We'll need this in the graph to get audio property pages
    hr = gcap.pFg->AddFilter(gcap.pACap, L"Audio Capture Filter");
    if(hr != NOERROR)
	{
        ErrMsg(TEXT("Error %x: Cannot add audcap to filtergraph"), hr);
        goto InitCapFiltersFail;
    }

    // Calling FindInterface below will result in building the upstream
    // section of the capture graph (any WDM TVAudio's or Crossbars we might
    // need).

    // !!! What if this interface isn't supported?
    // we use this interface to set the captured wave format
//    hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Audio,
//        gcap.pACap, IID_IAMStreamConfig, (void **)&gcap.pASC);
//    if(hr != NOERROR)
//	{
//        ErrMsg(TEXT("Cannot find ACapture:IAMStreamConfig"));
//    }

	// Get the IAMBufferNegotiation interface and set the buffer options
	// This is where you set the audio buffer size etc
/*
    hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Audio,
        gcap.pACap, IID_IAMBufferNegotiation, (void **)&audioBuffer);
	if(audioBuffer)
	{
		ALLOCATOR_PROPERTIES prop;
		prop.cbBuffer = 17640;
		//prop.cbPrefix = 0;
		//prop.cBuffers = 10;
		audioBuffer->SuggestAllocatorProperties(&prop);
		audioBuffer->Release();
		audioBuffer = NULL;
	}
	else
	{
		ErrMsg(TEXT("Cannot get IAMBufferNegotiation interface from sound driver"));
	}
*/
SkipAudio:

	// Now Set the current Tuner Channel if needed
	changeChannel(appData->TunerChannel);

	// Set the audio format if available
	if(gcap.aboutHWND != NULL)
		SetDlgItemText(gcap.aboutHWND, IDC_STATUS, "Setting Audio Format");
	SetAudioFormat();

	// Set the brighness, hue, contrast and saturation
	// of the video amp
	if(gcap.aboutHWND != NULL)
		SetDlgItemText(gcap.aboutHWND, IDC_STATUS, "Setting Video Amp");
	setVideoAmpSettings();

	// If we have a crossbar set the video input
	// that we want to use initially
	if(gcap.aboutHWND != NULL)
		SetDlgItemText(gcap.aboutHWND, IDC_STATUS, "Setting CrossBar");
	if(gcap.pCrossbar)
		gcap.pCrossbar->SetInputIndex(appData->iVideoInputIndex);

    return TRUE;

InitCapFiltersFail:
    FreeCapFilters();
    return FALSE;
}


// all done with the capture filters and the graph builder
//
void FreeCapFilters() 
{
	if(gcap.log) logAction(TEXT("FreeCapFilters()"));

    if(gcap.pFg)
	{
        int ref = gcap.pFg->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing FilterGraph %d"), ref);
		gcap.pFg = NULL;
	}

	if(gcap.pBuilder)
    {
		int ref = gcap.pBuilder->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing Graph Builder %d"), ref);
		gcap.pBuilder = NULL;
	}

    if(gcap.pVCap)
	{
        int ref = gcap.pVCap->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing Video Cap Filter %d"), ref);
		gcap.pVCap = NULL;
	}

	if(gcap.pACap)
	{
		int ref = gcap.pACap->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing Audio Cap Filter %d"), ref);
		gcap.pACap = NULL;
	}

//	if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing Audio Stream Control"));
//    if(gcap.pASC)
//        gcap.pASC->Release();
//    gcap.pASC = NULL;

//	if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing Video Stream Control"));
//    if(gcap.pVSC)
//        gcap.pVSC->Release();
//    gcap.pVSC = NULL;
/*
    if(gcap.pVC)
	{
        int ref = gcap.pVC->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing Video C %d"), ref);
		gcap.pVC = NULL;
	}
*/
	
    if(gcap.pDlg)
	{
        int ref = gcap.pDlg->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing Video Cap Dialogs %d"), ref);
		gcap.pDlg = NULL;
	}

	
	if(gcap.SmartTeeFilter)
	{
		int ref = gcap.SmartTeeFilter->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing smart tee %d"), ref);
		gcap.SmartTeeFilter = NULL;
	}

//    if(gcap.pVComp)
//        gcap.pVComp->Release();
//    gcap.pVComp = NULL;

	
	if(gcap.pFileWriter)
	{
		int ref = gcap.pFileWriter->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing file writer %d"), ref);
		gcap.pFileWriter = NULL;
	}

	
	if(gcap.pAVIMux)
	{
		int ref = gcap.pAVIMux->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - freeing avi mux %d"), ref);
		gcap.pAVIMux = NULL;
	}

	if(gcap.VideoRender)
	{
		int ref = gcap.VideoRender->Release();
		if(gcap.log) logAction(TEXT("FreeCapFilters() - VideoRender %d"), ref);
		gcap.VideoRender = NULL;
	}

    if(gcap.pCrossbar)
	{
		if(gcap.log) logAction(TEXT("FreeCapFilters() - Deleting Crossbar Object"));
		delete gcap.pCrossbar;
        gcap.pCrossbar = NULL;
    }
}


// build the capture graph
//
BOOL BuildCaptureGraph() 
{
	if(gcap.log) logAction(TEXT("BuildCaptureGraph()"));

    USES_CONVERSION;
    int cy, cyBorder;
    HRESULT hr;
//    BOOL f;
//    AM_MEDIA_TYPE *pmt;
	IAMGraphStreams *streamsControl = NULL;
	IFileSinkFilter2 *pFileWriterConfig = NULL;

	// Reset the file segmant count
	gcap.capFileCount = 0;

    // we have one already
    if(gcap.fCaptureGraphBuilt)
        return TRUE;

    // No rebuilding while we're running
    if(gcap.fCapturing || gcap.fPreviewing)
        return FALSE;

    // We don't have the necessary capture filters
    if(gcap.pVCap == NULL)
        return FALSE;
    if(gcap.pACap == NULL && appData->fCapAudio)
        return FALSE;

    // we already have another graph built... tear down the old one
    if(gcap.fPreviewGraphBuilt)
        TearDownGraph();

	// Set the capture Video Format
	if(SetVideoFormat(0) < 0)
		return FALSE;

    //
    // We need a rendering section that will write the capture file out in AVI
    // file format. Builds this from scratch so we have access to the FileSinkFilter2
	// interface for file overwite etc
    //
	
	// Create the File Writer filter and add it
    CoCreateInstance(CLSID_FileWriter, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&gcap.pFileWriter);
	hr = gcap.pFg->AddFilter(gcap.pFileWriter, L"File Writer");
    if(hr != S_OK)
	{
        ErrMsg(TEXT("Can not add the File Writer Filter!"));
		goto SetupCaptureFail;
	}

	// Create the AVI mux filter and add it
    CoCreateInstance(CLSID_AviDest, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&gcap.pAVIMux);
	hr = gcap.pFg->AddFilter(gcap.pAVIMux, L"AVI Mux");
    if(hr != S_OK)
	{
        ErrMsg(TEXT("Can not add the AVI Mux Filter!"));
		goto SetupCaptureFail;
	}

	// Render the AVI Mux to the File Writer
	hr = gcap.pBuilder->RenderStream(NULL, NULL,
		gcap.pAVIMux, NULL, gcap.pFileWriter);
	if(hr != NOERROR)
	{
		ErrMsg(TEXT("Could not render filter\nAVI Mux -> File Writer"));
		goto SetupCaptureFail;
	}

	// Get the FileWriter sink interface to set the overwite option
	hr = gcap.pFileWriter->QueryInterface(IID_IFileSinkFilter2, (void **)&pFileWriterConfig);
	if(!pFileWriterConfig)
	{
		ErrMsg(TEXT("Could not get the FileSinkFilter Interface."));
		goto SetupCaptureFail;
	}

	// Append the date/time stamp to the file name if required
	if(appData->appendStampToName)
	{
		// This where the time/date stamp is inserted into the file name

		// get the time stamp from the system time
		struct tm *newtime;
		time_t long_time;
		time(&long_time);
		newtime = localtime(&long_time);
		TCHAR fileName2[256];
		TCHAR fileName[256];

		// copy accross the file name minus the extension
		int nameLength = 0;
		for(int x=0; x < 256; x++)
		{
			if(appData->szCaptureFile[x] == 0)
			{
				nameLength = x;
				break;
			}
		}
		strncpy(fileName2, appData->szCaptureFile, nameLength-4);
		fileName2[nameLength-4] = 0;

		// Rebuild the file name with the date/time stamp in it
		wsprintf(fileName, "%s (%.4d-%.2d-%.2d at %.2d-%.2d-%.2d).avi",
			fileName2,
			newtime->tm_year+1900,
			newtime->tm_mon+1,
			newtime->tm_mday,
			newtime->tm_hour,
			newtime->tm_min,
			newtime->tm_sec);
	
		// Set the new file name
		hr = pFileWriterConfig->SetFileName(T2W(fileName), NULL);

		wsprintf(gcap.currentFileName, "%s.txt", fileName);
	}
	else
	{
		hr = pFileWriterConfig->SetFileName(T2W(appData->szCaptureFile), NULL);
		wsprintf(gcap.currentFileName, "%s.txt", appData->szCaptureFile);
	}

	if(hr != NOERROR)
	{
		ErrMsg(TEXT("Cannot set output file name"));
		goto SetupCaptureFail;
	}

	// Set the mode to AM_FILE_OVERWRITE
	hr = pFileWriterConfig->SetMode(AM_FILE_OVERWRITE);
	pFileWriterConfig->Release();
	pFileWriterConfig = NULL;

    // Now tell the AVIMUX to write AVI files that old apps can read properly.
    // If we don't, most apps won't be able to tell where the keyframes are,
    // slowing down editing considerably
    // Doing this will cause one seek (over the area the index will go) when
    // you capture past 1 Gig, but that's no big deal.
    // NOTE: This is on by default, so it's not necessary to turn it on

    // Also, set the proper MASTER STREAM
    hr = gcap.pAVIMux->QueryInterface(IID_IConfigAviMux,
        (void **)&gcap.pConfigAviMux);

	if(!gcap.pConfigAviMux)
	{
		ErrMsg(TEXT("Could not get AVI Mux Config Interface"));
		goto SetupCaptureFail;
	}

    if(hr == NOERROR && gcap.pConfigAviMux)
	{
        gcap.pConfigAviMux->SetOutputCompatibilityIndex(appData->compatAVIindex);
        if(appData->fCapAudio)
		{
            hr = gcap.pConfigAviMux->SetMasterStream(appData->iMasterStream);
            if(hr != NOERROR)
                ErrMsg(TEXT("SetMasterStream failed!"));
        }
    }

	// add the video compressor to the graph
	if(appData->CompressVideo)
	{
		hr = gcap.pFg->AddFilter(appData->VideoCodec->getCurrentCodec(), L"Video Compressor");
		if(hr != NOERROR)
		{
			ErrMsg(TEXT("Could not add Video Compressor"));
			goto SetupCaptureFail;
		}
	}

    // Create and add the Smart Tee filter
	if(!gcap.SmartTeeFilter)
	{
		AddSmartTeeToGraph();
	}

    // Create and add the Video Flow Meter
    CoCreateInstance(CLSID_FlowMeter, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&gcap.FlowMeterVideo);
	hr = gcap.pFg->AddFilter(gcap.FlowMeterVideo, L"Video Flow Meter");
    if(hr != S_OK)
	{
        ErrMsg(TEXT("Error adding the Video Flow Meter\nMake sure you have installed the filters in the filter directory!"));
		goto SetupCaptureFail;
	}

	// Add Compression if needed
	if(appData->CompressVideo)
	{
		// Render the Video Compression Codec
		hr = gcap.pBuilder->RenderStream(NULL, NULL, gcap.SmartTeeFilter,
			appData->VideoCodec->getCurrentCodec(), gcap.FlowMeterVideo);
		if(hr != NOERROR)
		{
			ErrMsg(TEXT("Could not render filter\nSmart Tee -> Compression Codec -> Flow Meter"));
			goto SetupCaptureFail;
		}

		// Render the Flow meter to the mux
		hr = gcap.pBuilder->RenderStream(NULL, NULL,
			gcap.FlowMeterVideo, NULL, gcap.pAVIMux);
		if(hr != NOERROR)
		{
			ErrMsg(TEXT("Could not render filter\nFlow Meter -> AVI Mux"));
			goto SetupCaptureFail;
		}
    }
	else
	{
		// Render the Smart tee to the Mux through the Flow Meter
		hr = gcap.pBuilder->RenderStream(NULL, NULL,
			gcap.SmartTeeFilter, gcap.FlowMeterVideo, gcap.pAVIMux);
		if(hr != NOERROR)
		{
			ErrMsg(TEXT("Could not render filter\nSmart Tee -> Flow Meter -> AVI Mux"));
			goto SetupCaptureFail;
		}
    }

	// If we want preview then set that up now
    if(appData->fWantPreview)
	{
		// if we want to preview the Smart Tee or 
		// directly from the capture driver filter preview pin
		if(appData->UseSmartTee)
		{
			hr = gcap.pBuilder->RenderStream(
				NULL, NULL, gcap.SmartTeeFilter, NULL, NULL);
			if(hr != S_OK)
				ErrMsg(TEXT("Error rendering the Smart Tee filter preview pin!"));
			gcap.fPreviewGraphBuilt = TRUE;
		}
		else
		{
			hr = gcap.pBuilder->RenderStream(
				&PIN_CATEGORY_PREVIEW, NULL, gcap.pVCap, NULL, NULL);
			if(hr != S_OK)
				ErrMsg(TEXT("Error rendering the capture filter preview pin!\nTry using the \"Smart Tee\" option for previewing"));
			gcap.fPreviewGraphBuilt = TRUE;
		}
	}

    //
    // Render the audio capture pin?
    //

    if(appData->fCapAudio)
	{
		// Create and add the Audio Flow Meter
		CoCreateInstance(CLSID_FlowMeter, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&gcap.FlowMeterAudio);
		hr = gcap.pFg->AddFilter(gcap.FlowMeterAudio, L"Audio Flow Meter");
		if(hr != S_OK)
		{
			ErrMsg(TEXT("Error adding the Audio Flow Meter\nMake sure you have installed the filters in the filter directory!"));
			goto SetupCaptureFail;
		}


		// add the audio compressor to the graph
		IAMStreamConfig *caps = NULL;
		AM_MEDIA_TYPE *audio_pmt = NULL;
		if(appData->CompressAudio)
		{
			hr = gcap.pFg->AddFilter(appData->AudioCodec->getCurrentCodec(), L"Audio Compressor");
			if(hr != NOERROR)
			{
				ErrMsg(TEXT("Could not add Audio Compressor"));
				goto SetupCaptureFail;
			}

			// get the stream config interface for our audio codec
			// get the config settings at the index requested
			gcap.pBuilder->FindInterface(NULL,
				&MEDIATYPE_Audio, appData->AudioCodec->getCurrentCodec(), 
				IID_IAMStreamConfig, (void **)&caps);
			
			if(caps)
			{
				int number = 0;
				int sizeOfStruc = 0;
				caps->GetNumberOfCapabilities(&number, &sizeOfStruc);
				
				unsigned char *pSCC = new unsigned char[sizeOfStruc];
				
				if(appData->Audio_Capability_Index > -1)
				{
					caps->GetStreamCaps(appData->Audio_Capability_Index, &audio_pmt, pSCC);
					WAVEFORMATEX *wf = (WAVEFORMATEX *)audio_pmt->pbFormat;
				}
			}

		}

		if(appData->useAudioResample)
		{
			// Create and add the Audio Resample Filter
			CoCreateInstance(CLSID_AudioResample, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&gcap.AudioResampleFilter);
			hr = gcap.pFg->AddFilter(gcap.AudioResampleFilter, L"Audio Resample Filter");
			if(hr != S_OK)
			{
				ErrMsg(TEXT("Error adding the Audio Resample Filter\nMake sure you have installed the filters in the filter directory!"));
				goto SetupCaptureFail;
			}

			if(appData->CompressAudio)
			{
				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.pACap, gcap.FlowMeterAudio, gcap.AudioResampleFilter);

				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.AudioResampleFilter, appData->AudioCodec->getCurrentCodec(), gcap.pAVIMux);
			}
			else
			{
				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.pACap, gcap.FlowMeterAudio, gcap.AudioResampleFilter);

				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.AudioResampleFilter, NULL, gcap.pAVIMux);
			}
		}
		// Do not add the audio resample filter, just connect 
		// the audio source filter to the mux through the 
		// audio flow meter and compressor is needed
		else
		{
			if(appData->CompressAudio)
			{
				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.pACap, NULL, gcap.FlowMeterAudio);

				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.FlowMeterAudio, appData->AudioCodec->getCurrentCodec(), gcap.pAVIMux);
			}
			else
			{
				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.pACap, gcap.FlowMeterAudio, gcap.pAVIMux);
			}
		}

        if(hr != NOERROR)
		{
            ErrMsg(TEXT("Cannot render audio capture stream"));
            goto SetupCaptureFail;
        }


		// Now that the audio codec is connected
		// set the selected format
		if(appData->CompressAudio)
		{
			if(caps && audio_pmt)
			{
				HRESULT audioHR = caps->SetFormat(audio_pmt);
				DeleteMediaType(audio_pmt);
				caps->Release();

				if(audioHR == VFW_E_INVALIDMEDIATYPE)
					MsgBox(TEXT("There was an error settings the Audio Compression settings\n The defaults were used instead."));

				if(audioHR == E_OUTOFMEMORY)
					MsgBox(TEXT("E_OUTOFMEMORY"));
				if(audioHR == E_POINTER)
					MsgBox(TEXT("E_POINTER"));
				if(audioHR == VFW_E_NOT_CONNECTED)
					MsgBox(TEXT("VFW_E_NOT_CONNECTED"));
				if(audioHR == VFW_E_NOT_STOPPED)
					MsgBox(TEXT("VFW_E_NOT_STOPPED"));
				if(audioHR == VFW_E_WRONG_STATE)
					MsgBox(TEXT("VFW_E_WRONG_STATE"));
			}
			else
			{
				ErrMsg(TEXT("There was an error settings the Audio Compression settings\n The defaults were used instead."));
			}
		}


		// This is where we set up the stream offset options
		// there are two option, sync using stream offsets or
		// a manual offset

		// Set up the Audio Sync options

		if(appData->SyncUsingStreamOffset == TRUE)
		{
			// Set the auto mode, this mode should be able
			// to work out what the stream offsets are and then 
			// align them to match, this does not always work
			// for all drivers

			if(appData->streamOffsetType == 0)
			{
				// Set the SyncUsingStreamOffset option
				gcap.pFg->QueryInterface(IID_IAMGraphStreams, (void**)&streamsControl);
				if(streamsControl)
				{
					streamsControl->SyncUsingStreamOffset(true);
					streamsControl->Release();
				}
				streamsControl = NULL;
			}

			// This is the manual way of aligning the streams offset
			// this will delay the audio stream by a mS amount.

			else if(appData->streamOffsetType == 1)
			{
				//  Set the delay on the audio stream
				IAMPushSource *pushSourceAudio = NULL;
				IPin *pP;
				ULONG u;
				IEnumPins *pins = NULL;
				hr = gcap.pACap->EnumPins(&pins);
				pins->Reset();
				while(hr == NOERROR)
				{
					hr = pins->Next(1, &pP, &u);
					if(hr == S_OK && pP)
					{
						pP->QueryInterface(IID_IAMPushSource, (void**)&pushSourceAudio);
						if(pushSourceAudio)
						{
							pushSourceAudio->SetStreamOffset(appData->AudioDelay * 10000);
							pushSourceAudio->Release();
						}
						pP->Release();
					}
				}
				if(pins)
					pins->Release();
			}
		}

    }

    // Get the preview window to be a child of our app's window
    // This will find the IVideoWindow interface on the renderer.  It is 
    // important to ask the filtergraph for this interface... do NOT use
    // ICaptureGraphBuilder2::FindInterface, because the filtergraph needs to
    // know we own the window so it can give us display changed messages, etc.

    if(appData->fWantPreview)
	{
        hr = gcap.pFg->QueryInterface(IID_IVideoWindow, (void **)&gcap.pVW);
        if(hr != NOERROR && appData->fWantPreview)
		{
            ErrMsg(TEXT("This graph cannot preview"));
        }
        else if(hr == NOERROR)
		{
            RECT rc;
            gcap.pVW->put_Owner((OAHWND)ghwndApp);    // We own the window now
            gcap.pVW->put_WindowStyle(WS_CHILD);    // you are now a child
            // give the preview window all our space but where the status bar is
            GetClientRect(ghwndApp, &rc);
            cyBorder = GetSystemMetrics(SM_CYBORDER);

			if(appData->HideBars)
			{
				gcap.pVW->SetWindowPosition(0, 0, rc.right, rc.bottom); // be this big
			}
			else
			{
				cy = statusGetHeight() + cyBorder;
				rc.bottom -= cy;
				gcap.pVW->SetWindowPosition(0, gcap.toolBarHeight, rc.right, rc.bottom-gcap.toolBarHeight);
			}

            gcap.pVW->put_Visible(OATRUE);
        }
    }

    // now ask the filtergraph to tell us when something is completed or aborted
    // (EC_COMPLETE, EC_USERABORT, EC_ERRORABORT).  This is how we will find out
    // if the disk gets full while capturing
    hr = gcap.pFg->QueryInterface(IID_IMediaEventEx, (void **)&gcap.pME);
    if(hr == NOERROR)
	{
        gcap.pME->SetNotifyWindow((OAHWND)ghwndApp, WM_FGNOTIFY, 0);
	}

    // Add our graph to the running object table, which will allow
    // the GraphEdit application to "spy" on our graph
#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(gcap.pFg, &g_dwGraphRegister);
    if (FAILED(hr))
    {
        ErrMsg(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr);
        g_dwGraphRegister = 0;
    }
#endif

	changeChannel(appData->TunerChannel);

    // All done.
    gcap.fCaptureGraphBuilt = TRUE;

	if(gcap.log) logAction(TEXT("BuildCaptureGraph() - exit"));

    return TRUE;

SetupCaptureFail:

    TearDownGraph();
	FreeCapFilters();
	InitCapFilters();

	gcap.tunerSet = false;
	changeChannel(appData->TunerChannel);
				
	if(gcap.pCrossbar)
		gcap.pCrossbar->SetInputIndex(appData->iVideoInputIndex);
				
	// Set the video amp colour settings
	setVideoAmpSettings();

	if(gcap.log) logAction(TEXT("BuildCaptureGraph() - Return with fail"));

    return FALSE;
}

// This adds the smart tee to the graph and if
// enabled adds any Pre filtering to the filtert chain

BOOL AddSmartTeeToGraph()
{
	if(gcap.log) logAction(TEXT("AddSmartTeeToGraph()"));

	HRESULT hr;

	if(appData->UsePreFilters && appData->filterList->getFilterPointer(0))
	{
		WCHAR filterName[256];
		TCHAR filterAddNumber[256];
		filterName[0] = 0;
		
		int filterNumber = 0;
		while(appData->filterList->getFilterPointer(filterNumber))
		{
			wsprintf(filterAddNumber, TEXT("Filter %d"), filterNumber);

#ifndef UNICODE
			MultiByteToWideChar(CP_ACP, 0, filterAddNumber, -1, filterName, NUMELMS(filterName));
#else
			wcscpy(filterName, T2W(filterAddNumber));
#endif

			ASSERT(appData->filterList->getFilterPointer(filterNumber));
			hr = gcap.pFg->AddFilter(appData->filterList->getFilterPointer(filterNumber), filterName);
			if(hr != S_OK)
				ErrMsg(TEXT("Error adding Pre Filter %d"), filterNumber);

			if(filterNumber == 0)
			{
				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					gcap.pVCap, NULL, appData->filterList->getFilterPointer(filterNumber));
			}
			else
			{
				hr = gcap.pBuilder->RenderStream(NULL, NULL,
					appData->filterList->getFilterPointer(filterNumber-1), NULL, appData->filterList->getFilterPointer(filterNumber));
			}
			filterNumber++;
		}
		
		//Now add the Smart Tee
		CoCreateInstance(CLSID_SmartTee, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&gcap.SmartTeeFilter);

		hr = gcap.pFg->AddFilter(gcap.SmartTeeFilter, L"Smart Tea");
		if(hr != S_OK)
		{
			ErrMsg(TEXT("Error adding the Smart Tea"));
			return false;
		}
		hr = gcap.pBuilder->RenderStream(NULL, NULL,
			appData->filterList->getFilterPointer(filterNumber-1), NULL, gcap.SmartTeeFilter);
	}
	else
	{
		//Now add the Smart Tee
		CoCreateInstance(CLSID_SmartTee, NULL, CLSCTX_INPROC, IID_IBaseFilter, (void **)&gcap.SmartTeeFilter);

		hr = gcap.pFg->AddFilter(gcap.SmartTeeFilter, L"Smart Tea");
		if(hr != S_OK)
		{
			ErrMsg(TEXT("Error adding the Smart Tea"));
			return false;
		}
		hr = gcap.pBuilder->RenderStream(&PIN_CATEGORY_CAPTURE, NULL,
			gcap.pVCap, NULL, gcap.SmartTeeFilter);
		if(hr != S_OK)
		{
			ErrMsg(TEXT("Error connecting the Smart Tea to capture filter"));
			return false;
		}
	}

	return true;
}

// build the preview graph!
//
// !!! PLEASE NOTE !!!  Some new WDM devices have totally separate capture
// and preview settings.  An application that wishes to preview and then 
// capture may have to set the preview pin format using IAMStreamConfig on the
// preview pin, and then again on the capture pin to capture with that format.
// In this sample app, there is a separate page to set the settings on the 
// capture pin and one for the preview pin.  To avoid the user
// having to enter the same settings in 2 dialog boxes, an app can have its own
// UI for choosing a format (the possible formats can be enumerated using
// IAMStreamConfig) and then the app can programmatically call IAMStreamConfig
// to set the format on both pins.
//
BOOL BuildPreviewGraph() 
{
	if(gcap.log) logAction(TEXT("BuildPreviewGraph()"));

    int cy, cyBorder;
    HRESULT hr;
    //AM_MEDIA_TYPE *pmt;

    // No rebuilding while we're running
    if(gcap.fCapturing || gcap.fPreviewing)
        return FALSE;

    // we already have another graph built... tear down the old one
    if(gcap.fPreviewGraphBuilt || gcap.fCaptureGraphBuilt)
	{
		StopPreview();
		TearDownGraph();		
	}

    // We don't have the necessary capture filters
    if(gcap.pVCap == NULL)
        return FALSE;
    if(gcap.pACap == NULL && appData->fCapAudio)
        return FALSE;

	if(SetVideoFormat(0) < 0)
		return FALSE;

    // Create and add the Smart Tee filter if we
	// already dont have one, the smart tee should stay in
	// the graph all the time except when adding transform filters.
	if(!gcap.SmartTeeFilter)
	{
		AddSmartTeeToGraph();
	}

	// Add the video Render
	if(gcap.videoRender == NULL && appData->ShowHistogram)
		CoCreateInstance(CLSID_VideoRenderer, NULL, CLSCTX_INPROC, IID_IBaseFilter,
			(void **)&gcap.videoRender);
		
	if(gcap.videoRender == NULL && appData->ShowHistogram)
	{
		ErrMsg(TEXT("Error creating the preview Video Render"));
		goto BuildPreviewFailed;
	}

	if(gcap.videoRender != NULL)
	{
		hr = gcap.pFg->AddFilter(gcap.videoRender, L"Video Render");
		if(hr != S_OK)
		{
			ErrMsg(TEXT("Error adding the preview Video Render"));
			goto BuildPreviewFailed;
		}
	}

	// if we want to preview the Smart Tee or 
	// directly from the capture driver filter preview pin
	if(appData->UseSmartTee)
	{
		if(gcap.log) logAction(TEXT("BuildPreviewGraph() - using smart tee for peview"));

		hr = gcap.pBuilder->RenderStream(
			NULL, NULL, gcap.SmartTeeFilter, NULL, gcap.videoRender);

		if(hr != S_OK)
		{
			ErrMsg(TEXT("Error rendering the Smart Tee filter preview pin!"));
			goto BuildPreviewFailed;
		}
	}
	else
	{
		if(gcap.log) logAction(TEXT("BuildPreviewGraph() - using preview pin for peview"));

		hr = gcap.pBuilder->RenderStream(
			&PIN_CATEGORY_PREVIEW, NULL, gcap.pVCap, NULL, gcap.videoRender);

		if(hr != S_OK)
		{
			ErrMsg(TEXT("Error rendering the Preview!\nTry using the \"Smart Tee\" option for previewing."));
			goto BuildPreviewFailed;
		}
	}

	gcap.fPreviewGraphBuilt = TRUE;

	//
	// Show the Audio Scope if needed
	//
	if(appData->ShowAudioScope)
	{
		if(gcap.log) logAction(TEXT("Creating, Adding and Rendering the Audio Scope"));

		if(gcap.audioScope == NULL)
		{
			CoCreateInstance(CLSID_Scope, NULL, CLSCTX_INPROC, IID_IBaseFilter,
				(void **)&gcap.audioScope);
		}

		hr = gcap.pFg->AddFilter(gcap.audioScope, L"Audio Scope");

		if(hr != S_OK)
		{
	        ErrMsg(TEXT("Error adding the Audio Scope Filter\nMake sure you have installed the filters in the filter directory!"));
			goto BuildPreviewFailed;
		}
		else
		{
			hr = gcap.pBuilder->RenderStream(NULL, NULL,
				gcap.pACap, NULL, gcap.audioScope);
			if(hr != S_OK)
			{
				ErrMsg(TEXT("Error adding the Audio Scope Filter\nMake sure you have installed the filters in the filter directory!"));
				goto BuildPreviewFailed;
			}
		}
	}

    // GMakeet the preview window a child of our app's window
    // This will find the IVideoWindow interface on the renderer.  It is 
    // important to ask the filtergraph for this interface... do NOT use
    // ICaptureGraphBuilder2::FindInterface, because the filtergraph needs to
    // know we own the window so it can give us display changed messages, etc.

	// get the IVideoWindow from the actual video render
	// if we have one, we may not as this is onlyused when using the colour histogram
	if(gcap.videoRender != NULL)
		hr = gcap.videoRender->QueryInterface(IID_IVideoWindow, (void **)&gcap.pVW);
	else
		hr = gcap.pFg->QueryInterface(IID_IVideoWindow, (void **)&gcap.pVW);

    if(hr != NOERROR)
	{
        ErrMsg(TEXT("This graph cannot preview properly"));
	}
    else
	{
        RECT rc;
        gcap.pVW->put_Owner((OAHWND)ghwndApp);    // We own the window now
        gcap.pVW->put_WindowStyle(WS_CHILD);    // you are now a child

        // give the preview window all our space but where the status bar is
        GetClientRect(ghwndApp, &rc);

        cyBorder = GetSystemMetrics(SM_CYBORDER);

		if(appData->HideBars)
		{
			gcap.pVW->SetWindowPosition(0, 0, rc.right, rc.bottom); // be this big
		}
		else
		{
			cy = statusGetHeight() + cyBorder;
			rc.bottom -= cy;
			gcap.pVW->SetWindowPosition(0, gcap.toolBarHeight, rc.right, rc.bottom-gcap.toolBarHeight);
		}

        gcap.pVW->put_Visible(OATRUE);
    }

    // messages from the preview graph to the application
    hr = gcap.pFg->QueryInterface(IID_IMediaEventEx, (void **)&gcap.pME);
	if(hr == NOERROR)
	{
		gcap.pME->SetNotifyWindow((OAHWND)ghwndApp, WM_FGNOTIFY, 0);
	}

	//
	// Show the Colour Histogram if needed
	//
	if(appData->ShowHistogram)
	{
		if(gcap.log) logAction(TEXT("Creating, Adding and Rendering the Video Histogram Filter"));

		if(gcap.colourHistogram == NULL)
		{
			CoCreateInstance(CLSID_Histogram, NULL, CLSCTX_INPROC, IID_IBaseFilter,
				(void **)&gcap.colourHistogram);
		}

		hr = gcap.pFg->AddFilter(gcap.colourHistogram, L"Colour Histogram");

		if(hr != S_OK)
		{
	        ErrMsg(TEXT("Error adding the Colour Histogram Filter\nMake sure you have installed the filters in the filter directory!"));
		}
		else
		{
			hr = gcap.pBuilder->RenderStream(NULL, NULL,
				gcap.SmartTeeFilter, gcap.colourHistogram, NULL);
		}
	}

    // Add our graph to the running object table, which will allow
    // the GraphEdit application to "spy" on our graph
#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(gcap.pFg, &g_dwGraphRegister);
    if (FAILED(hr))
    {
        ErrMsg(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr);
        g_dwGraphRegister = 0;
    }
#endif

	changeChannel(appData->TunerChannel);

    // All done.
    gcap.fPreviewGraphBuilt = TRUE;

	if(gcap.log) logAction(TEXT("BuildPreviewGraph() - exit"));

	return TRUE;

BuildPreviewFailed:

    TearDownGraph();
	FreeCapFilters();
	InitCapFilters();

	gcap.tunerSet = false;
	changeChannel(appData->TunerChannel);
				
	if(gcap.pCrossbar)
		gcap.pCrossbar->SetInputIndex(appData->iVideoInputIndex);
				
	// Set the video amp colour settings
	setVideoAmpSettings();

	if(gcap.log) logAction(TEXT("BuildPreviewGraph() - Returned Failed"));

	return FALSE;
}


// Start previewing
//
BOOL StartPreview()
{
	if(gcap.log) logAction(TEXT("StartPreview()"));

	if(gcap.fCapturing)
	{
		// disable for now until I can work out what is going on
/*
		if(gcap.pVW)
		{
			IPin *preview;
			gcap.SmartTeeFilter->FindPin(L"Preview", &preview);

			IAMStreamControl *control;
			if(preview)
				preview->QueryInterface(IID_IAMStreamControl, (void **)&control);

			if(control)
				control->StartAt(NULL, NULL);

			PIN_INFO info;
			IBaseFilter *base;
			if(preview)
			{
				preview->QueryPinInfo(&info);
				base = info.pFilter;
			}
			if(base)
			{
				//base->Pause();
				base->Stop();
				HRESULT result = base->Run(NULL);

				if(result != S_OK)
					ErrMsg(TEXT("Error Starting Filter %d"), result);

				base->Release();
			}
			base = NULL;

			if(control)
				control->Release();
			control = NULL;

			if(preview)
				preview->Release();
			preview = NULL;

			gcap.pVW->put_Visible(OATRUE);
		}
*/
		gcap.fPreviewing = TRUE;
		return TRUE;
	}

    // way ahead of you
    if(gcap.fPreviewing)
        return TRUE;

	BuildPreviewGraph();

    if(!gcap.fPreviewGraphBuilt)
        return FALSE;

    // run the graph
    IMediaControl *pMC = NULL;
    HRESULT hr = gcap.pFg->QueryInterface(IID_IMediaControl, (void **)&pMC);
    if(SUCCEEDED(hr))
	{
		if(gcap.log) logAction(TEXT("StartPreview() - Starting Preview Graph"));

        hr = pMC->Run();
        if(FAILED(hr))
		{
            // stop parts that ran
            pMC->Stop();
        }
        pMC->Release();
    }
    if(FAILED(hr))
	{
		CHAR error[256];
		hrToText(hr, error);
        ErrMsg(TEXT("Cannot run preview graph : %s : %x"), error, hr);
        return FALSE;
    }

	ResizeWindow();

    gcap.fPreviewing = TRUE;

	// Set the toolbar button's
	if(gcap.toolBarHWND != NULL)
	{
		SendMessage(
			gcap.toolBarHWND,
			TB_CHECKBUTTON,
			(WPARAM) MENU_PREVIEW,	//button number
			(LPARAM) MAKELONG (appData->fWantPreview, 0)
			);
	}

	if(gcap.log) logAction(TEXT("StartPreview() - exit"));

    return TRUE;
}


// stop the preview graph
//
BOOL StopPreview() 
{
	if(gcap.log) logAction(TEXT("StopPreview()"));

	if(gcap.fCapturing)
	{
		if(gcap.pVW)
		{
			IPin *preview = NULL;
			gcap.SmartTeeFilter->FindPin(L"Preview", &preview);

			IAMStreamControl *control = NULL;
			if(preview)
			{
				preview->QueryInterface(IID_IAMStreamControl, (void **)&control);
				preview->Release();
			}
			preview = NULL;

			if(control != NULL)
			{
				control->StopAt(NULL, true, NULL);
				control->Release();
			}
			control = NULL;

			// Clear the Screen
			gcap.pVW->put_Visible(OAFALSE);			
			InvalidateRect(ghwndApp, NULL, TRUE);
			
		}

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_PREVIEW,
			MAKELONG (true, 0)
			);

		gcap.fPreviewing = FALSE;
		return TRUE;
	}

	// way ahead of you
    if(!gcap.fPreviewing)
	{
        return FALSE;
    }

    // stop the graph
    IMediaControl *pMC = NULL;
    HRESULT hr = gcap.pFg->QueryInterface(IID_IMediaControl, (void **)&pMC);
    if(SUCCEEDED(hr))
	{
        hr = pMC->Stop();
        pMC->Release();
    }
    if(FAILED(hr))
	{
        ErrMsg(TEXT("Error %x: Cannot stop preview graph"), hr);
        return FALSE;
    }

    gcap.fPreviewing = FALSE;

	// Clear the Screen
	gcap.pVW->put_Visible(OAFALSE);			
	InvalidateRect(ghwndApp, NULL, TRUE);

	TearDownGraph();

	// Set the toolbar button's
	if(gcap.toolBarHWND != NULL)
	{
		SendMessage(
			gcap.toolBarHWND,
			TB_CHECKBUTTON,
			(WPARAM) MENU_PREVIEW,	//button number
			(LPARAM) MAKELONG (appData->fWantPreview, 0)
			);
	}

    return TRUE;
}


// start the capture graph
//
BOOL StartCapture() 
{
	if(gcap.log) logAction(TEXT("StartCapture()"));

    HRESULT hr;

    // way ahead of you
    if(gcap.fCapturing)
        return TRUE;

    // or we'll get confused
    if(gcap.fPreviewing)
        StopPreview();

    // or we'll crash
    if(!gcap.fCaptureGraphBuilt)
        return FALSE;

	if(gcap.FlowMeterVideoInterface)
		gcap.FlowMeterVideoInterface->resetALL();
	if(gcap.FlowMeterAudioInterface)
		gcap.FlowMeterAudioInterface->resetALL();

	if(appData->useAudioResample && gcap.AudioResampleFilter)
	{
		gcap.AudioResampleFilter->QueryInterface(IID_IAudioResample,
			(void **)&gcap.AudioResampleInterface);

		if(gcap.AudioResampleInterface && appData->audioResampleType == 1)
			gcap.AudioResampleInterface->set_Percent(appData->ResampleAudioTo);
	}

	//Set the base priority to High
	SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);

	// Disable Screen Saver and power managment
	SystemParametersInfo(SPI_GETSCREENSAVEACTIVE, 0, &gcap.screenSaverState, 0);
	SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, FALSE, 0, 0);

	SystemParametersInfo(SPI_SETLOWPOWERTIMEOUT, FALSE, 0, 0);
	SystemParametersInfo(SPI_SETPOWEROFFTIMEOUT, FALSE, 0, 0);


    // prepare to run the graph
    IMediaControl *pMC = NULL;
    hr = gcap.pFg->QueryInterface(IID_IMediaControl, (void **)&pMC);
    if(FAILED(hr))
	{
		// Reset Screen Saver State to what it was
		SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, gcap.screenSaverState, 0, 0);

        ErrMsg(TEXT("Error %x: Cannot get IMediaControl"), hr);
        return FALSE;
    }

	// Run the Graph
	hr = pMC->Run();

    if(FAILED(hr))
	{
        // stop parts that started
        pMC->Stop();
        pMC->Release();
        ErrMsg(TEXT("Error %x: Cannot start graph"), hr);
        return FALSE;
    }
  
    pMC->Release();

    // when did we start capture?
    gcap.lCapStartTime = timeGetTime();

    // update status every 1 second
    SetTimer(ghwndApp, 1, 1000, NULL);

	//Resize the main window to the same size as the output video
	ResizeWindow();

	if(gcap.fPreviewGraphBuilt) 
		gcap.fPreviewing = TRUE;

	//Get free HD space when cap started
	gcap.freeHDatStart = GetFreeDiskSpaceInKB(appData->szCaptureFile);
	gcap.freeHDatStart *= 1024;

    gcap.fCapturing = TRUE;

	// Set the toolbar button's
	if(gcap.toolBarHWND != NULL)
	{
		SendMessage(
			gcap.toolBarHWND,
			TB_CHECKBUTTON,
			(WPARAM) MENU_PREVIEW,
			(LPARAM) MAKELONG (appData->fWantPreview, 0)
			);

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_START_CAP,
			MAKELONG (true, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SHOW_DROPPED_FRAMES,
			MAKELONG (true, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SETTINGS,
			MAKELONG (true, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SAVESETTINGS,
			MAKELONG (true, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SAVETOFILE,
			MAKELONG (true, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_LOADFROMFILE,
			MAKELONG (true, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_STOP_CAP,
			MAKELONG (false, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_PREVIEW,
			MAKELONG ((!appData->fWantPreview), 0)
			); 

	}

    return TRUE;
}

// stop the capture graph
//
BOOL StopCapture() 
{
	if(gcap.log) logAction(TEXT("StopCapture()"));

    // If not capturing return
    if(!gcap.fCapturing)
	{
        return FALSE;
    }

    // stop the graph
    IMediaControl *pMC = NULL;
    HRESULT hr = gcap.pFg->QueryInterface(IID_IMediaControl, (void **)&pMC);
    if(SUCCEEDED(hr))
	{
        hr = pMC->Stop();
        pMC->Release();
    }
    if(FAILED(hr))
	{
        ErrMsg(TEXT("Error %x: Cannot stop graph"), hr);
        return FALSE;
    }

    gcap.fCapturing = FALSE;
	gcap.fPreviewing = FALSE;

    // when the graph was stopped
    gcap.lCapStopTime = timeGetTime();

    // no more status bar updates
    KillTimer(ghwndApp, 1);

    // one last time for the final count and all the stats
	// if Show on stop is set display the stats window
	if(appData->showStatsOnStop)
		ShowCapStats(true);
    UpdateStatus(true);

    // !!! get rid of menu garbage
    InvalidateRect(ghwndApp, NULL, TRUE);
	
	// Reset the time hold
	gcap.lTimeHold = 0;

	// Set priority to normal
	SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);

	// Reset Screen Saver State to what it was before capture
	SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, gcap.screenSaverState, 0, 0);

	// Get the dropped frame list
	if(gcap.FlowMeterVideoInterface)
	{
		if(gcap.droppedList)
		{
			delete []gcap.droppedList;
			gcap.droppedList = NULL;
		}

		if(gcap.droppedList)
			delete gcap.droppedList;

		gcap.droppedList = new __int64[5000];
		memset(gcap.droppedList , 0, sizeof(gcap.droppedList[0])*5000);
		gcap.FlowMeterVideoInterface->get_DroppedFramesList(gcap.droppedList);
	}

	TearDownGraph();

	// Set the toolbar button's
	if(gcap.toolBarHWND != NULL)
	{
		SendMessage(
			gcap.toolBarHWND,
			TB_CHECKBUTTON,
			(WPARAM) MENU_PREVIEW,
			(LPARAM) MAKELONG (appData->fWantPreview, 0)
			);

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_START_CAP,
			MAKELONG (false, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SHOW_DROPPED_FRAMES,
			MAKELONG (false, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SETTINGS,
			MAKELONG (false, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SAVESETTINGS,
			MAKELONG (false, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_SAVETOFILE,
			MAKELONG (false, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_LOADFROMFILE,
			MAKELONG (false, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_STOP_CAP,
			MAKELONG (true, 0)
			); 

        SendMessage(
			gcap.toolBarHWND,
			TB_HIDEBUTTON,
			(WPARAM) MENU_PREVIEW,
			MAKELONG (false, 0)
			);

	}

    return TRUE;
}

//
void UpdateStatus(bool lastUpdate) 
{
    HRESULT hr;
    LONG lDropped = 0, lNot=0, lAvgFrameSize = 0;
    TCHAR tach[160];

	double VidComp = 0;
	double videoRate = 0;
	double audioRate = 0;
	LONGLONG droppedFrames = 0;
	LONGLONG VideoDataBytes = 0;
	LONGLONG AudioDataBytes = 0;
	LONGLONG AudioSamples = 0;
	LONGLONG VideoSamples = 0;
	LONGLONG TotalDataBytes = 0;

	double actualPercent = 0;
	
    if(!gcap.pDF)
	{
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
			&MEDIATYPE_Video, gcap.pVCap,
			IID_IAMDroppedFrames, (void **)&gcap.pDF);
    }

    // this filter can't tell us dropped frame info.
    if(!gcap.pDF)
	{
        statusUpdateStatus(ghwndStatus,
            TEXT("Filter cannot report capture information"), 1);
        return;
    }

    // average size of frame captured
    gcap.pDF->GetAverageFrameSize(&lAvgFrameSize);

	// Get the Flow Meter interfacees for Audio and Video Flow Meters
    if(!gcap.FlowMeterVideoInterface)
		gcap.pBuilder->FindInterface(NULL, NULL, gcap.FlowMeterVideo, IID_IFlowMeter, (void **)&gcap.FlowMeterVideoInterface);

	// Only get the Audio interface if we have an audio flow meter
    if(gcap.FlowMeterAudio && !gcap.FlowMeterAudioInterface)
		gcap.pBuilder->FindInterface(NULL, NULL, gcap.FlowMeterAudio, IID_IFlowMeter, (void **)&gcap.FlowMeterAudioInterface);

	// Get the Bytes throughput data from the flow meters
	gcap.FlowMeterVideoInterface->get_FlowData(&VideoDataBytes);
	gcap.FlowMeterVideoInterface->get_TransformCount(&VideoSamples);
	gcap.FlowMeterVideoInterface->get_SamplesPerSecond(&videoRate);

	// get Dropped Frames from the FlowMeter filter
	gcap.FlowMeterVideoInterface->get_DroppedFrames(&droppedFrames);

	// If no audio then audio bytes always zero
	if(gcap.FlowMeterAudioInterface)
	{
		gcap.FlowMeterAudioInterface->get_FlowData(&AudioDataBytes);
		gcap.FlowMeterAudioInterface->get_TransformCount(&AudioSamples);
		gcap.FlowMeterAudioInterface->get_SamplesPerSecond(&audioRate);
	}

	// Calculate the audio video rate difference
	double AVdiff =
		(videoRate / (double)appData->FrameRate) -
		(audioRate / (double)appData->Audio_nSamplesPerSec);

	// Calculate the total data throughput
	TotalDataBytes = VideoDataBytes + AudioDataBytes;

	// Get the dropped and not dropped fromthe Video capture
	// filter/driver
    hr = gcap.pDF->GetNumDropped(&lDropped);
    if(hr == S_OK)
        hr = gcap.pDF->GetNumNotDropped(&lNot);
    if(hr != S_OK)
        return;

	// Time the capture has been running
	LONG lTime = timeGetTime() - gcap.lCapStartTime;
	
	LONG lHours = lTime / (1000 * 60 * 60);
	LONG lMinutes = (lTime - (lHours * 1000 * 60 * 60)) / (1000 * 60);
	LONG lSecond = (lTime - (lHours * 1000 * 60 * 60) - (lMinutes * 1000 * 60)) / 1000;
	
	// Calculate the Stats
	gcap.lDroppedBase = lDropped;
	gcap.lNotBase = lNot;
	
	__int64 freeSpace = GetFreeDiskSpaceInKB(appData->szCaptureFile);
	freeSpace = freeSpace * 1024;

	LONG TotalBytesSec = (TotalDataBytes * 1000) / lTime;
	LONG VideoBytesSec = (VideoDataBytes * 1000) / lTime;
	LONG AudioBytesSec = (AudioDataBytes * 1000) / lTime;
	LONGLONG SecondsLeft = freeSpace / (__int64)TotalBytesSec;
	VidComp = ((DOUBLE)lAvgFrameSize / (DOUBLE)VideoDataBytes) * lNot;

	// Calcualte the time left
	LONG lTimeLeftHours = SecondsLeft / (60 * 60);
	LONG lTimeLeftMinutes = (SecondsLeft - (lTimeLeftHours * 60 * 60)) / 60;
	LONG lTimeLeftSecond = (SecondsLeft - (lTimeLeftHours * 60 * 60) - (lTimeLeftMinutes * 60));

	// Update the Status line at the bottom of the app window
	// with a summary of the capture stats
	wsprintf(tach, TEXT("Capturing : %.2d:%.2d:%.2d"),
		lHours,
		lMinutes,
		lSecond);
	
	statusUpdateStatus(ghwndStatus, tach, 1);

/////////////////////////////////////////////////////////////////////////

	// Calculate the Audio Video Sync offset
	// Use a smoothing factor to average out interuptions
	// in the stream
	// Ignore the first 10 seconds 
	// also ignore the data until 60 sec is reached

	// Now adjust the audio resample filter if required
	if(appData->useAudioResample)
	{

		// get the audio resample filter interface if we already dont have it
		if(gcap.AudioResampleFilter && !gcap.AudioResampleInterface)
		{
			gcap.AudioResampleFilter->QueryInterface(IID_IAudioResample,
				(void **)&gcap.AudioResampleInterface);
		}

/*
this is to show the audio resample filter properties page.
this is not needed at the moment but may be added later as an option.

		// This will show the properties page of the Audio resample filter
		if(gcap.AudioResampleInterface && false)
		{
			
			ISpecifyPropertyPages *pProp;
			HRESULT hr = gcap.AudioResampleFilter->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pProp);
			if (SUCCEEDED(hr)) 
			{
				// Get the filter's name and IUnknown pointer.
				IUnknown *pFilterUnk = NULL;
				gcap.AudioResampleFilter->QueryInterface(IID_IUnknown, (void **)&pFilterUnk);
				
				// Show the page. 
				CAUUID caGUID;
				pProp->GetPages(&caGUID);
				pProp->Release();
				
				OleCreatePropertyFrame(
					NULL,                   // Parent window
					0, 0,                   // (Reserved)
					L"Filter",				// Caption for the dialog box
					1,                      // Number of objects (just the filter)
					&pFilterUnk,            // Array of object pointers. 
					caGUID.cElems,          // Number of property pages
					caGUID.pElems,          // Array of property page CLSIDs
					0,                      // Locale identifier
					0, NULL                 // Reserved
					);
				
				// Clean up.
				pFilterUnk->Release();
				CoTaskMemFree(caGUID.pElems);
			}
		}
*/

		if(gcap.AudioResampleInterface)
		{
			if(appData->audioResampleType == 1)
			{
				gcap.AVsync = appData->ResampleAudioTo;
			}
			else if(lTime < 60000)
			{
				gcap.AVsync = 100.0;
			}
			else
			{
				gcap.AVsync = (gcap.AVsync * 24) + ((AVdiff + 1) * 100);
				gcap.AVsync /= 25;
			}

			gcap.AudioResampleInterface->get_ActualPercent(&actualPercent);
			gcap.AudioResampleInterface->set_Percent(gcap.AVsync);
		}
	}
	else
		gcap.AVsync = 0;

//////////////////////////////////////////////////////////////////////

	// log data to log file is required
	if(appData->LogAVdiff)
		logData(lTime, videoRate, audioRate, AVdiff, gcap.AVsync, lDropped, droppedFrames, lastUpdate);

	// Update the Capture Stats Dialog with the current stats
	// If it is available
	if (IsWindow(hwndSTATUS)) 
	{ 
		TCHAR tach[_MAX_PATH];
		
		wsprintf(tach, TEXT("%d"), lNot);
		SetDlgItemText(hwndSTATUS, IDC_Frames, tach);
		
		wsprintf(tach, TEXT("%d"), lDropped);
		SetDlgItemText(hwndSTATUS, IDC_Dropped, tach);

		wsprintf(tach, TEXT("%d"), droppedFrames);
		SetDlgItemText(hwndSTATUS, IDC_Dropped_NEW, tach);		

		wsprintf(tach, TEXT("%.2d:%.2d:%.2d"), lHours, lMinutes, lSecond);
		SetDlgItemText(hwndSTATUS, IDC_Time, tach);
		
		wsprintf(tach, TEXT("%.2d:%.2d:%.2d"), lTimeLeftHours, lTimeLeftMinutes, lTimeLeftSecond);
		SetDlgItemText(hwndSTATUS, IDC_TimeLeft, tach);
		
		FormatNumber(freeSpace, tach);
		SetDlgItemText(hwndSTATUS, IDC_FreeSpace, tach);

		FormatNumber(VideoDataBytes, tach);
		SetDlgItemText(hwndSTATUS, IDC_VideoBytes, tach);

		FormatNumber(AudioDataBytes, tach);
		SetDlgItemText(hwndSTATUS, IDC_AudioBytes, tach);

		FormatNumber(TotalDataBytes, tach);
		SetDlgItemText(hwndSTATUS, IDC_TotalBytes, tach);

		FormatNumber(TotalBytesSec, tach);
		SetDlgItemText(hwndSTATUS, IDC_TotalBytesSec, tach);

		FormatNumber(VideoBytesSec, tach);
		SetDlgItemText(hwndSTATUS, IDC_VideoBytesSec, tach);

		FormatNumber(AudioBytesSec, tach);
		SetDlgItemText(hwndSTATUS, IDC_AudioBytesSec, tach);

		_stprintf(tach, TEXT("%.3f"), VidComp);
		SetDlgItemText(hwndSTATUS, IDC_VideoCompression, tach);

		// Set the Free Space Bar
		HWND hwndFreeBar = GetDlgItem(hwndSTATUS, IDC_FREESPACEBAR);
		int currentMax = SendMessage(hwndFreeBar, PBM_GETRANGE, 0, 0); 
		int startedWith = gcap.freeHDatStart / 1048576;
		int currentFree = freeSpace / 1048576;

		if(currentMax < startedWith)
			SendMessage(hwndFreeBar, PBM_SETRANGE, 0, MAKELPARAM(0, startedWith));
		else if(currentMax < currentFree)
			SendMessage(hwndFreeBar, PBM_SETRANGE, 0, MAKELPARAM(0, currentFree));
		
		SendMessage(hwndFreeBar, PBM_SETPOS, currentFree, 0);

		// Set Audio/Video rate info
		_stprintf(tach, TEXT("%f"), videoRate);
		SetDlgItemText(hwndSTATUS, IDC_VIDEO_ACT, tach);

		_stprintf(tach, TEXT("%f"), audioRate);
		SetDlgItemText(hwndSTATUS, IDC_AUDIO_ACT, tach);

		_stprintf(tach, TEXT("%f"), AVdiff);
		SetDlgItemText(hwndSTATUS, IDC_AUDIO_DIFF, tach);

		_stprintf(tach, TEXT("%f"), gcap.AVsync);
		SetDlgItemText(hwndSTATUS, IDC_AVadjust, tach);

		_stprintf(tach, TEXT("%f"), actualPercent);
		SetDlgItemText(hwndSTATUS, IDC_AVadjust_actual, tach);
	}

	///////////////////////////////////////////////////////////
	// Now Write the capture stats to a file is needed
	///////////////////////////////////////////////////////////
	if(lastUpdate && appData->SaveCapStats)
	{
		TCHAR numFormat[32];
		::ofstream  captureStats;

		captureStats.open(gcap.currentFileName);
		captureStats<<"Capture Stats\n";

		_stprintf(tach, "Frames Captured: %d\n", lNot);
		captureStats<<tach;

		_stprintf(tach, "Frames Dropped1: %d\n", lDropped);
		captureStats<<tach;

		_stprintf(tach, "Frames Dropped2: %d\n", droppedFrames);
		captureStats<<tach;

		_stprintf(tach, "Time : %.2d:%.2d:%.2d\n", lHours, lMinutes, lSecond);
		captureStats<<tach;

		_stprintf(tach, "Time Left: %.2d:%.2d:%.2d\n", lTimeLeftHours, lTimeLeftMinutes, lTimeLeftSecond);
		captureStats<<tach;

		FormatNumber(freeSpace, numFormat);
		_stprintf(tach, "Free Space: %s\n", numFormat);
		captureStats<<tach;

		FormatNumber(VideoDataBytes, numFormat);
		_stprintf(tach, "Video Bytes: %s\n", numFormat);
		captureStats<<tach;

		FormatNumber(AudioDataBytes, numFormat);
		_stprintf(tach, "Audio Bytes: %s\n", numFormat);
		captureStats<<tach;

		FormatNumber(TotalDataBytes, numFormat);
		_stprintf(tach, "Total Bytes: %s\n", numFormat);
		captureStats<<tach;

		FormatNumber(VideoBytesSec, numFormat);
		_stprintf(tach, "Video Bytes Sec: %s\n", numFormat);
		captureStats<<tach;

		FormatNumber(AudioBytesSec, numFormat);
		_stprintf(tach, "Audio Bytes Sec: %s\n", numFormat);
		captureStats<<tach;

		FormatNumber(TotalBytesSec, numFormat);
		_stprintf(tach, "Total Bytes Sec: %s\n", numFormat);
		captureStats<<tach;

		_stprintf(tach, "Video Compression: %.3f\n", VidComp);
		captureStats<<tach;

		_stprintf(tach, "Video Rate: %f\n", videoRate);
		captureStats<<tach;		

		_stprintf(tach, "Audio Rate: %f\n", audioRate);
		captureStats<<tach;	

		_stprintf(tach, "AV Diff: %f\n", AVdiff);
		captureStats<<tach;	

		_stprintf(tach, "AV Adjust: %f\n", gcap.AVsync);
		captureStats<<tach;	
		
		_stprintf(tach, "AV Actual: %f\n", actualPercent);
		captureStats<<tach;	
		
		captureStats.close();
	}

}

// Check the devices we're currently using and make filters for them
//
void ChooseDevices(IMoniker *pmVideo, IMoniker *pmAudio) 
{
	if(gcap.log) logAction(TEXT("ChooseDevices(IMoniker *pmVideo, IMoniker *pmAudio)"));

    USES_CONVERSION;
#define VERSIZE 40
#define DESCSIZE 80
    int versize = VERSIZE;
    int descsize = DESCSIZE;
    WCHAR wachVer[VERSIZE]={0}, wachDesc[DESCSIZE]={0};
    TCHAR tachStatus[VERSIZE + DESCSIZE + 5]={0};

    // they chose a new device. rebuild the graphs
    if(gcap.pmVideo != pmVideo || gcap.pmAudio != pmAudio)
	{
        if(pmVideo)
		{
            pmVideo->AddRef();
        }
        if(pmAudio)
		{
            pmAudio->AddRef();
        }

        IMonRelease(gcap.pmVideo);
        IMonRelease(gcap.pmAudio);
        gcap.pmVideo = pmVideo;
        gcap.pmAudio = pmAudio;

        if(gcap.fPreviewing)
            StopPreview();
        if(gcap.fCaptureGraphBuilt || gcap.fPreviewGraphBuilt)
            TearDownGraph();

        FreeCapFilters();
        InitCapFilters();

//        if(appData->fWantPreview)
//		{
//			// were we previewing?
//            BuildPreviewGraph();
//			StartPreview();
//		}
    }
/*
    // Put the video driver name in the status bar - if the filter supports
    // IAMVideoCompression::GetInfo, that's the best way to get the name and
    // the version.  Otherwise use the name we got from device enumeration
    // as a fallback.
    if(gcap.pVC)
	{
        HRESULT hr = gcap.pVC->GetInfo(wachVer, &versize, wachDesc, &descsize,
            NULL, NULL, NULL, NULL);
        if(hr == S_OK)
		{
            // It's possible that the call succeeded without actually filling
            // in information for description and version.  If these strings
            // are empty, just display the device's friendly name.
            if(wcslen(wachDesc) && wcslen(wachVer))
			{
                wsprintf(tachStatus, TEXT("%s - %s"), W2T(wachDesc), W2T(wachVer));
                statusUpdateStatus(ghwndStatus, tachStatus);
                return;
            }
        }
    }

    // Since the GetInfo method failed (or the interface did not exist),
    // display the device's friendly name.
    statusUpdateStatus(ghwndStatus, W2T(gcap.wachFriendlyName));
*/
}

void ChooseInitDevices(int indexVideo, int indexAudio) 
{
	if(gcap.log) logAction(TEXT("ChooseInitDevices(%d, %d)"), indexVideo, indexAudio);

	IMoniker *pmVideo = 0, *pmAudio = 0;

	if(gcap.rgpmVideoMenu[indexVideo])
	{
		pmVideo = gcap.rgpmVideoMenu[indexVideo];
		pmVideo->AddRef();
	}

	if(gcap.rgpmAudioMenu[indexAudio])
	{
		pmAudio = gcap.rgpmAudioMenu[indexAudio];
		pmAudio->AddRef();
	}

	if(pmVideo && pmAudio)
		ChooseDevices(pmVideo, pmAudio);

    IMonRelease(pmVideo);
    IMonRelease(pmAudio);
}

// put all installed video and audio devices in the menus
//
void ScanForDevices()  //AddDevicesToMenu() 
{
	if(gcap.log) logAction(TEXT("ScanForDevices()"));

    USES_CONVERSION;

    if(gcap.fDeviceMenuPopulated)
	{
        return;
    }
    gcap.fDeviceMenuPopulated = true;
    gcap.iNumVCapDevices = 0;
	gcap.iNumACapDevices = 0;

    UINT    uIndex = 0;
    HRESULT hr;
    BOOL bCheck = FALSE;

    // enumerate all video capture devices
    ICreateDevEnum *pCreateDevEnum;
    hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER,
        IID_ICreateDevEnum, (void**)&pCreateDevEnum);
    if(hr != NOERROR)
	{
        ErrMsg(TEXT("Error Creating Device Enumerator"));
        return;
    }

    IEnumMoniker *pEm;
    hr = pCreateDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory,
        &pEm, 0);
    if(hr != NOERROR)
	{
        ErrMsg(TEXT("Sorry, you have no video capture hardware"));
        goto EnumAudio;
    }

    pEm->Reset();
    ULONG cFetched;
    IMoniker *pM;
    while(hr = pEm->Next(1, &pM, &cFetched), hr==S_OK)
	{
        IPropertyBag *pBag;
        hr = pM->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pBag);
        if(SUCCEEDED(hr))
		{
            VARIANT var;
            var.vt = VT_BSTR;
            hr = pBag->Read(L"FriendlyName", &var, NULL);
            if(hr == NOERROR)
			{
                SysFreeString(var.bstrVal);

                ASSERT(gcap.rgpmVideoMenu[uIndex] == 0);
                gcap.rgpmVideoMenu[uIndex] = pM;
                pM->AddRef();
            }
            pBag->Release();
        }
        pM->Release();
        uIndex++;
    }
    pEm->Release();

    gcap.iNumVCapDevices = uIndex;

EnumAudio:

    // enumerate all audio capture devices
    uIndex = 0;
    bCheck = FALSE;

    ASSERT(pCreateDevEnum != NULL);

    hr = pCreateDevEnum->CreateClassEnumerator(CLSID_AudioInputDeviceCategory,
        &pEm, 0);
    pCreateDevEnum->Release();
    if(hr != NOERROR)
        return;
    pEm->Reset();
    while(hr = pEm->Next(1, &pM, &cFetched), hr==S_OK)
	{
        IPropertyBag *pBag;
        hr = pM->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pBag);
        if(SUCCEEDED(hr))
		{
            VARIANT var;
            var.vt = VT_BSTR;
            hr = pBag->Read(L"FriendlyName", &var, NULL);
            if(hr == NOERROR)
			{
                SysFreeString(var.bstrVal);

                ASSERT(gcap.rgpmAudioMenu[uIndex] == 0);
                gcap.rgpmAudioMenu[uIndex] = pM;
                pM->AddRef();                
            }
            pBag->Release();
        }
        pM->Release();
        uIndex++;
    }
    pEm->Release();
	gcap.iNumACapDevices = uIndex;
}

//
// Get the current audio format ans tore it in the appData object
//
BOOL GetAudioFormat() 
{
	if(gcap.log) logAction(TEXT("GetAudioFormat()"));

	IAMStreamConfig *audioStreamConfig = NULL;

	HRESULT hr = 0;

	if(gcap.pACap)
	{
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Audio,
				gcap.pACap, IID_IAMStreamConfig, (void **)&audioStreamConfig);
	}

//	if(hr == S_OK)
//		ErrMsg(TEXT("S_OK %x"), hr);
	if(hr == E_FAIL)
		ErrMsg(TEXT("E_FAIL %x"), hr);
	if(hr == E_NOINTERFACE)
		ErrMsg(TEXT("E_NOINTERFACE %x"), hr);
	if(hr == E_POINTER)
		ErrMsg(TEXT("E_POINTER %x"), hr);

    if(hr != NOERROR || audioStreamConfig == NULL)
	{
        ErrMsg(TEXT("Get Audio Format : Cannot find Audio IAMStreamConfig interface : %x"), hr);
		return FALSE;
    }

	AM_MEDIA_TYPE *pmt = NULL;
	LPWAVEFORMATEX lpwfx;

	hr = audioStreamConfig->GetFormat(&pmt);
	
    if(hr != NOERROR && pmt != NULL)
	{
		ErrMsg(TEXT("Error getting audio format media type"));
        return FALSE;
	}

	lpwfx = (LPWAVEFORMATEX)pmt->pbFormat;
	
	appData->Audio_nChannel = lpwfx->nChannels;
	appData->Audio_nSamplesPerSec = lpwfx->nSamplesPerSec;
	appData->Audio_nAvgBytesPerSec = lpwfx->nAvgBytesPerSec;
	appData->Audio_nBlockAlign = lpwfx->nBlockAlign;
	appData->Audio_wBitsPerSample = lpwfx->wBitsPerSample;
	appData->Audio_wFormatTag = lpwfx->wFormatTag;
	
    DeleteMediaType(pmt);

	if(audioStreamConfig)
		audioStreamConfig->Release();
	audioStreamConfig = NULL;

	return true;
}

int SetVideoFormat(int pintype)
{
	if(gcap.log) logAction(TEXT("SetVideoFormat(%d)"), pintype);

	if(gcap.pVCap == NULL)
		return -1;

	//MsgBox(TEXT("%d"), IsBadReadPtr(gcap.pVCap, sizeof(IBaseFilter)));

	int result = 0;

	if(gcap.fPreviewing)
		StopPreview();
	if(gcap.fCaptureGraphBuilt || gcap.fPreviewGraphBuilt)
		TearDownGraph();

	IAMStreamConfig *videoConfig = NULL;
	AM_MEDIA_TYPE *pmt = NULL;

	HRESULT hr = 0;

	if(pintype == 0)
	{
		hr = gcap.pBuilder->FindInterface(
		&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video,
		gcap.pVCap,
		IID_IAMStreamConfig,
		(void **)&videoConfig);
	}
	else
	{
		hr = gcap.pBuilder->FindInterface(
		&PIN_CATEGORY_PREVIEW,
		&MEDIATYPE_Video,
		gcap.pVCap,
		IID_IAMStreamConfig,
		(void **)&videoConfig);
	}

	if(videoConfig == NULL)
		return -2;

	if(gcap.log) logAction(TEXT("Getting Current Format"));

	hr = videoConfig->GetFormat(&pmt);

	if(hr != S_OK || pmt == NULL)
	{
		if(gcap.log) logAction(TEXT("Could Not Get Current Format"));

		if(videoConfig)
			videoConfig->Release();

		if(pmt)
			DeleteMediaType(pmt);

		return -9;
	}

    // Set the Custom settings
    if(appData->fUseCustomFormat)
	{
		if(gcap.log) logAction(TEXT("Setting New Format"));

		if(pmt->formattype == FORMAT_VideoInfo)
		{
			VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER *)pmt->pbFormat;
			pvi->AvgTimePerFrame = (LONGLONG)(((double)10000000) / appData->FrameRate);
			pvi->bmiHeader.biWidth = appData->CustomWidth;
			pvi->bmiHeader.biHeight = appData->CustomHeight;
			pmt->subtype = appData->ColourFormat;
			hr = videoConfig->SetFormat(pmt);
			if(hr != NOERROR)
			{
				CHAR error[256];
				hrToText(hr, error);
				ErrMsg(TEXT("Cannot Set Video Format : %s : %x"), error, hr);
				result = -3;
			}
		}
		else
		{
			ErrMsg(TEXT("Cannot set Video Format : Wrong Video Type : %x"), hr);
			result = -4;
		}
		
		if(gcap.log) logAction(TEXT("Deleting pmt"));

		DeleteMediaType(pmt);

    }
	else
		if(gcap.log) logAction(TEXT("Leaving Format Alone - No change made"));

	if(videoConfig)
	{
		if(gcap.log) logAction(TEXT("Releasing Pointer to video IAMStreamConfig"));
		videoConfig->Release();
	}
	videoConfig = NULL;

	if(gcap.log) logAction(TEXT("SetVideoFormat(%d) returning %d"), pintype, result);

	return result;
}

//
// Get the current audio format and store it in the appData object
//
BOOL SetAudioFormat() 
{
	if(gcap.log) logAction(TEXT("SetAudioFormat()"));

	if(gcap.fPreviewing)
		StopPreview();
	if(gcap.fCaptureGraphBuilt || gcap.fPreviewGraphBuilt)
		TearDownGraph();

	IAMStreamConfig *audioStreamConfig = NULL;

	HRESULT hr = 0;

	if(gcap.pACap)
	{

		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Audio,
			gcap.pACap, IID_IAMStreamConfig, (void **)&audioStreamConfig);
	}

//	if(hr == S_OK)
//		ErrMsg(TEXT("S_OK %x"), hr);
	if(hr == E_FAIL)
		ErrMsg(TEXT("E_FAIL %x"), hr);
	if(hr == E_NOINTERFACE)
		ErrMsg(TEXT("E_NOINTERFACE %x"), hr);
	if(hr == E_POINTER)
		ErrMsg(TEXT("E_POINTER %x"), hr);

    if(hr != NOERROR || audioStreamConfig == NULL)
	{
        ErrMsg(TEXT("Set Audio Format : Cannot find Audio IAMStreamConfig interface : %x"), hr);
		return FALSE;
    }

    DWORD dwSize;
    LPWAVEFORMATEX lpwfx;
    AM_MEDIA_TYPE *pmt = NULL;
	
    // What's the largest format size around?
    acmMetrics(NULL, ACM_METRIC_MAX_SIZE_FORMAT, &dwSize);

    hr = audioStreamConfig->GetFormat(&pmt);
    if(pmt == NULL || hr != NOERROR)
	{
		ErrMsg(TEXT("Error setting audio format: Can not get Media Type"));
        return false;
	}
	
    lpwfx = (LPWAVEFORMATEX)pmt->pbFormat;
    dwSize = max(dwSize, lpwfx->cbSize + sizeof(WAVEFORMATEX));
    lpwfx = (LPWAVEFORMATEX)GlobalAllocPtr(GHND, dwSize);

    if(lpwfx)
	{
        CopyMemory(lpwfx, pmt->pbFormat, pmt->cbFormat);
		
		lpwfx->nChannels = appData->Audio_nChannel;
		lpwfx->nSamplesPerSec = appData->Audio_nSamplesPerSec;
		lpwfx->nAvgBytesPerSec = appData->Audio_nAvgBytesPerSec;
		lpwfx->nBlockAlign = appData->Audio_nBlockAlign;
		lpwfx->wBitsPerSample = appData->Audio_wBitsPerSample;
		lpwfx->wFormatTag = appData->Audio_wFormatTag;
		
		// while streaming
		((CMediaType *)pmt)->SetFormat((LPBYTE)lpwfx,
			lpwfx->cbSize + sizeof(WAVEFORMATEX));

//MsgBox(TEXT("Saving new format"));	
		
		try
		{
			hr = audioStreamConfig->SetFormat(pmt);  // filter will reconnect
		}
		catch(...)
		{
			MsgBox(TEXT("There was an error thrown when setting the audio format"));
		}

		if(hr != NOERROR)
			ErrMsg(TEXT("%x: Cannot set custom settings for audio"), hr);
		
        GlobalFreePtr(lpwfx) ;
    }

    DeleteMediaType(pmt);

	if(audioStreamConfig)
		audioStreamConfig->Release();
	audioStreamConfig = NULL;
	
	return true;
}

//
// Process all of our WM_COMMAND messages.
//
LONG PASCAL AppCommand(HWND hwnd, unsigned msg, WPARAM wParam, LPARAM lParam) 
{

    int id = GET_WM_COMMAND_ID(wParam, lParam);
    switch(id)
	{
		case MENU_SETTINGS:
			{
				if(gcap.log)
					logAction(TEXT("Menu Open Settings Dialog MENU_SETTINGS"));

				BOOL wantedPreview = appData->fWantPreview;
				appData->fWantPreview = FALSE;

//				if(gcap.fPreviewing)
//					StopPreview();
//				TearDownGraph();

				ShowPropPages(hwnd);

				appData->fWantPreview = wantedPreview;

				if(gcap.fPreviewing)
					StopPreview();
				if(gcap.fCaptureGraphBuilt || gcap.fPreviewGraphBuilt)
					TearDownGraph();

				if(appData->fWantPreview)
				{
					StartPreview();
				}

				SetAppCaption();

				break;
			}
		case MENU_SHOW_DROPPED_FRAMES:
			{
				DialogBox(ghInstApp, MAKEINTRESOURCE(IDD_DroppedFrames), hwnd, 
					(DLGPROC)DroppedFrameProc);
				break;
			}

		// Save current settings as default
		case MENU_SAVESETTINGS:
			{
				if(gcap.log)
					logAction(TEXT("Menu Save Default Settings MENU_SAVESETTINGS"));

				GetAudioFormat();
				getTunerSettings();
				getVideoAmpSettings();
				//GetCurrentDevices();
				// Save the persistand data in the INI file
				// then delete the appData object

				// Get the current audio input settings
				if(gcap.pACap)
				{
					CAudioMixer *mix = new CAudioMixer(gcap.pACap);

					appData->audioInputIndex = mix->getInput();
					appData->audioInputLevel = mix->getLevel(appData->audioInputIndex);

					delete mix;
				}

				BOOL saved = appData->savePersistentData("INIT", true);

				if(!saved)
					ErrMsg(TEXT("Error saving to the INI File"));
				else
					MsgBox(TEXT("Current capture settings saved as default"));
				break;
			}

		// Save Profile information to file
		case MENU_SAVETOFILE:
			{
				if(gcap.log)
					logAction(TEXT("Menu Save Settings to File MENU_SAVETOFILE"));

				GetAudioFormat();
				getTunerSettings();
				getVideoAmpSettings();
				//GetCurrentDevices();

				// Get the current audio input settings
				if(gcap.pACap)
				{
					CAudioMixer *mix = new CAudioMixer(gcap.pACap);

					appData->audioInputIndex = mix->getInput();
					appData->audioInputLevel = mix->getLevel(appData->audioInputIndex);

					delete mix;
				}

				LPSTR FileName = new char[_MAX_PATH];
				BOOL haveOne = OpenSaveDialog(hwnd, FileName, _MAX_PATH);
				if(!haveOne) break;
				BOOL saved = appData->savePersistentData(FileName, false);
				delete []FileName;
				if(!saved)
					ErrMsg(TEXT("Error saving to the INI File"));
				break;
			}

		// Load Profile information from file
		case MENU_LOADFROMFILE:
			{
				if(gcap.log)
					logAction(TEXT("Menu Load Settings From File MENU_LOADFROMFILE"));

				LPSTR FileName = new char[_MAX_PATH];
				BOOL haveOne = OpenLoadDialog(hwnd, FileName, _MAX_PATH);
				if(!haveOne) break;

				BOOL loaded = appData->loadProfileData(FileName, false);
				if(!loaded)
					ErrMsg(TEXT("Error loading from the INI File"));
				delete []FileName;

				BOOL wantedPreview = appData->fWantPreview;
				appData->fWantPreview = FALSE;

				// Rebuild graph with new settings
				if(gcap.fPreviewing)
					StopPreview();
				TearDownGraph();    // now we need to rebuild

				ChooseInitDevices(appData->VideoDeviceIndex, appData->AudioDeviceIndex);

				SetAudioFormat();

				// Set the audio input settings
				if(appData->audioInputIndex > -1)
				{
					CAudioMixer *mix = new CAudioMixer(gcap.pACap);

					mix->setInput(appData->audioInputIndex);
					mix->setLevel(appData->audioInputLevel, appData->audioInputIndex);

					delete mix;
				}

				gcap.tunerSet = false;
				changeChannel(appData->TunerChannel);

				if(gcap.pCrossbar)
					gcap.pCrossbar->SetInputIndex(appData->iVideoInputIndex);

				appData->fWantPreview = wantedPreview;

				if(appData->fWantPreview)
				{
//					BuildPreviewGraph();
					StartPreview();
				}

				SetAppCaption();

				// Show Cap Stats
				ShowCapStats(appData->fWantCaptureStats);

				// Set the video amp colour settings
				setVideoAmpSettings();

				break;
			}

		//
		// Toggle Capture Stats
		//
        case MENU_SHOWSTATS:
			if(gcap.log)
				logAction(TEXT("Menu Show Capture Stats MENU_SHOWSTATS"));

			appData->fWantCaptureStats = !appData->fWantCaptureStats;
			ShowCapStats(appData->fWantCaptureStats);

			break;

        //
        // Exit App
        //
        case MENU_EXIT_APP:
            PostMessage(hwnd,WM_CLOSE,0,0L);
            break;
		
		//
		// start capturing
        //
        case MENU_START_CAP:
			if(gcap.log)
				logAction(TEXT("Menu Start Capture MENU_START_CAP"));

			TryToStartCapture();
			break;

		//
        // toggle preview
        // 
        case MENU_PREVIEW:
			if(gcap.log)
				logAction(TEXT("Menu Toggle Preview MENU_PREVIEW"));

            appData->fWantPreview = !appData->fWantPreview;

            if(appData->fWantPreview)
                StartPreview();
            else
                StopPreview();

			if(gcap.toolBarHWND != NULL)
			{
				SendMessage(
					gcap.toolBarHWND,
					TB_CHECKBUTTON,
					(WPARAM) MENU_PREVIEW,	//button number
					(LPARAM) MAKELONG (appData->fWantPreview, 0)
					);
			}

            break;

		//
        // stop capture
        //
        case MENU_STOP_CAP:
			if(gcap.log)
				logAction(TEXT("Menu Stop Capture MENU_STOP_CAP"));

            StopCapture();
            if(appData->fWantPreview)
			{
//				BuildPreviewGraph();
				StartPreview();
            }
            break;

    }
    return 0L;
}

// MsgBox - Opens a Message box with a message in it.  The user can
// select the OK button to continue

int MsgBox(LPTSTR sz,...) 
{
    static TCHAR tach[2000];
    va_list va;
    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);
    MessageBox(ghwndApp, tach, "Virtual VCR Message", MB_OK|MB_ICONINFORMATION|MB_TASKMODAL);
    return FALSE;
}

// ErrMsg - Opens a Message box with a error message in it.  The user can
// select the OK button to continue

int ErrMsg(LPTSTR sz,...) 
{
    static TCHAR tach[2000];
    va_list va;
    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);
    MessageBox(ghwndApp, tach, "Virtual VCR Error", MB_OK|MB_ICONEXCLAMATION|MB_TASKMODAL);
    return FALSE;
}

//
// Version Info 
//
int GetVersionInfo(char *versionTEXT, bool longVersion)
{
	TCHAR fileName[_MAX_PATH+1];
	GetModuleFileName(NULL, fileName, _MAX_PATH);

	DWORD dwVerHnd;
	DWORD dwVerInfoSize;
	int ret = 0;

	dwVerInfoSize = GetFileVersionInfoSize(fileName, &dwVerHnd);

	if (dwVerInfoSize)
	{
		// If we were able to get the information, process it:
		HANDLE hMem;
		LPVOID lpvMem;
		BOOL fRet;
		UINT cchVer = 0;
		VS_FIXEDFILEINFO *vInfo;
		
		hMem = GlobalAlloc(GMEM_MOVEABLE, dwVerInfoSize);
		lpvMem = GlobalLock(hMem);

		GetFileVersionInfo(fileName, NULL, dwVerInfoSize, lpvMem);

		fRet = VerQueryValue(lpvMem, TEXT("\\"), (LPVOID*)&vInfo, &cchVer);

		if (fRet && cchVer && vInfo)
		{
            WORD maj = HIWORD(vInfo->dwFileVersionMS);
			WORD min = LOWORD(vInfo->dwFileVersionMS);
			WORD rev = HIWORD(vInfo->dwFileVersionLS);
			WORD bld = LOWORD(vInfo->dwFileVersionLS);
			ret = bld;
			if(longVersion)
				_stprintf(versionTEXT, TEXT("Version : %d.%d.%d  Build:%d"), maj, min, rev, bld);
			else
				_stprintf(versionTEXT, TEXT("%d.%d.%d.%d"), maj, min, rev, bld);
		}

		GlobalUnlock(hMem);
		GlobalFree(hMem);
	}
	else
	{
		ret = 0;
		if(longVersion)
			_stprintf(versionTEXT, TEXT("Version : Not Available"));
		else
			_stprintf(versionTEXT, TEXT("N/A"));
			
	}
	return ret;
}

//
// Dialog Procedure for the "about" dialog box.
//
BOOL CALLBACK AboutDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) 
{
	HBRUSH g_hbrBackground = CreateSolidBrush(RGB(0, 0, 0));
    switch(msg)
	{
        case WM_COMMAND:
		{
            EndDialog(hwnd, TRUE);
            return TRUE;
		}
        case WM_INITDIALOG:
		{
			gcap.aboutHWND = hwnd;

			char verText[256];
			int ver = GetVersionInfo(verText, true);
			SetDlgItemText(hwnd, IDC_VERSION_INFO, verText);

            return TRUE;
		}

		case WM_CTLCOLORDLG:
		{
			return (LONG)g_hbrBackground;
		}

		case WM_CTLCOLORSTATIC:
		{
			HDC hdcStatic = (HDC)wParam;
			SetTextColor(hdcStatic, RGB(255, 255, 255));
			SetBkMode(hdcStatic, TRANSPARENT);
			return (LONG)g_hbrBackground;
		}

    }
    return FALSE;
}

/*
 * Put up the open file dialog
 */
BOOL OpenFileDialog(HWND hWnd, LPTSTR pszName, int cb) 
{
    OPENFILENAME ofn;
    LPTSTR p;
    TCHAR        szFileName[_MAX_PATH];
    TCHAR        szBuffer[_MAX_PATH] ;

    if(pszName == NULL || cb <= 0)
        return FALSE;

    // start with capture file as current file name
    szFileName[0] = 0;
    lstrcpy(szFileName, appData->szCaptureFile);

    // Get just the path info
    // Terminate the full path at the last backslash
    lstrcpy(szBuffer, szFileName);
    for(p = szBuffer + lstrlen(szBuffer); p > szBuffer; p--)
	{
        if(*p == '\\')
		{
            *(p+1) = '\0';
            break;
        }
    }

    _fmemset(&ofn, 0, sizeof(OPENFILENAME)) ;
    ofn.lStructSize = sizeof(OPENFILENAME) ;
    ofn.hwndOwner = hWnd ;
    ofn.lpstrFilter = TEXT("AVI file\0*.avi\0\0");
    ofn.nFilterIndex = 0 ;
    ofn.lpstrFile = szFileName;
    ofn.nMaxFile = sizeof(szFileName) ;
    ofn.lpstrFileTitle = NULL;
    ofn.lpstrTitle = TEXT("Set Capture File");
    ofn.nMaxFileTitle = 0;
    ofn.lpstrInitialDir = szBuffer;
	ofn.lpstrDefExt = TEXT("avi");
    ofn.Flags = OFN_HIDEREADONLY | OFN_NOREADONLYRETURN | OFN_PATHMUSTEXIST ;

    if(GetOpenFileName(&ofn))
	{
        // We have a capture file name
        lstrcpyn(pszName, szFileName, cb);
        return TRUE;
    }
    else
	{
        return FALSE;
    }
}


/*
 * Open Save Dialog
 */
BOOL OpenSaveDialog(HWND hWnd, LPTSTR pszName, int cb) 
{
    OPENFILENAME ofn;
    TCHAR        szFileName[_MAX_PATH];
    TCHAR        szBuffer[_MAX_PATH] ;

    if(pszName == NULL || cb <= 0)
        return FALSE;

    szFileName[0] = 0;

    _fmemset(&ofn, 0, sizeof(OPENFILENAME)) ;
    ofn.lStructSize = sizeof(OPENFILENAME) ;
    ofn.hwndOwner = hWnd ;
    ofn.lpstrFilter = TEXT("VCR Profile\0*.vcr\0\0");
    ofn.nFilterIndex = 0 ;
    ofn.lpstrFile = szFileName;
    ofn.nMaxFile = sizeof(szFileName) ;
    ofn.lpstrFileTitle = NULL;
    ofn.lpstrTitle = TEXT("Save Profile to File");
    ofn.nMaxFileTitle = 0;
    ofn.lpstrInitialDir = szBuffer;
	ofn.lpstrDefExt = TEXT("vcr");
    ofn.Flags = OFN_HIDEREADONLY | OFN_NOREADONLYRETURN | OFN_PATHMUSTEXIST ;

    if(GetSaveFileName(&ofn))
	{
        // We have a capture file name
        lstrcpyn(pszName, szFileName, cb);
        return TRUE;
    }
    else
	{
        return FALSE;
    }
}


/*
 * Open Load Dialog
 */
BOOL OpenLoadDialog(HWND hWnd, LPTSTR pszName, int cb) 
{
    OPENFILENAME ofn;
    TCHAR        szFileName[_MAX_PATH];
    TCHAR        szBuffer[_MAX_PATH] ;

    if(pszName == NULL || cb <= 0)
        return FALSE;

    szFileName[0] = 0;

    _fmemset(&ofn, 0, sizeof(OPENFILENAME));
    ofn.lStructSize = sizeof(OPENFILENAME);
    ofn.hwndOwner = hWnd ;
    ofn.lpstrFilter = TEXT("VCR Profile\0*.vcr\0\0");
    ofn.nFilterIndex = 0 ;
    ofn.lpstrFile = szFileName;
    ofn.nMaxFile = sizeof(szFileName) ;
    ofn.lpstrFileTitle = NULL;
    ofn.lpstrTitle = TEXT("Load Profile from File");
    ofn.nMaxFileTitle = 0;
    ofn.lpstrInitialDir = szBuffer;
	ofn.lpstrDefExt = TEXT("vcr");
    ofn.Flags = OFN_HIDEREADONLY | OFN_NOREADONLYRETURN | OFN_PATHMUSTEXIST ;

    if(GetOpenFileName(&ofn))
	{
        // We have a capture file name
        lstrcpyn(pszName, szFileName, cb);
        return TRUE;
    }
    else
	{
        return FALSE;
    }
}

/*
 * Put up a dialog to allow the user to select a capture file.
 */
BOOL SetCaptureFile(HWND hWnd) 
{
    USES_CONVERSION;

    if(OpenFileDialog(hWnd, appData->szCaptureFile, _MAX_PATH))
	{

#ifdef UNICODE
        // Convert Multibyte string to ANSI
        char szCaptureFile[STR_MAX_LENGTH];
        int rc = WideCharToMultiByte(CP_ACP, 0, gcap.szCaptureFile, -1, 
            szCaptureFile, STR_MAX_LENGTH, NULL, NULL);
#else
        TCHAR *szCaptureFile = appData->szCaptureFile;
#endif
		
    }
    else
	{
        return FALSE;
    }

    return TRUE;
}

// brings up a dialog box
//
int DoDialog(HWND hwndParent, int DialogID, DLGPROC fnDialog, long lParam) 
{
    DLGPROC fn;
    int result;

    fn = (DLGPROC)MakeProcInstance(fnDialog, ghInstApp);
    result = (int) DialogBoxParam(ghInstApp,
        MAKEINTRESOURCE(DialogID),
        hwndParent,
        fn,
        lParam);
    FreeProcInstance(fn);

    return result;
}

// brings up a modelass dialog box
//
HWND DoDialogModeless(HWND hwndParent, int DialogID, DLGPROC fnDialog, long lParam)  
{
    HWND hwndGoto = NULL;

	hwndGoto = CreateDialog(ghInstApp, 
		MAKEINTRESOURCE(DialogID), 
		hwndParent, (DLGPROC)fnDialog); 

	ShowWindow(hwndGoto, SW_SHOW);

    return hwndGoto;
}

// brings up a modelass dialog box
//
HWND ShowHelpPOPup(HWND hwndParent, CHAR *helpText, int x, int y, int numberChars) 
{
    HWND hwndHELP = NULL;

	hwndHELP = CreateDialog(
		ghInstApp, 
		MAKEINTRESOURCE(IDD_HELP_POPUP), 
		hwndParent,
		(DLGPROC)HelpDlgProc);

	SetDlgItemText(hwndHELP, IDC_HELP_TEXT, helpText);

	HWND text = GetDlgItem(hwndHELP, IDC_HELP_TEXT);

	HDC hdc = GetDC(text);
	TEXTMETRIC lptm; 
	GetTextMetrics(hdc, (LPTEXTMETRIC)&lptm);

	//MsgBox(TEXT("%d x %d"), lptm.tmHeight, lptm.tmAveCharWidth);

	int charsPerLine = 30;

	int width = (charsPerLine * lptm.tmAveCharWidth);
	// Add 20 as we are sizing the dialog here not the text window
	width += 20;

	int height = ((numberChars / charsPerLine) + 1) * lptm.tmHeight;
	// Add 10 as we are sizing the dialog here not the text window
	height += 10;

	width = 250;
	height = 150;

	if(x > 0 && y > 0)
	{
		SetWindowPos(
			hwndHELP, // handle to window
			HWND_TOP,  // placement-order handle
			x - (width / 2), // horizontal position
			y - (height / 2), // vertical position
			width, // width
			height, // height
			SWP_SHOWWINDOW  // window-positioning options
			);
	}
	else
	{
		SetWindowPos(
			hwndHELP, // handle to window
			HWND_TOP,  // placement-order handle
			0, // horizontal position
			0, // vertical position
			width, // width
			height, // height
			SWP_SHOWWINDOW | SWP_NOMOVE  // window-positioning options
			);
	}

    return hwndHELP;
}

int FAR PASCAL ScanProgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) 
{
    switch(msg)
	{
        case WM_COMMAND:
		{
			switch(wParam)
			{
				case IDC_STOPSCAN:
				case IDCANCEL:
					gcap.channelScanning = FALSE;

					EndDialog(hwnd, FALSE);
					break;

				case IDC_STARTSCAN:
				{
					int aggressive = IsDlgButtonChecked(hwnd, IDC_AGGRESSIVESCAN);

					if(aggressive == 1)
					{
						int sure = MessageBox(
							ghwndApp,
							"Doing an aggressive channel scan for TV channels can\r\n" \
							"sometimes return channels that are not tuned properly, this \r\n" \
							"option may return all the channels available to your tuner even \r\n" \
							"if they do not have a signal!\r\n\r\n" \
							"Are you sure you want to do an aggressive scan?",
							"Do Aggressive Scan",
							MB_YESNO|MB_ICONINFORMATION|MB_TASKMODAL);
						
						if(sure != IDYES)
							aggressive = 0;
					}

					EnableWindow(GetDlgItem(hwnd, IDC_STARTSCAN), FALSE);
					EnableWindow(GetDlgItem(hwnd, IDC_AGGRESSIVESCAN), FALSE);

					SendMessage(GetDlgItem(hwnd, IDC_STOPSCAN),
						WM_SETTEXT,
						0, 
						(LPARAM)"ABORT");

					_beginthread( scannForTVChannels, 0, (void *)(aggressive) );

					break;
				}
			}
			break;

            return TRUE;
		}
        case WM_INITDIALOG:
		{
			SetDlgItemText(hwnd, IDC_SCANPROG, "Ready To Scan");
			gcap.channelScanningStatus = hwnd;

            return TRUE;
		}
    }
    return FALSE;

}


BOOL CALLBACK HelpDlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) 
{
	HBRUSH g_hbrBackground = CreateSolidBrush(RGB(255, 255, 238));

    switch(msg)
	{
        case WM_COMMAND:
		{
			switch(wParam)
			{
				case IDC_CLOSEHELP:
					EndDialog(hwnd, FALSE);
					break;
			}
			break;

            return TRUE;
		}
        case WM_INITDIALOG:
		{
            return TRUE;
		}

		case WM_CTLCOLORDLG:
		{
			return (LONG)g_hbrBackground;
		}

		case WM_CTLCOLORSTATIC:
		{
			HDC hdcStatic = (HDC)wParam;
			SetTextColor(hdcStatic, RGB(0, 0, 0));
			SetBkMode(hdcStatic, TRANSPARENT);
			return (LONG)g_hbrBackground;
		}

        case WM_SIZE:
        {
            RECT rc;

            GetClientRect(hwnd, &rc);

            MoveWindow(GetDlgItem(hwnd, IDC_HELP_TEXT),    // get child window handle
                rc.left + 10,
				rc.top + 20,
                (rc.right - rc.left) - 20,
                (rc.bottom - rc.top) - 25,
                TRUE);

            MoveWindow(GetDlgItem(hwnd, IDC_CLOSEHELP),    // get child window handle
                rc.right - 20,
				rc.top + 2,
                18,
                18,
                TRUE);

            break;
        }
    }
    return FALSE;
}

int GetHelpText(int helpID, CHAR *helpData, int sizeOfBuffer)
{
	LPSTR pathInfo = new char[_MAX_PATH+1]; 

	GetCurrentDirectory(_MAX_PATH, pathInfo);

	TCHAR *helpFile = new TCHAR[_MAX_PATH + 50];
	wsprintf(helpFile, TEXT("%s\\%s"), pathInfo, "hlpcntx.dat");

	delete[] pathInfo;

	TCHAR helpIDTEXT[5];
	wsprintf(helpIDTEXT, TEXT("%d"), helpID);

	TCHAR noHelp[64];
	wsprintf(noHelp, TEXT("No help Available for : %d"), helpID);

	int charCount = GetPrivateProfileString(
		TEXT("HelpCNTX"), helpIDTEXT, noHelp,
		helpData, sizeOfBuffer-1, helpFile);

	//MsgBox(TEXT("%d"), charCount);

	for(int x = 0; x < charCount-1; x++)
	{
		if(helpData[x] == '<' && helpData[x+1] == '-')
		{
			helpData[x] = '\r';
			helpData[x+1] = '\n';
			x++;
		}
	}

	return charCount;
}


//
// GetFreeDiskSpace: Function to Measure Available Disk Space
//
static long GetFreeDiskSpaceInKB(LPTSTR pFile) 
{
    DWORD dwFreeClusters, dwBytesPerSector, dwSectorsPerCluster, dwClusters;
    TCHAR RootName[MAX_PATH];
    LPTSTR ptmp;    //required arg
    ULARGE_INTEGER ulA, ulB, ulFreeBytes;

    // need to find path for root directory on drive containing
    // this file.

    GetFullPathName(pFile, sizeof(RootName)/sizeof(RootName[0]), RootName, &ptmp);

    // truncate this to the name of the root directory (how tedious)
    if(RootName[0] == '\\' && RootName[1] == '\\') {

        // path begins with  \\server\share\path so skip the first
        // three backslashes
        ptmp = &RootName[2];
        while(*ptmp && (*ptmp != '\\')) {
            ptmp++;
        }
        if(*ptmp) {
            // advance past the third backslash
            ptmp++;
        }
    }
    else {
        // path must be drv:\path
        ptmp = RootName;
    }

    // find next backslash and put a null after it
    while(*ptmp && (*ptmp != '\\')) {
        ptmp++;
    }
    // found a backslash ?
    if(*ptmp) {
        // skip it and insert null
        ptmp++;
        *ptmp = '\0';
    }

    // the only real way of finding out free disk space is calling
    // GetDiskFreeSpaceExA, but it doesn't exist on Win95

    HINSTANCE h = LoadLibrary(TEXT("kernel32.dll"));
    if(h) {
        typedef BOOL(WINAPI *MyFunc)(LPCTSTR RootName, PULARGE_INTEGER pulA, PULARGE_INTEGER pulB, PULARGE_INTEGER pulFreeBytes);

#ifdef UNICODE
        MyFunc pfnGetDiskFreeSpaceEx = (MyFunc)GetProcAddress(h, "GetDiskFreeSpaceExW");
#else
        MyFunc pfnGetDiskFreeSpaceEx = (MyFunc)GetProcAddress(h, "GetDiskFreeSpaceExA");
#endif
        FreeLibrary(h);

        if(pfnGetDiskFreeSpaceEx) {
            if(!pfnGetDiskFreeSpaceEx(RootName, &ulA, &ulB, &ulFreeBytes))
                return -1;
            else
                return (long)(ulFreeBytes.QuadPart / 1024);
        }
    }

    if(!GetDiskFreeSpace(RootName, &dwSectorsPerCluster, &dwBytesPerSector,
        &dwFreeClusters, &dwClusters))
        return (-1);
    else
        return(MulDiv(dwSectorsPerCluster * dwBytesPerSector,
            dwFreeClusters,
            1024));
}

int FAR PASCAL CompCapabilityProc(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{

    switch(msg)
	{
	case WM_INITDIALOG:
		{
			// connect the audio compression codec to the audio source
			// to enable the format selection

			IAMStreamConfig *caps = NULL;

			if(gcap.pBuilder == NULL || appData->AudioCodec->getCurrentCodec() == NULL)
				break;

			gcap.pBuilder->FindInterface(NULL,
				&MEDIATYPE_Audio, appData->AudioCodec->getCurrentCodec(), 
				IID_IAMStreamConfig, (void **)&caps);

			if(!caps)
			{
				ErrMsg(TEXT("Could not get Stream Config interface for Codec"));
				break;
			}

			int number = 0;
			int sizeOfStruc = 0;
			caps->GetNumberOfCapabilities(&number, &sizeOfStruc);

			if(number < 1)
			{
				ErrMsg(TEXT("No Capabilities to display"));
				if(caps)
					caps->Release();
				break;
			}

			AM_MEDIA_TYPE *pmt = NULL;
			unsigned char *pSCC = new unsigned char[sizeOfStruc];

			TCHAR  tach[32];
			SendDlgItemMessage(hwnd, IDC_CAPABILITIES, LB_RESETCONTENT, 0, 0);

			for(int x = 0; x < number; x++)
			{
				HRESULT hr = caps->GetStreamCaps(x, &pmt, pSCC);
				if(hr == S_FALSE)
					ErrMsg(TEXT("S_FALSE"));
				if(hr == E_INVALIDARG)
					ErrMsg(TEXT("E_INVALIDARG"));
				if(hr == E_OUTOFMEMORY)
					ErrMsg(TEXT("E_OUTOFMEMORY"));
				if(hr == E_POINTER)
					ErrMsg(TEXT("E_POINTER"));
				if(hr == E_POINTER)
					ErrMsg(TEXT("E_POINTER"));
				if(hr == VFW_E_NOT_CONNECTED)
					ErrMsg(TEXT("VFW_E_NOT_CONNECTED"));

				if(pmt)
				{
					WAVEFORMATEX *wf = (WAVEFORMATEX *)pmt->pbFormat;
					
					char *channels = "Stereo";
					if(wf->nChannels == 1)
						channels = "Mono";

					_stprintf(tach,
						TEXT("%d : %dKbs - %d - %s"),
						x,
						(wf->nAvgBytesPerSec * 8) / 1000,
						wf->nSamplesPerSec,
						channels);

					SendDlgItemMessage(hwnd, IDC_CAPABILITIES, LB_ADDSTRING, 0, (LPARAM)tach);
				}

			}
			if(appData->Audio_Capability_Index > -1)
				SendDlgItemMessage(hwnd, IDC_CAPABILITIES, LB_SETCURSEL, (WPARAM)appData->Audio_Capability_Index, (LPARAM)0);

			if(caps)
				caps->Release();

		break;
		}
	case WM_COMMAND:
		switch(wParam)
		{
		case IDCANCEL:
			EndDialog(hwnd, FALSE);
			break;
		case IDOK:
			{
				int selectedCompat = -1;
				selectedCompat = SendDlgItemMessage(hwnd, IDC_CAPABILITIES, LB_GETCURSEL, 0, 0);

				if(selectedCompat > -1)
					appData->Audio_Capability_Index = selectedCompat;

				EndDialog(hwnd, TRUE);
				break;
			}
		}
		break;
		

        default:
            return FALSE;
    }

    return TRUE;
}

int FAR PASCAL DroppedFrameProc(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{
    TCHAR tach[10000];
	TCHAR temp[100];

    switch(msg)
	{
	case WM_INITDIALOG:
		{
			strcpy(tach, "Dropped Frames\r\n");

			if(gcap.droppedList)
				{
					for(int x = 0; x < 5000; x++)
					{
						if(gcap.droppedList[x] == 0) break;
						// for some reason the transferm count is alway 2 more 
						// then the actual frame number so -2 will fix this
						wsprintf(temp, TEXT("%d\r\n"), gcap.droppedList[x]-2);
						strcat(tach, temp);

					}
				}


		SetDlgItemText(hwnd, IDC_DroppedFrameList, tach);
		break;
		}
	case WM_COMMAND:
		switch(wParam)
		{
		case IDCANCEL:
			EndDialog(hwnd, FALSE);
			break;
		case IDOK:
			EndDialog(hwnd, TRUE);
			break;
		}
		break;
		

        default:
            return FALSE;
    }
    return TRUE;
}

//
// Capture Stats Proc function
//
int FAR PASCAL CaptureStatsProc(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{
    //TCHAR tach[_MAX_PATH];
	
    switch(msg)
	{
	case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
	case WM_INITDIALOG:
		{
			//wsprintf(tach, TEXT("%s"), "Hello");
			//SetDlgItemText(hwnd, IDC_Frames, tach);
			
			SetTimer(hwnd, 1, 1000, NULL);
			break;
		}
		
	case WM_COMMAND:
		{
			switch(wParam)
			{
				case IDCANCEL:
					EndDialog(hwnd, FALSE);
					break;
					
				case IDOK:
					EndDialog(hwnd, TRUE);
					break;	
			}
			break;
		}
		
		case WM_CLOSE:
			{
				appData->fWantCaptureStats = false;
				ShowCapStats(appData->fWantCaptureStats);
				break;
			}
			
		case WM_MOVE:
			{
				RECT rcW;
				GetWindowRect(hwnd, &rcW);
				int offsetX = rcW.left - appData->iApplicationWindowX;
				int offsetY = rcW.top - appData->iApplicationWindowY;
				
				if(appData->iApplicationWindowX + offsetX > 0)
					appData->iCaptureStatsWindowX = offsetX;
				
				if(appData->iApplicationWindowY + offsetY > 0)
					appData->iCaptureStatsWindowY = offsetY;
				
				return 0;
			}
			
		case WM_TIMER:
			{
				HWND currentWindow = GetForegroundWindow();
				if(hwnd == currentWindow)
					BringWindowToTop(ghwndApp);
				return (LRESULT) 1;
			}
			
			
        default:
            return FALSE;
    }
    return TRUE;
}


DWORDLONG GetSize(LPCTSTR tach) 
{
    HANDLE hFile = CreateFile(tach, GENERIC_READ, FILE_SHARE_READ, 0,
        OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

    if(hFile == INVALID_HANDLE_VALUE) {
        return 0;
    }

    DWORD dwSizeHigh;
    DWORD dwSizeLow = GetFileSize(hFile, &dwSizeHigh);

    DWORDLONG dwlSize = dwSizeLow + ((DWORDLONG)dwSizeHigh << 32);

    if(!CloseHandle(hFile)) {
        dwlSize = 0;
    }

    return dwlSize;
}

void OnClose() 
{
	if(gcap.log) logAction(TEXT("OnClose() - enter"));

	// Before we release the cap filters get the
	// current settings if any
	getTunerSettings();
	getVideoAmpSettings();

    StopPreview();
    StopCapture();
    TearDownGraph();
    FreeCapFilters();
	
	// Save the persistand data in the INI file
	// then delete the appData object
	BOOL saved = appData->saveSessionData();
	if(!saved)
		ErrMsg(TEXT("Error saving to the INI File"));

	delete appData;

	if(gcap.log) logAction(TEXT("OnClose() - exit"));
}
/*
void GetCurrentDevices() 
{
	// Get the display names of the devices and store them in the appData object
    WCHAR *szDisplayName = NULL;
    szDisplayName = 0;
    if(gcap.pmVideo)
	{
        if(SUCCEEDED(gcap.pmVideo->GetDisplayName(0, 0, &szDisplayName)))
		{   
            if(szDisplayName)
			{
                wsprintf(appData->szVideoDisplayName, TEXT("%S"), szDisplayName);
                CoTaskMemFree(szDisplayName);
            }

        }
    }

    szDisplayName = 0;
    if(gcap.pmAudio)
	{
        if(SUCCEEDED(gcap.pmAudio->GetDisplayName(0, 0, &szDisplayName)))
		{
            if(szDisplayName)
			{
                wsprintf(appData->szAudioDisplayName, TEXT("%S"), szDisplayName);
                CoTaskMemFree(szDisplayName);
            }

        }
    }
}
*/
// Adds a DirectShow filter graph to the Running Object Table,
// allowing GraphEdit to "spy" on a remote filter graph.
HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
{
    IMoniker * pMoniker;
    IRunningObjectTable *pROT;
    WCHAR wsz[128];
    HRESULT hr;

    if (FAILED(GetRunningObjectTable(0, &pROT)))
        return E_FAIL;

    wsprintfW(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, 
              GetCurrentProcessId());

    hr = CreateItemMoniker(L"!", wsz, &pMoniker);
    if (SUCCEEDED(hr)) 
    {
        hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
        pMoniker->Release();
    }
    pROT->Release();
    return hr;
}

// Removes a filter graph from the Running Object Table
void RemoveGraphFromRot(DWORD pdwRegister)
{
    IRunningObjectTable *pROT;

    if (SUCCEEDED(GetRunningObjectTable(0, &pROT))) 
    {
        pROT->Revoke(pdwRegister);
        pROT->Release();
    }
}

/*
Parses the command line args
*/
BOOL ParseCommand(LPSTR szCmdLine, char *value, char *param)
{
	// This method will pase a command line option and return
	// a bool of true if it was found or false if it was not.
	
	// It will also copy any parameter info into a new char array
	// so you can use it elsewere.
	
	// format of the command line parameter is as follows.
	
	// virtualvcr.exe <parameter01> "<data01>" <parameter02> "<data02>"
	// virtualvcr.exe -capture -channel "3"
	
	// Copy the command line parameter
	// so we dont stuff it up with the 
	// string tokenizer
	char *cmd = new char[1024];
	char *pch;
	int offSet = 0;
	BOOL start = false;
	BOOL found = false;
	int endOfParameter = 0;

	try
	{
		strcpy(cmd, szCmdLine);

		pch = strtok (cmd, " ");
		while (pch != NULL)
		{
			if(strcmp(pch, param) == 0)
			{
				//Found the option so we will return true
				found = true;
				
				// If we do not want to read in a value then return
				if(value == NULL) break;

				//Loop through all characters and only include one
				//inside quotes 
				for(int charAt = 0; charAt < 265; charAt++)
				{
					// only search for 2 characters after the end of the
					// parameter name.
					if(!start && pch[charAt] == ' ' && endOfParameter == 0) endOfParameter = charAt;
					if(endOfParameter > 0 && !start && charAt > 2) break;
					
					//Save the data to a new char aray
					if(pch[charAt] == '"' && !start) start = true;
					else if(pch[charAt] == '"' && start) break;
					else if(start) value[offSet++] = pch[charAt];
				}
				//Exit after the first occurance of the paramter
				//we are after
				break;
			}
			
			pch = strtok (NULL, " ");
		}
		
		//Terminate the string will null
		if(value != NULL) value[offSet] = 0;
	}
	catch(...)
	{
		ErrMsg(TEXT("An error occured when parsing the command line argument: %s"), param);
		return false;
	}

	delete []cmd;
	return found;
}


/*
	Format byte number for display groups
	000,000,000,000 etc
*/
BOOL FormatNumber(LONGLONG number, TCHAR* buff)
{
	TCHAR tempBuff[60];
	char intBuff[3];
	int bufferOffset = 0;

	int LowNumber = 0;
	while(number > 0)
	{
		LowNumber = number % 10;
		_stprintf(intBuff, TEXT("%.1d"), LowNumber);
		tempBuff[bufferOffset++] = intBuff[0];
		if(number > 10 && (bufferOffset == 3 || bufferOffset == 7 || bufferOffset == 11 || bufferOffset == 15))
			tempBuff[bufferOffset++] = ',';
		number = number / 10;
	}

	bufferOffset--;

	for(int x = 0; x <= bufferOffset; x++)
	{
		buff[x] = tempBuff[bufferOffset - x];
	}

	buff[bufferOffset+1] = 0;

	return true;
}

// Show a loader Dialog while initialising stuff

BOOL ShowLoaderDialog(BOOL show)
{
	// Show Cap Stats
	if(show)
	{
		if(!IsWindow(hwndLOADER))
		{ 
			hwndLOADER = CreateDialog(ghInstApp, 
				MAKEINTRESOURCE(IDD_ABOUT), 
				ghwndApp,
				(DLGPROC) AboutDlgProc);
						
			ShowWindow(hwndLOADER, SW_SHOW); 
		}
	} 
	else
	{
		if(IsWindow(hwndLOADER))
		{ 
			DestroyWindow(hwndLOADER); 
			hwndLOADER = NULL;
		}
	}
	return true;
}

BOOL ShowCapStats(BOOL show)
{
	// Show Cap Stats
	if(show)
	{
		if(!IsWindow(hwndSTATUS))
		{ 
			hwndSTATUS = CreateDialog(ghInstApp,
				MAKEINTRESOURCE(IDD_CaptureStats),
				ghwndApp,
				(DLGPROC) CaptureStatsProc);

			int newXlocation = appData->iApplicationWindowX + appData->iCaptureStatsWindowX;
			int newYlocation = appData->iApplicationWindowY + appData->iCaptureStatsWindowY;
			if(newXlocation < 0 || newXlocation > 5000) newXlocation = 0;
			if(newYlocation < 0 || newYlocation > 5000) newYlocation = 0;
			
			SetWindowPos(
				hwndSTATUS, // handle to window
				HWND_TOP,  // placement-order handle
				newXlocation, // horizontal position
				newYlocation, // vertical position
				NULL, // width
				NULL, // height
				SWP_NOSIZE  // window-positioning options
				);

			ShowWindow(hwndSTATUS, SW_SHOW);
			BringWindowToTop(ghwndApp);
		}
	} 
	else
	{
		DestroyWindow(hwndSTATUS); 
		hwndSTATUS = NULL;     
	}

	if(gcap.toolBarHWND != NULL)
	{
		SendMessage(
			gcap.toolBarHWND,
			TB_CHECKBUTTON,
			(WPARAM) MENU_SHOWSTATS,	//button number
			(LPARAM) MAKELONG (appData->fWantCaptureStats, 0)
			);
	}

	return true;
}

BOOL TryToStartCapture()
{
	if(gcap.log) logAction(TEXT("TryToStartCapture()"));

	// Work out if we have enough space on the drive to start capturing
	// You need at least 10 meg
	long freespace = GetFreeDiskSpaceInKB(appData->szCaptureFile) / 1024;
	if(freespace < (appData->dwFreeSpaceLimit + 10))
	{
		ErrMsg(TEXT("Not enough free space after stop condition to start capture"));
		return false;
	}
	else if(freespace  < 10)
	{
		ErrMsg(TEXT("Not enough free space to start capture"));
		return false;
	}
	
	//Now Do capture
	if(gcap.fPreviewing)
		StopPreview();

	// tear down the graph and build a capture graph
	// even if we aleardy have one
	TearDownGraph();
	BuildCaptureGraph();

	StartCapture();
	
	return true;
}

void setVideoAmpSettings()
{
	if(gcap.log) logAction(TEXT("setVideoAmpSettings()"));

	HRESULT hr;
	IAMVideoProcAmp *pAMP = NULL; 
	
	if(gcap.pBuilder == NULL || gcap.pVCap == NULL)
		return;
	
	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap,
		IID_IAMVideoProcAmp, (void **)&pAMP);
	
	if(!pAMP)
		return;

	if(appData->Video_Brit > -1)
		hr = pAMP->Set(VideoProcAmp_Brightness ,appData->Video_Brit, VideoProcAmp_Flags_Manual);

	if(appData->Video_Cont > -1)
		hr = pAMP->Set(VideoProcAmp_Contrast ,appData->Video_Cont, VideoProcAmp_Flags_Manual);

	if(appData->Video_Hue > -1)
		hr = pAMP->Set(VideoProcAmp_Hue ,appData->Video_Hue, VideoProcAmp_Flags_Manual);

	if(appData->Video_Sat > -1)
		hr = pAMP->Set(VideoProcAmp_Saturation ,appData->Video_Sat, VideoProcAmp_Flags_Manual);

	hr = pAMP->Set(VideoProcAmp_ColorEnable ,appData->Video_Colour, VideoProcAmp_Flags_Manual);

	if(appData->Video_Gamma > -1)
		hr = pAMP->Set(VideoProcAmp_Gamma ,appData->Video_Gamma, VideoProcAmp_Flags_Manual);

	if(appData->Video_Sharp > -1)
		hr = pAMP->Set(VideoProcAmp_Sharpness ,appData->Video_Sharp, VideoProcAmp_Flags_Manual);
	
	pAMP->Release();
	pAMP = NULL;
}

void getVideoAmpSettings()
{
	if(gcap.log) logAction(TEXT("getVideoAmpSettings()"));

	HRESULT hr;
	IAMVideoProcAmp *pAMP = NULL; 
	
	if(gcap.pBuilder == NULL || gcap.pVCap == NULL)
		return;
	
	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap,
		IID_IAMVideoProcAmp, (void **)&pAMP);
	
	if(!pAMP)
		return;
	
	long bright = -1;
	long cont = -1;
	long hue = -1;
	long sat = -1;
	long colour = 1;
	long gamma = -1;
	long sharp = -1;
	
	long flags = 0;
	//VideoProcAmp_Flags_Auto
	//VideoProcAmp_Flags_Manual
	
	hr = pAMP->Get(VideoProcAmp_Brightness ,&bright, &flags);
	if(flags == VideoProcAmp_Flags_Auto) bright = -1;

	hr = pAMP->Get(VideoProcAmp_Contrast ,&cont, &flags);
	if(flags == VideoProcAmp_Flags_Auto) cont = -1;

	hr = pAMP->Get(VideoProcAmp_Hue ,&hue, &flags);
	if(flags == VideoProcAmp_Flags_Auto) hue = -1;

	hr = pAMP->Get(VideoProcAmp_Saturation ,&sat, &flags);
	if(flags == VideoProcAmp_Flags_Auto) sat = -1;

	hr = pAMP->Get(VideoProcAmp_ColorEnable ,&colour, &flags);
	if(flags == VideoProcAmp_Flags_Auto) colour = -1;

	hr = pAMP->Get(VideoProcAmp_Gamma ,&gamma, &flags);
	if(flags == VideoProcAmp_Flags_Auto) gamma = -1;

	hr = pAMP->Get(VideoProcAmp_Sharpness ,&sharp, &flags);
	if(flags == VideoProcAmp_Flags_Auto) sharp = -1;
	
	appData->Video_Brit = bright;
	appData->Video_Cont = cont;
	appData->Video_Hue = hue;
	appData->Video_Sat = sat;
	appData->Video_Colour = colour;
	appData->Video_Gamma = gamma;
	appData->Video_Sharp = sharp;
	
	pAMP->Release();
	pAMP = NULL;
}

void getTunerSettings()
{
	if(gcap.log) logAction(TEXT("getTunerSettings()"));

	if(gcap.pBuilder == NULL || gcap.pVCap == NULL)
		return;

	HRESULT hr;
	IAMTVTuner *pTV = NULL;
	IAMAnalogVideoDecoder *pAD = NULL;

	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap,
		IID_IAMTVTuner, (void **)&pTV);

	if(!pTV) return;

	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap,
		IID_IAMAnalogVideoDecoder, (void **)&pAD);

	if(!pAD)
	{
		pTV->Release();
		return;
	}

	// Get the Country Setting
	long countrySetting = 0;
	pTV->get_CountryCode(&countrySetting);
	appData->tunerCountryCode = countrySetting;

	// Get the input Type ant/cable
	TunerInputType inputType;
	pTV->get_InputType(0, &inputType);
	if(inputType == TunerInputAntenna)
		appData->tunerInputType = 0;
	else if(inputType == TunerInputCable)
		appData->tunerInputType = 1;

	LONG TunerMode = 0;
	pAD->get_TVFormat(&TunerMode);
	appData->tunerMode = TunerMode;

	pTV->Release();
	pAD->Release();

	//MsgBox(TEXT("Mode %d"), appData->tunerMode);
}

void logData(LONG lTime, double videoRate, double audioRate, double AVdiff, double AVsync, LONG lDropped, LONG droppedFrames, BOOL lastUpdate)
{
	TCHAR data[256];

	if(!gcap.logFile.is_open())
	{
		gcap.logFile.open("AVdiff.txt");
		gcap.logFile<<"Time, Video, Audio, Diff, Sync%, Dropped01, Dropped02\n";
	}

	if(!gcap.logFile.is_open())
		return;

	_stprintf(data, "%d, %f, %f, %f, %f, %d, %d\n", 
		lTime,
		videoRate,
		audioRate,
		AVdiff,
		AVsync,
		lDropped,
		droppedFrames);

	gcap.logFile<<data;

	if(lastUpdate)
		gcap.logFile.close();

}

void logAction(LPTSTR sz,...)
{
    static TCHAR tach[2000];
    va_list va;
    va_start(va, sz);
    wvsprintf(tach, sz, va);
    va_end(va);

	if(!gcap.actionLog.is_open())
	{
		gcap.actionLog.open("actionLog.txt");
		gcap.actionLog<<"Log File Started for ";

		char verText[256];
		int ver = GetVersionInfo(verText, true);
		gcap.actionLog<<verText;
		gcap.actionLog<<"\n";
		gcap.actionLog.flush();
	}

	if(!gcap.actionLog.is_open())
		return;

	// get the time stamp from the system time
	struct tm *newtime;
	time_t long_time;
	time(&long_time);
	newtime = localtime(&long_time);
	CHAR timeStamp[256];
	
	// Rebuild the file name with the date/time stamp in it
	wsprintf(timeStamp, "%.4d-%.2d-%.2d at %.2d-%.2d-%.2d - ",
		newtime->tm_year+1900,
		newtime->tm_mon+1,
		newtime->tm_mday,
		newtime->tm_hour,
		newtime->tm_min,
		newtime->tm_sec);

	gcap.actionLog<<timeStamp;
	gcap.actionLog<<tach;
	gcap.actionLog<<"\n";
	gcap.actionLog.flush();

}

void changeChannel(int channel)
{
	if(gcap.log) logAction(TEXT("changeChannel(%d) tunerSet=%d"), channel, gcap.tunerSet);

	if(gcap.aboutHWND != NULL)
		SetDlgItemText(gcap.aboutHWND, IDC_STATUS, "Setting Tuner Device");

	HRESULT hr;
	IAMTVTuner *pTV = NULL;
	IAMAnalogVideoDecoder *pAD = NULL;

	if(gcap.pBuilder == NULL || gcap.pVCap == NULL)
		return;

	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap,
		IID_IAMTVTuner, (void **)&pTV);

	if(!pTV) return;

	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap,
		IID_IAMAnalogVideoDecoder, (void **)&pAD);

	if(!pAD)
	{
		pTV->Release();
		return;
	}

	// Set the tuner up for country and input type
	// This only needs to be done once
	if(!gcap.tunerSet)
	{
		if(appData->tunerInputType == 0)
			pTV->put_InputType(0, TunerInputAntenna);
		if(appData->tunerInputType == 1)
			pTV->put_InputType(0, TunerInputCable);

		if(gcap.log)
			logAction(TEXT("tuner input number set to: %d"), appData->tunerInputType);

		pTV->put_CountryCode(appData->tunerCountryCode);

		if(gcap.log)
			logAction(TEXT("tuner country code set: %d"), appData->tunerCountryCode);

		pAD->put_TVFormat(appData->tunerMode);

		if(gcap.log)
			logAction(TEXT("tuner mode set to: %d"), appData->tunerMode);

		// Alway use tuning space ZERO 
		pTV->put_TuningSpace(0);

		gcap.tunerSet = true;
	}

	// Change the channel
	pTV->put_Channel(channel, AMTUNER_SUBCHAN_NO_TUNE, AMTUNER_SUBCHAN_NO_TUNE);

	pAD->Release();
	pTV->Release();
}

// Process Key input data
void DoKeyPressInput(int key)
{
	if(gcap.log) logAction(TEXT("DoKeyPressInput(%d)"), key);

	if(key == VK_ESCAPE && gcap.fCapturing)
	{
		if(gcap.log)
			logAction(TEXT("Key Stop Capture ESC"));

		StopCapture();
		if(appData->fWantPreview)
		{
			StartPreview();
		}
	}
	else if(key == VK_F5 && !gcap.fCapturing)
	{
		if(gcap.log)
			logAction(TEXT("Key Start Capture F5"));

		TryToStartCapture();
	}
	else if(key == 0x53) //0x53 = s
	{
		if(gcap.log)
			logAction(TEXT("Key Toggle Capture Stats S"));

		appData->fWantCaptureStats = !appData->fWantCaptureStats;
		ShowCapStats(appData->fWantCaptureStats);
	}
	else if(key == 0x50) //0x50 = p
	{
		if(gcap.log)
			logAction(TEXT("Key Toggle Preview P"));

		appData->fWantPreview = !appData->fWantPreview;
		
		if(appData->fWantPreview)
			StartPreview();
		else
			StopPreview();
	}
	else if(key == VK_F1)
	{
		// this is for testing only
		//RollCapFileOver();
	}
	else if(key == 0x46) // 0x46 = f toggle full screen
	{
		toggleFullScreen();
	}
	else if(key == 0x43 && !gcap.fCapturing) // 0x43 = C key
	{
		// Post Show Settings message
		PostMessage(ghwndApp, WM_COMMAND, MENU_SETTINGS, 0L);
	}
	else if(key == 0x48) // 0x48 = H key
	{
		appData->HideBars = !appData->HideBars;
		ResizeWindow();
	}
	else if(key == 0x54) // 0x54 is T
	{
		appData->AlwaysOnTop = !appData->AlwaysOnTop;
		ResizeWindow();
	}
	else if(key == VK_PRIOR)
	{
		ChannelChange(1);
	}
	else if(key == VK_NEXT)
	{
		ChannelChange(-1);
	}
	else if(!gcap.fCapturing)
	{
		// Do the change channel key inputs
		if(key >= 0x60 && key <= 0x69)
		{
			ProcessNumInput((int)key - 96, false);
		}
		else if(key == VK_RETURN)
			ProcessNumInput(0, true);
	}
}

// Process the keypad number and change channel
void ProcessNumInput(int number, bool change)
{
	if(change)
	{
		if(gcap.channelBuilder > 0)
		{
			char status[256];
			const char *name = NULL;

			int newChannel = -1;
			int display = -1;

			for(int x = 0; x < 256; x++)
			{
				if(appData->channels[x] != NULL)
				{
					if(appData->channels[x]->getChannelMAP() == gcap.channelBuilder)
					{
						newChannel = appData->channels[x]->getChannel();
						name = appData->channels[x]->getChannelName();
					}
				}
				else
					break;
			}

			if(newChannel > -1)
			{
				appData->TunerChannel = newChannel;

				int nameLength = strlen(name);

				if(nameLength > 0)
					wsprintf(status, TEXT("Channel : %d - %s"), gcap.channelBuilder, name);
				else
					wsprintf(status, TEXT("Channel : %d"), gcap.channelBuilder);

				changeChannel(appData->TunerChannel);
			}
			else
			{
				wsprintf(status, TEXT("Channel Not Found"));

			}

			statusUpdateStatus(ghwndStatus, status, 1);
		}
		gcap.channelBuilder = 0;
		
	}
	else
	{
		LONG now = timeGetTime();
		LONG timeDiff = now - gcap.lastKeysPressed;

		if(timeDiff > 1000)
			gcap.channelBuilder = 0;

		if(gcap.channelBuilder == 0)
			gcap.channelBuilder = number;
		else if(gcap.channelBuilder < 10)
			gcap.channelBuilder = (gcap.channelBuilder*10) + number;
		else if(gcap.channelBuilder < 100)
			gcap.channelBuilder = (gcap.channelBuilder*10) + number;
		else if(gcap.channelBuilder < 1000)
			gcap.channelBuilder = (gcap.channelBuilder*10) + number;

		gcap.lastKeysPressed = now;
	}
}

bool RollCapFileOver()
{
	if(gcap.log) logAction(TEXT("RollCapFileOver()"));

/*
//This works but dropps frames
	USES_CONVERSION;
    IMediaControl *pMC = NULL;
    gcap.pFg->QueryInterface(IID_IMediaControl, (void **)&pMC);
    if(pMC)
	{
		IFileSinkFilter2 *pFileWriterConfig;
		gcap.pFileWriter->QueryInterface(IID_IFileSinkFilter2, (void **)&pFileWriterConfig);

		if(pFileWriterConfig)
		{
			WCHAR *currentFile;// = new WCHAR[_MAX_PATH];
			WCHAR *newFile = new WCHAR[_MAX_PATH];
			pFileWriterConfig->GetCurFile(&currentFile, NULL);

			int indexAVI = wcslen(currentFile);

			WCHAR *current = new WCHAR[_MAX_PATH];
			wcsncpy(current, currentFile, indexAVI-8);
			current[indexAVI-8] = 0;
			current[indexAVI-7] = 0;

			// Rebuild the file name with the new segment number
			swprintf(newFile, T2W("%s.%.3d.avi"),
				current,
				gcap.capFileCount++);


			pMC->Stop();

			pFileWriterConfig->SetFileName(newFile, NULL);
			pFileWriterConfig->SetMode(AM_FILE_OVERWRITE);

			pMC->Run();


			pFileWriterConfig->Release();
			pFileWriterConfig = NULL;
		}

        
		pMC->Release();
    }
	else return false;
*/
	return true;
}

int FAR PASCAL DIALOG_AUDIO_INPUT(HWND hwnd, UINT msg, UINT wParam, LONG lParam)
{
    switch(msg)
	{
		case WM_INITDIALOG:
		{
			if(gcap.audioMix)
			{
				TCHAR tach[256];

				// Get the current volume setting
				int currentVolume = (int)
					((100.0 * gcap.audioMix->getLevel(appData->audioInputIndex)) + 0.5);

				int currentPan = 100 * gcap.audioMix->getPan(appData->audioInputIndex);

				// populate the combo box with the available input names
				HWND hwndCombo = GetDlgItem(hwnd, IDC_INPUT_LIST);
				gcap.audioMix->populateCombo(hwndCombo);

				SendMessage(hwndCombo,
					CB_SETCURSEL,
					(WPARAM) appData->audioInputIndex,
					(LPARAM) 0);

				// Set up the volume slider control
				HWND hwndVolume = GetDlgItem(hwnd, IDC_INPUT_VOLUME);

				SendMessage(hwndVolume,
					TBM_SETRANGE,
					(WPARAM) TRUE,
					(LPARAM) MAKELONG (0, 100));

				SendMessage(hwndVolume,
					TBM_SETPOS,
					(WPARAM) TRUE,
					(LPARAM) (100 - currentVolume));  

				SendMessage(hwndVolume,
					TBM_SETTICFREQ,
					(WPARAM) 5,
					(LPARAM) 0);  

				// Set the volume text
				_stprintf(tach, TEXT("%d"), currentVolume);
				SetDlgItemText(hwnd, IDC_INPUT_VOLUME_TEXT, tach);


				// Set up the pan slider control
				HWND hwndPan = GetDlgItem(hwnd, IDC_INPUT_PAN);

				SendMessage(hwndPan,
					TBM_SETRANGE,
					(WPARAM) TRUE,
					(LPARAM) MAKELONG (-100, 100));

				SendMessage(hwndPan,
					TBM_SETPOS,
					(WPARAM) TRUE,
					(LPARAM) currentPan); 

				SendMessage(hwndPan,
					TBM_SETTICFREQ,
					(WPARAM) 10,
					(LPARAM) 0); 

				// Set the pan text
				_stprintf(tach, TEXT("%d"), currentPan);
				SetDlgItemText(hwnd, IDC_INPUT_PAN_TEXT, tach);


				SetTimer(hwnd, 1, +500, NULL);
			}

			break;
		}

		case WM_TIMER:
		{
			TCHAR tach[256];
			// Set the Volume Level
			int currentVolume = (int)
				((100.0 * gcap.audioMix->getLevel(appData->audioInputIndex)) + 0.5);

			SendMessage(GetDlgItem(hwnd, IDC_INPUT_VOLUME),
				TBM_SETPOS,
				(WPARAM) TRUE,
				(LPARAM) (100 - currentVolume));  
				
			_stprintf(tach, TEXT("%d"), currentVolume);
			SetDlgItemText(hwnd, IDC_INPUT_VOLUME_TEXT, tach);

			// Set the Pan level
			int currentPan = 100 * gcap.audioMix->getPan(appData->audioInputIndex);

			SendMessage(GetDlgItem(hwnd, IDC_INPUT_PAN),
				TBM_SETPOS,
				(WPARAM) TRUE,
				(LPARAM) currentPan); 

			_stprintf(tach, TEXT("%d"), currentPan);
			SetDlgItemText(hwnd, IDC_INPUT_PAN_TEXT, tach);

			return (LRESULT) 1;
		}

		case WM_HSCROLL:
			{
				HWND scrolBar = (HWND)lParam;

				int value = SendMessage(scrolBar,
					TBM_GETPOS,
					(WPARAM) 0,
					(LPARAM) 0);  

				// Set the volume text
				TCHAR tach[256];
				_stprintf(tach, TEXT("%d"), value);
				SetDlgItemText(hwnd, IDC_INPUT_PAN_TEXT, tach);

				double newLevel = value / 100.0;
				gcap.audioMix->setPan(newLevel, appData->audioInputIndex);

				return FALSE;
			}
		break;

		case WM_VSCROLL:
			{
				HWND scrolBar = (HWND)lParam;

				int value = SendMessage(scrolBar,
					TBM_GETPOS,
					(WPARAM) 0,
					(LPARAM) 0);  

				// Set the volume text
				TCHAR tach[256];
				_stprintf(tach, TEXT("%d"), 100 - value);
				SetDlgItemText(hwnd, IDC_INPUT_VOLUME_TEXT, tach);

				double newLevel = (100 - value) / 100.0;
				gcap.audioMix->setLevel(newLevel, appData->audioInputIndex);

				return FALSE;
			}
		break;

		case WM_COMMAND:
		switch(HIWORD(wParam))
		{
			case CBN_SELCHANGE:
			{
				// The input selection has been changed
				switch(LOWORD(wParam))
				{
					case IDC_INPUT_LIST:

					int selected = 0;
					HWND hwndCombo = GetDlgItem(hwnd, IDC_INPUT_LIST);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);

					appData->audioInputIndex = selected;
					gcap.audioMix->setInput(appData->audioInputIndex);

					// Reset the dialog to use the new input
					SendMessage(hwnd,
						WM_INITDIALOG,
						(WPARAM) 0,
						(LPARAM) 0);  

					break;
				}

				break;
			}
			default:
			{
				switch(LOWORD(wParam))
				{
					case IDOK:
						EndDialog(hwnd, TRUE);
						break;
				}
				break;
			}
		}
		
        default:
            return FALSE;
    }

	return TRUE;
}

////////////////////////////////////////////////////////////////////////
/// Properties Settings Setup
////////////////////////////////////////////////////////////////////////

VOID ShowPropPages(HWND hwndOwner)
{
    PROPSHEETPAGE psp[10];
    PROPSHEETHEADER psh;

    psp[0].dwSize = sizeof(PROPSHEETPAGE);
    psp[0].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[0].hInstance = ghInstApp;
    psp[0].pszTemplate = MAKEINTRESOURCE(IDD_PROP_FILE);
    psp[0].pszIcon = NULL;
    psp[0].pfnDlgProc = PROP_FILE_PROC;
    psp[0].pszTitle = (LPSTR) "File";
    psp[0].lParam = 0;
    psp[0].pfnCallback = NULL;

    psp[1].dwSize = sizeof(PROPSHEETPAGE);
    psp[1].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[1].hInstance = ghInstApp;
    psp[1].pszTemplate = MAKEINTRESOURCE(IDD_PROP_VIDEO);
    psp[1].pszIcon = NULL;
    psp[1].pfnDlgProc = PROP_VIDEO_PROC;
    psp[1].pszTitle = (LPSTR) "Video";
    psp[1].lParam = 0;
    psp[1].pfnCallback = NULL;

    psp[2].dwSize = sizeof(PROPSHEETPAGE);
    psp[2].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[2].hInstance = ghInstApp;
    psp[2].pszTemplate = MAKEINTRESOURCE(IDD_PROP_FILTERS);
    psp[2].pszIcon = NULL;
    psp[2].pfnDlgProc = PROP_FILTERS_PROC;
    psp[2].pszTitle = (LPSTR) "Filters";
    psp[2].lParam = 0;
    psp[2].pfnCallback = NULL;

    psp[3].dwSize = sizeof(PROPSHEETPAGE);
    psp[3].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[3].hInstance = ghInstApp;
    psp[3].pszTemplate = MAKEINTRESOURCE(IDD_PROP_AUDIO);
    psp[3].pszIcon = NULL;
    psp[3].pfnDlgProc = PROP_AUDIO_PROC;
    psp[3].pszTitle = (LPSTR) "Audio";
    psp[3].lParam = 0;
    psp[3].pfnCallback = NULL;

    psp[4].dwSize = sizeof(PROPSHEETPAGE);
    psp[4].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[4].hInstance = ghInstApp;
    psp[4].pszTemplate = MAKEINTRESOURCE(IDD_PROP_AVSYNC);
    psp[4].pszIcon = NULL;
    psp[4].pfnDlgProc = PROP_AVSYNC_PROC;
    psp[4].pszTitle = (LPSTR) "AV Sync";
    psp[4].lParam = 0;
    psp[4].pfnCallback = NULL;

    psp[5].dwSize = sizeof(PROPSHEETPAGE);
    psp[5].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[5].hInstance = ghInstApp;
    psp[5].pszTemplate = MAKEINTRESOURCE(IDD_PROP_STOP);
    psp[5].pszIcon = NULL;
    psp[5].pfnDlgProc = PROP_STOP_PROC;
    psp[5].pszTitle = (LPSTR) "Stop";
    psp[5].lParam = 0;
    psp[5].pfnCallback = NULL;

    psp[6].dwSize = sizeof(PROPSHEETPAGE);
    psp[6].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[6].hInstance = ghInstApp;
    psp[6].pszTemplate = MAKEINTRESOURCE(IDD_PROP_VIEW);
    psp[6].pszIcon = NULL;
    psp[6].pfnDlgProc = PROP_VIEW_PROC;
    psp[6].pszTitle = (LPSTR) "View";
    psp[6].lParam = 0;
    psp[6].pfnCallback = NULL;

    psp[7].dwSize = sizeof(PROPSHEETPAGE);
    psp[7].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[7].hInstance = ghInstApp;
    psp[7].pszTemplate = MAKEINTRESOURCE(IDD_PROG_DEVICES);
    psp[7].pszIcon = NULL;
    psp[7].pfnDlgProc = PROP_DEVICES_PROC;
    psp[7].pszTitle = (LPSTR) "Devices";
    psp[7].lParam = 0;
    psp[7].pfnCallback = NULL;

    psp[8].dwSize = sizeof(PROPSHEETPAGE);
    psp[8].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[8].hInstance = ghInstApp;
    psp[8].pszTemplate = MAKEINTRESOURCE(IDD_PROP_TUNER);
    psp[8].pszIcon = NULL;
    psp[8].pfnDlgProc = PROP_TUNER_PROC;
    psp[8].pszTitle = (LPSTR) "Tuner";
    psp[8].lParam = 0;
    psp[8].pfnCallback = NULL;

    psp[9].dwSize = sizeof(PROPSHEETPAGE);
    psp[9].dwFlags = PSP_USEICONID | PSP_USETITLE;
    psp[9].hInstance = ghInstApp;
    psp[9].pszTemplate = MAKEINTRESOURCE(IDD_PROP_ABOUT);
    psp[9].pszIcon = NULL;
    psp[9].pfnDlgProc = PROP_ABOUT_PROC;
    psp[9].pszTitle = (LPSTR) "About";
    psp[9].lParam = 0;
    psp[9].pfnCallback = NULL;

    psh.dwSize = sizeof(PROPSHEETHEADER);
    psh.dwFlags = PSH_USEICONID | PSH_PROPSHEETPAGE;
    psh.hwndParent = hwndOwner;
    psh.hInstance = ghInstApp;
    psh.pszIcon = NULL;//MAKEINTRESOURCE(IDI_CELL_PROPERTIES);
    psh.pszCaption = (LPSTR) "Settings";
    psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
    psh.nStartPage = 0;
    psh.ppsp = (LPCPROPSHEETPAGE) &psp;
    psh.pfnCallback = NULL;

    int worked = PropertySheet(&psh);

    return;
}


int FAR PASCAL PROP_ABOUT_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam)
{
	HBRUSH g_hbrBackground = CreateSolidBrush(RGB(0, 0, 0));
    switch(msg)
	{
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
		{

/*
			HFONT hFont = CreateFont(-12, 0, 0, 0, 0, 0, 0, 0, ANSI_CHARSET,
				OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
				DEFAULT_PITCH, "MS Sans Serif");
*/
			HWND filterInfo = GetDlgItem(hwnd, IDC_FILTER_INFO);
			SendMessage(filterInfo,   
				WM_SETFONT,
				(WPARAM) 0,      // Font handle
				(LPARAM) true    // redraw
				);  


			char dialogVersioninfo[256];

			char verText[64];
			int ver = GetVersionInfo(verText, false);

			char arFilter[32];
			char fmFilter[32];
			char scopeFilter[32];
			char cropFilter[32];
			char histFilter[32];

			int result = 0;

			result = getFilterVersion(arFilter, CLSID_AudioResample);
			if (result != 0) MsgBox(TEXT("Error Getting CLSID_AudioResample Filter Version %d"), result); 

			result = getFilterVersion(fmFilter, CLSID_FlowMeter);
			if (result != 0) MsgBox(TEXT("Error Getting CLSID_FlowMeter Filter Version %d"), result); 

			result = getFilterVersion(scopeFilter, CLSID_Scope);
			if (result != 0) MsgBox(TEXT("Error Getting CLSID_Scope Filter Version %d"), result); 

			result = getFilterVersion(cropFilter, CLSID_VideoCrop);
			if (result != 0) MsgBox(TEXT("Error Getting CLSID_VideoCrop Filter Version %d"), result); 

			result = getFilterVersion(histFilter, CLSID_Histogram);
			if (result != 0) MsgBox(TEXT("Error Getting CLSID_Histogram Filter Version %d"), result); 

			wsprintf(dialogVersioninfo,
				"VirtualVCR.exe - %s\nAudioResample.ax - %s\nFlowMeter.ax - %s\nScope.ax - %s\nVideoCrop.ax - %s\nHistogram.ax - %s",
				verText,
				arFilter,
				fmFilter,
				scopeFilter,
				cropFilter,
				histFilter);

			SetDlgItemText(hwnd, IDC_FILTER_INFO, dialogVersioninfo);

            return TRUE;
		}

		case WM_CTLCOLORDLG:
		{
			return (LONG)g_hbrBackground;
		}

		case WM_CTLCOLORSTATIC:
		{
			HDC hdcStatic = (HDC)wParam;
			SetTextColor(hdcStatic, RGB(255, 255, 255));
			SetBkMode(hdcStatic, TRANSPARENT);
			return (LONG)g_hbrBackground;
		}
    }
    return FALSE;
}

int FAR PASCAL PROP_VIDEO_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{
    TCHAR  tach[32];
    TCHAR  Cwidth[32];
    TCHAR  Cheight[32];
	HRESULT hr;

    USES_CONVERSION;

    switch(msg)
	{
	case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{

			CheckDlgButton(hwnd, IDC_USE_VIDEO_SETTINGS, appData->fUseCustomFormat);
				
            /* put the current frame rate in the box */
			_stprintf(tach, TEXT("%.3f"), appData->FrameRate);
            SetDlgItemText(hwnd, IDC_FRAMERATE, tach);

            wsprintf(Cwidth, TEXT("%d"), appData->CustomWidth);
            SetDlgItemText(hwnd, IDC_CUSTOM_WIDTH, Cwidth);

			wsprintf(Cheight, TEXT("%d"), appData->CustomHeight);
            SetDlgItemText(hwnd, IDC_CUSTOM_HEIGHT, Cheight);

			HWND hwndCombo = GetDlgItem(hwnd, IDC_CUSTOM_COLOUR);
			SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);

			//
			// Set up the Colour Format Drop down box
			//
			IAMStreamConfig *videoConfig = NULL;
			TCHAR name[32];

			if(gcap.pVCap)
				gcap.pBuilder->FindInterface(
					&PIN_CATEGORY_CAPTURE,
					&MEDIATYPE_Video,
					gcap.pVCap,
					IID_IAMStreamConfig,
					(void **)&videoConfig);

			if(videoConfig)
			{
				int iCount, iSize;
				VIDEO_STREAM_CONFIG_CAPS scc;
				AM_MEDIA_TYPE *pmt;
				hr = videoConfig->GetNumberOfCapabilities(&iCount, &iSize);
				
				for(int x = 0; x < iCount; x++)
				{
					hr = videoConfig->GetStreamCaps(x, &pmt, reinterpret_cast<BYTE*>(&scc));

					appData->ColourFormatName(pmt->subtype, name);

					int result = SendMessage(hwndCombo, CB_FINDSTRING, 0, (LPARAM) name);

					if(result < 0)
						SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) name);

					DeleteMediaType(pmt);
				}
				videoConfig->Release();
				videoConfig = NULL;
			}

			appData->ColourFormatName(appData->ColourFormat, name);
			SendMessage(hwndCombo, WM_SETTEXT, 0, (LPARAM) name);

			// Now do the Compression stuff

			hwndCombo = GetDlgItem(hwnd, IDC_VIDEO_COMP_LIST);
			SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);

			int codecIndex = 0;
			while(codecIndex < 400)
			{
				string name = appData->VideoCodec->getCodecName(codecIndex);
				if(name == "end") break;
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) name.c_str());
				SendMessage(hwndCombo, CB_SETITEMDATA, (WPARAM)codecIndex, (LPARAM)codecIndex);
				codecIndex++;
			}

			if(appData->VideoCodec->currentCodecINDEX() > -1)
			{
				SendMessage(hwndCombo, WM_SETTEXT, 0, 
					(LPARAM)appData->VideoCodec->getCodecName(appData->VideoCodec->currentCodecINDEX()).c_str());
			}

            CheckDlgButton(hwnd, IDC_USE_VIDEO_COMP, appData->CompressVideo);

            break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_APPLY:
				{
					appData->fUseCustomFormat = IsDlgButtonChecked(hwnd, IDC_USE_VIDEO_SETTINGS);

					/* get the new frame rate */
					GetDlgItemText(hwnd, IDC_FRAMERATE, tach, sizeof(tach)/sizeof(tach[0]));
					GetDlgItemText(hwnd, IDC_CUSTOM_WIDTH, Cwidth, sizeof(Cwidth)/sizeof(Cwidth[0]));
					GetDlgItemText(hwnd, IDC_CUSTOM_HEIGHT, Cheight, sizeof(Cheight)/sizeof(Cheight[0]));

#ifdef UNICODE
					int rc;

					// Convert Multibyte string to ANSI
					char szANSI[STR_MAX_LENGTH];
					char CWszANSI[STR_MAX_LENGTH];
					char CHszANSI[STR_MAX_LENGTH];

					rc = WideCharToMultiByte(CP_ACP, 0, tach, -1, szANSI, 
						STR_MAX_LENGTH, NULL, NULL);
					rc = WideCharToMultiByte(CP_ACP, 0, Cwidth, -1, CWszANSI, 
						STR_MAX_LENGTH, NULL, NULL);
					rc = WideCharToMultiByte(CP_ACP, 0, Cheight, -1, CHszANSI, 
						STR_MAX_LENGTH, NULL, NULL);
					double frameRate = atof(szANSI);
					int width = atol(CWszANSI);
					int height = atol(Cheight);
#else
					double frameRate = atof(tach);
					int width = atol(Cwidth);
					int height = atol(Cheight);
#endif

					// Checkframe rate
					if(frameRate <= 0.) {
						ErrMsg(TEXT("Invalid frame rate."));
						break;
					}
					else
						appData->FrameRate = frameRate;

					// Check width and height
					if(width <= 0 || width > 1000 || height <= 0 || height > 1000) {
						ErrMsg(TEXT("Invalid width or height rate."));
						break;
					}
					else
					{
						appData->CustomWidth = width;
						appData->CustomHeight = height;
					}

					// Set the colour format
					GetDlgItemText(hwnd, IDC_CUSTOM_COLOUR, tach, sizeof(tach)/sizeof(tach[0]));
					appData->ColourFormat = appData->ColourFormatGUID(tach);

					// Now do the compression codec stuff
					int selectedCodec = SendDlgItemMessage(hwnd, IDC_VIDEO_COMP_LIST, CB_GETCURSEL, 0, 0);
					int filterIndex = 0;
					filterIndex = SendDlgItemMessage(hwnd, IDC_VIDEO_COMP_LIST, CB_GETITEMDATA, (WPARAM)selectedCodec, 0);
					//MsgBox(TEXT("selected = %d"), filterIndex);
					if(filterIndex > -1)
						appData->VideoCodec->getFilterByINDEX(filterIndex);
					appData->CompressVideo = IsDlgButtonChecked(hwnd, IDC_USE_VIDEO_COMP);

					break;
				}
			}
        case WM_COMMAND:
            switch(wParam){

			case IDC_DEVICE_SETUP:
				{
					if(gcap.pVCap == NULL)
						break;

                    ISpecifyPropertyPages *pSpec;
                    CAUUID cauuid;
                    hr = gcap.pVCap->QueryInterface(IID_ISpecifyPropertyPages,
                        (void **)&pSpec);
                    if(hr == S_OK)
					{
                        hr = pSpec->GetPages(&cauuid);
                        hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
                            (IUnknown **)&gcap.pVCap, cauuid.cElems,
                            (GUID *)cauuid.pElems, 0, 0, NULL);
                        CoTaskMemFree(cauuid.pElems);
                        pSpec->Release();
                    }
					break;
				}

			case IDC_VIDEO_COMP_DLG_DS:
				{
					int selectedCodec = SendDlgItemMessage(hwnd, IDC_VIDEO_COMP_LIST, CB_GETCURSEL, 0, 0);
					int filterIndex = 0;
					filterIndex = SendDlgItemMessage(hwnd, IDC_VIDEO_COMP_LIST, CB_GETITEMDATA, (WPARAM)selectedCodec, 0);
					if(filterIndex > -1)
						appData->VideoCodec->getFilterByINDEX(filterIndex);
					appData->VideoCodec->showProperties(hwnd);
					break;
				}
            }
            break;

        default:
            return FALSE;
    }
    return TRUE;
}


int FAR PASCAL PROP_FILTERS_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{
    USES_CONVERSION;
	char filterName[256];
	int selectedFilter = -1;

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
		case WM_INITDIALOG:
		{
			HWND hwndCombo = GetDlgItem(hwnd, IDC_FilterList2);

			BOOL showAll = IsDlgButtonChecked(hwnd, IDC_SHOW_ALL_FILTERS);
			// Remove all the filters from the Drop down list
			SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
			
			// Populate the Drop down list with all the filter available
			int codecIndex = 0;
			int listCount = 0;
			while(codecIndex < 5000)
			{
				string filterNam = appData->filterList->getNameFromAllList(codecIndex);
				if(filterNam == "end") break;
				if(appData->filterList->isFilterMine(codecIndex) || showAll)
				{
					SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) filterNam.c_str());
					SendMessage(hwndCombo, CB_SETITEMDATA, (WPARAM)listCount++, (LPARAM)codecIndex);
				}
				codecIndex++;
			}
			// Set the drop down list initial text
			SendMessage(hwndCombo, WM_SETTEXT, 0, (LPARAM) "<Select Filter>");
			
			// Populate the list box with selected filters
			appData->filterList->populateListBoxWithSelected(hwnd, IDC_FilterList);

            CheckDlgButton(hwnd, IDC_FILTERS_USE, appData->UsePreFilters);
            break;
		}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_APPLY:
				{
					appData->UsePreFilters = IsDlgButtonChecked(hwnd, IDC_FILTERS_USE);
					break;
				}
				break;
			}

        case WM_COMMAND:
			switch(HIWORD(wParam))
			{
	   			case BN_CLICKED:
	    			switch(LOWORD(wParam))
	     			{
			            case IDC_FILTER_PROP:
						selectedFilter = SendDlgItemMessage(hwnd, IDC_FilterList, LB_GETCURSEL, 0, 0);
						appData->filterList->showPropBox(hwnd, selectedFilter);
						break;

			            case IDC_FILTER_MOVE_DOWN:
						selectedFilter = SendDlgItemMessage(hwnd, IDC_FilterList, LB_GETCURSEL, 0, 0);
						appData->filterList->moveFilterDown(selectedFilter);
						appData->filterList->populateListBoxWithSelected(hwnd, IDC_FilterList);
						break;

						case IDC_FILTER_MOVE_UP:
						selectedFilter = SendDlgItemMessage(hwnd, IDC_FilterList, LB_GETCURSEL, 0, 0);
						appData->filterList->moveFilterUP(selectedFilter);
						appData->filterList->populateListBoxWithSelected(hwnd, IDC_FilterList);
						break;

						case IDC_FILTER_REMOVE:
						selectedFilter = SendDlgItemMessage(hwnd, IDC_FilterList, LB_GETCURSEL, 0, 0);
						SendDlgItemMessage(hwnd, IDC_FilterList, LB_GETTEXT, selectedFilter, (LPARAM) filterName);
						appData->filterList->removeFilter(selectedFilter);
						appData->filterList->populateListBoxWithSelected(hwnd, IDC_FilterList);
						break;

						case IDC_ADD_FILTER:
							{
								selectedFilter = SendDlgItemMessage(hwnd, IDC_FilterList2, CB_GETCURSEL, 0, 0);
								int filterIndex = 0;
								filterIndex = SendDlgItemMessage(hwnd, IDC_FilterList2, CB_GETITEMDATA, (WPARAM)selectedFilter, 0);
								//MsgBox(TEXT("Selected Filter = %d %d"), selectedFilter, filterIndex);

								if(selectedFilter < 0 && filterIndex < 0)
									break;

								if(appData->filterList->isFilterInSelected(filterIndex))
								{
									ErrMsg(TEXT("This filter is already in the filter list, you can not add it again."));
									break;
								}

								appData->filterList->addFilterToSelected(filterIndex);
								appData->filterList->populateListBoxWithSelected(hwnd, IDC_FilterList);
								break;
							}
						
						case IDC_SHOW_ALL_FILTERS:
							{
								HWND hwndCombo = GetDlgItem(hwnd, IDC_FilterList2);
								BOOL showAll = IsDlgButtonChecked(hwnd, IDC_SHOW_ALL_FILTERS);
								// Remove all the filters from the Drop down list
								SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
								
								// Populate the Drop down list with all the filter available
								int codecIndex = 0;
								int listCount = 0;
								while(codecIndex < 5000)
								{
									string filterNam = appData->filterList->getNameFromAllList(codecIndex);
									if(filterNam == "end") break;
									if(appData->filterList->isFilterMine(codecIndex) || showAll)
									{
										SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) filterNam.c_str());
										SendMessage(hwndCombo, CB_SETITEMDATA, (WPARAM)listCount++, (LPARAM)codecIndex);
									}
									codecIndex++;
								}
								// Set the drop down list initial text
								SendMessage(hwndCombo, WM_SETTEXT, 0, (LPARAM) "<Select Filter>");
								
								break;
							}
							
					}
			
			}

        default:
            return FALSE;
    }
    return TRUE;
}


int FAR PASCAL PROP_FILE_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{
    TCHAR tach[256];
	bool needsEXT = true;

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{

				CheckDlgButton(hwnd, IDC_INSERTSTAMP, appData->appendStampToName);
				CheckDlgButton(hwnd, IDC_AVI_COMPAT, appData->compatAVIindex);
				CheckDlgButton(hwnd, IDC_SAVESTATS, appData->SaveCapStats);
				
				wsprintf(tach, TEXT("%s"), appData->szCaptureFile);
				SetDlgItemText(hwnd, IDC_CAPFILENAME, tach);

				break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_APPLY:
				{
					appData->appendStampToName =
						IsDlgButtonChecked(hwnd, IDC_INSERTSTAMP);

					appData->compatAVIindex = 
						IsDlgButtonChecked(hwnd, IDC_AVI_COMPAT);

					appData->SaveCapStats = 
						IsDlgButtonChecked(hwnd, IDC_SAVESTATS);

					GetDlgItemText(hwnd, IDC_CAPFILENAME, tach, 256);
					for(int x = 0; x < 250; x++)
					{
						if(tach[x] == '.')
						{
							if(tach[x+1] == 'a' || tach[x+1] == 'A')
							{
								if(tach[x+2] == 'v' || tach[x+2] == 'V')
								{
									if(tach[x+3] == 'i' || tach[x+3] == 'I')
									{
										needsEXT = false;
										break;
									}
								}
							}
						}

					}
					if(needsEXT)
						wsprintf(appData->szCaptureFile, "%s.avi", tach);
					else
						wsprintf(appData->szCaptureFile, "%s", tach);

					break;
				}
				break;
			}

        case WM_COMMAND:
            switch(wParam)
			{

			case IDC_SHOWOPENFILE:
                SetCaptureFile(hwnd);
				wsprintf(tach, TEXT("%s"), appData->szCaptureFile);
				SetDlgItemText(hwnd, IDC_CAPFILENAME, tach);
                break;

            }
            break;

        default:
            return FALSE;
    }
    return TRUE;
}

int FAR PASCAL PROP_STOP_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{
    TCHAR   tach[32];
    DWORD   temp;

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{
			SendMessage(GetDlgItem( hwnd, IDC_TIMELIMIT_HOURS_SPIN ),
				UDM_SETRANGE, 0, MAKELPARAM(59, 0));
			SendMessage(GetDlgItem( hwnd, IDC_TIMELIMIT_MIN_SPIN ),
				UDM_SETRANGE, 0, MAKELPARAM(59, 0));
			SendMessage(GetDlgItem( hwnd, IDC_TIMELIMIT_SEC_SPIN ),
				UDM_SETRANGE, 0, MAKELPARAM(59, 0));

			SendMessage(GetDlgItem( hwnd, IDC_FREESPACELIMIT_SPIN ),
				UDM_SETRANGE, 0, MAKELPARAM(1000, 0));

            /* put the current time limit info in the boxes */
			__int64 hours = appData->dwTimeLimit / 3600;
			__int64 minutes = (appData->dwTimeLimit - (hours * 3600)) / 60;
			__int64 seconds = appData->dwTimeLimit - (hours * 3600) - (minutes * 60);
            
			wsprintf(tach, TEXT("%d"), hours);
            SetDlgItemText(hwnd, IDC_TIMELIMIT_HOURS, tach);
			wsprintf(tach, TEXT("%d"), minutes);
            SetDlgItemText(hwnd, IDC_TIMELIMIT_MIN, tach);
			wsprintf(tach, TEXT("%d"), seconds);
            SetDlgItemText(hwnd, IDC_TIMELIMIT_SEC, tach);

            CheckDlgButton(hwnd, IDC_USETIMELIMIT, appData->fUseTimeLimit);
			CheckDlgButton(hwnd, IDC_USEFREESPACE, appData->fUseFreeSpace);
			CheckDlgButton(hwnd, IDC_QUITWHENDONE, appData->fQuitWhenDone);

			wsprintf(tach, TEXT("%d"), appData->dwFreeSpaceLimit);
            SetDlgItemText(hwnd, IDC_FREESPACELIMIT, tach);

            break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_APPLY:
				{

					temp = 0 ;
					temp = GetDlgItemInt(hwnd, IDC_TIMELIMIT_HOURS, NULL, FALSE) * 3600;
					temp += GetDlgItemInt(hwnd, IDC_TIMELIMIT_MIN, NULL, FALSE) * 60;
					temp += GetDlgItemInt(hwnd, IDC_TIMELIMIT_SEC, NULL, FALSE);
					appData->dwTimeLimit = temp;
					appData->fUseTimeLimit = IsDlgButtonChecked(hwnd, IDC_USETIMELIMIT);

					appData->fUseFreeSpace = IsDlgButtonChecked(hwnd, IDC_USEFREESPACE);
					temp = GetDlgItemInt(hwnd, IDC_FREESPACELIMIT, NULL, FALSE);
					appData->dwFreeSpaceLimit = temp;
					appData->fQuitWhenDone = IsDlgButtonChecked(hwnd, IDC_QUITWHENDONE);

					EndDialog(hwnd, TRUE);
					break;
				}

			break;
			}

        default:
            return FALSE;
    }
    return TRUE;
}

int FAR PASCAL PROP_AVSYNC_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{

	TCHAR   tach[32];

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{

			HWND hwndCombo = GetDlgItem(hwnd, IDC_MASTER_STREAM);

			SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
			SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"None");
			SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Video");
			SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Audio");

			if(appData->useAudioResample)
			{
				appData->iMasterStream = -1;
				EnableWindow(hwndCombo, FALSE);
			}

			SendMessage(hwndCombo, CB_SETCURSEL, 
				(WPARAM)(appData->iMasterStream+1), (LPARAM)0);

            CheckDlgButton(hwnd, IDC_SYNC_STREAM, appData->SyncUsingStreamOffset);
			CheckDlgButton(hwnd, IDC_RESAMPLE_AUDIO, appData->useAudioResample);
			CheckDlgButton(hwnd, IDC_LOG_AVDIFF, appData->LogAVdiff);

			if(appData->streamOffsetType == 0)
				CheckDlgButton(hwnd, IDC_SYNC_STREAM_OFFSET, true);
			else
				CheckDlgButton(hwnd, IDC_SYNC_STEAM_CONST, true);

			if(appData->audioResampleType == 0)
				CheckDlgButton(hwnd, IDC_AUDIO_DYN, true);
			else
				CheckDlgButton(hwnd, IDC_AUDIO_CONST, true);

			wsprintf(tach, TEXT("%d"), appData->AudioDelay);
            SetDlgItemText(hwnd, IDC_AUDIO_OFFSET, tach);

			_stprintf(tach, TEXT("%.5f"), appData->ResampleAudioTo);
            SetDlgItemText(hwnd, IDC_RESAMPLE_TO, tach);

            break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_APPLY:
				{
					appData->useAudioResample = 
						IsDlgButtonChecked(hwnd, IDC_RESAMPLE_AUDIO);

					if(appData->useAudioResample)
					{
						appData->iMasterStream = -1;
					}
					else
					{
						HWND hwndCombo = GetDlgItem(hwnd, IDC_MASTER_STREAM);
						int master = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);
						appData->iMasterStream = master - 1;
					}

					if(IsDlgButtonChecked(hwnd, IDC_SYNC_STREAM_OFFSET))
						appData->streamOffsetType = 0;
					else
						appData->streamOffsetType = 1;

					if(IsDlgButtonChecked(hwnd, IDC_AUDIO_DYN))
						appData->audioResampleType = 0;
					else
						appData->audioResampleType = 1;

					appData->SyncUsingStreamOffset = 
						IsDlgButtonChecked(hwnd, IDC_SYNC_STREAM);

					appData->LogAVdiff = 
						IsDlgButtonChecked(hwnd, IDC_LOG_AVDIFF);

					appData->AudioDelay = 
						GetDlgItemInt(hwnd, IDC_AUDIO_OFFSET, NULL, TRUE);

					TCHAR tach[64];
					GetDlgItemText(hwnd, IDC_RESAMPLE_TO, tach, sizeof(tach)/sizeof(tach[0]));
					appData->ResampleAudioTo = atof(tach);

					break;
				}

			break;
			}

        case WM_COMMAND:
            switch(wParam)
			{
			
			case IDC_RESAMPLE_AUDIO:
				{
					if(IsDlgButtonChecked(hwnd, IDC_RESAMPLE_AUDIO))
					{
						appData->iMasterStream = -1;

						SendMessage(GetDlgItem(hwnd, IDC_MASTER_STREAM), CB_SETCURSEL, 
							(WPARAM)(appData->iMasterStream+1), (LPARAM)0);

						EnableWindow(GetDlgItem(hwnd, IDC_MASTER_STREAM), FALSE);
					}
					else
					{
						EnableWindow(GetDlgItem(hwnd, IDC_MASTER_STREAM), TRUE);
					}
				}

			}
			break;

        default:
            return FALSE;
    }
    return TRUE;
}

int FAR PASCAL PROP_VIEW_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{

			CheckDlgButton(hwnd, IDC_SHOW_ON_STOP, appData->showStatsOnStop);
			CheckDlgButton(hwnd, IDC_DOCK_STATS, appData->CapStatsRelativePos);
			CheckDlgButton(hwnd, IDC_USE_SMART_TEE, appData->UseSmartTee);
			CheckDlgButton(hwnd, IDC_SHOW_HISTOGRAM, appData->ShowHistogram);
			CheckDlgButton(hwnd, IDC_SHOW_AUDIOSCOPE, appData->ShowAudioScope);
			CheckDlgButton(hwnd, IDC_ALWAYSONTOP, appData->AlwaysOnTop);
			CheckDlgButton(hwnd, IDC_HIDEBARS, appData->HideBars);
			

            break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_APPLY:
				{

					appData->showStatsOnStop = 
						IsDlgButtonChecked(hwnd, IDC_SHOW_ON_STOP);

					appData->CapStatsRelativePos = 
						IsDlgButtonChecked(hwnd, IDC_DOCK_STATS);

					appData->UseSmartTee = 
						IsDlgButtonChecked(hwnd, IDC_USE_SMART_TEE);

					appData->ShowHistogram = 
						IsDlgButtonChecked(hwnd, IDC_SHOW_HISTOGRAM);

					appData->ShowAudioScope = 
						IsDlgButtonChecked(hwnd, IDC_SHOW_AUDIOSCOPE);

					appData->AlwaysOnTop = 
						IsDlgButtonChecked(hwnd, IDC_ALWAYSONTOP);

					appData->HideBars = 
						IsDlgButtonChecked(hwnd, IDC_HIDEBARS);

					break;
				}

			break;
			}

        default:
            return FALSE;
    }
    return TRUE;
}

int FAR PASCAL PROP_AUDIO_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{
				int selected = 0;
				GetAudioFormat();

				HWND hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_BITS);
				SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"8");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"16");
				if(appData->Audio_wBitsPerSample == 8) selected = 0;
				if(appData->Audio_wBitsPerSample == 16) selected = 1;
				SendMessage(hwndCombo, CB_SETCURSEL, 
					(WPARAM)selected, (LPARAM)0); 

				hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_RATE);
				SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"8 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"11.025 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"12 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"16 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"22.05 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"24 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"32 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"44.1 kHz");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"48 kHz");
				if(appData->Audio_nSamplesPerSec == 8000) selected = 0;
				if(appData->Audio_nSamplesPerSec == 11025) selected = 1;
				if(appData->Audio_nSamplesPerSec == 12000) selected = 2;
				if(appData->Audio_nSamplesPerSec == 16000) selected = 3;
				if(appData->Audio_nSamplesPerSec == 22050) selected = 4;
				if(appData->Audio_nSamplesPerSec == 24000) selected = 5;
				if(appData->Audio_nSamplesPerSec == 32000) selected = 6;
				if(appData->Audio_nSamplesPerSec == 44100) selected = 7;
				if(appData->Audio_nSamplesPerSec == 48000) selected = 8;

				SendMessage(hwndCombo, CB_SETCURSEL, 
					(WPARAM)selected, (LPARAM)0);  

				hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_CHANNELS);
				SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Mono");
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Stereo");
				if(appData->Audio_nChannel == 1) selected = 0;
				if(appData->Audio_nChannel == 2) selected = 1;
				SendMessage(hwndCombo, CB_SETCURSEL, 
					(WPARAM)selected, (LPARAM)0); 

				CheckDlgButton(hwnd, IDC_CAP_AUDIO, 
					appData->fCapAudio);

				// Show the Audio Compression Codecs List
				hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_COMP_LIST);
				SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);

				int codecIndex = 0;
				while(codecIndex < 400)
				{
					string name = appData->AudioCodec->getCodecName(codecIndex);
					if(name == "end") break;
					SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) name.c_str());
					SendMessage(hwndCombo, CB_SETITEMDATA, (WPARAM)codecIndex, (LPARAM)codecIndex);
					codecIndex++;
				}
				if(appData->AudioCodec->currentCodecINDEX() > -1)
				{
					SendMessage(hwndCombo, CB_SELECTSTRING, 0, 
						(LPARAM)appData->AudioCodec->getCodecName(appData->AudioCodec->currentCodecINDEX()).c_str());
				}

				CheckDlgButton(hwnd, IDC_COMP_AUDIO, appData->CompressAudio);

            break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_KILLACTIVE:
				{

					return FALSE;
				}
			case PSN_APPLY:
				{
					int selected = -1;
					HWND hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_RATE);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);

					if(selected == 0) appData->Audio_nSamplesPerSec = 8000;
					if(selected == 1) appData->Audio_nSamplesPerSec = 11025;
					if(selected == 2) appData->Audio_nSamplesPerSec = 12000;
					if(selected == 3) appData->Audio_nSamplesPerSec = 16000;
					if(selected == 4) appData->Audio_nSamplesPerSec = 22050;
					if(selected == 5) appData->Audio_nSamplesPerSec = 24000;
					if(selected == 6) appData->Audio_nSamplesPerSec = 32000;
					if(selected == 7) appData->Audio_nSamplesPerSec = 44100;
					if(selected == 8) appData->Audio_nSamplesPerSec = 48000;

					hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_BITS);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);
					if(selected == 0) appData->Audio_wBitsPerSample = 8;
					if(selected == 1) appData->Audio_wBitsPerSample = 16;

					hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_CHANNELS);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);
					if(selected == 0) appData->Audio_nChannel = 1;
					if(selected == 1) appData->Audio_nChannel = 2;

					appData->Audio_nAvgBytesPerSec =
						appData->Audio_nChannel *
						appData->Audio_nSamplesPerSec *
						(appData->Audio_wBitsPerSample / 8);

					appData->Audio_nBlockAlign = 
						appData->Audio_nChannel *
						(appData->Audio_wBitsPerSample / 8);
/*
					MsgBox(TEXT("%d %d %d %d %d"),
						appData->Audio_nSamplesPerSec,
						appData->Audio_nChannel,
						appData->Audio_wBitsPerSample,
						appData->Audio_nAvgBytesPerSec,
						appData->Audio_nBlockAlign);
*/
					SetAudioFormat();

					appData->fCapAudio = 
						IsDlgButtonChecked(hwnd, IDC_CAP_AUDIO);

					// get the audio compression codec
					int selectedCodec = 0;
					selectedCodec = SendDlgItemMessage(hwnd, IDC_AUDIO_COMP_LIST, CB_GETCURSEL, 0, 0);
					int filterIndex = 0;
					filterIndex = SendDlgItemMessage(hwnd, IDC_AUDIO_COMP_LIST, CB_GETITEMDATA, (WPARAM)selectedCodec, 0);
					//MsgBox(TEXT("selected = %d index = %d"), selectedCodec, filterIndex);
					if(filterIndex > -1)
						appData->AudioCodec->getFilterByINDEX(filterIndex);

					appData->CompressAudio = IsDlgButtonChecked(hwnd, IDC_COMP_AUDIO);

					break;
				}

			break;
			}
        case WM_COMMAND:
            switch(wParam)
			{
			
			case IDC_AUDIO_COMP_SETTINGS:
				{

					// get the audio compression codec
					int selectedCodec = 0;
					selectedCodec = SendDlgItemMessage(hwnd, IDC_AUDIO_COMP_LIST, CB_GETCURSEL, 0, 0);
					int filterIndex = 0;
					filterIndex = SendDlgItemMessage(hwnd, IDC_AUDIO_COMP_LIST, CB_GETITEMDATA, (WPARAM)selectedCodec, 0);
					//MsgBox(TEXT("selected = %d index = %d"), selectedCodec, filterIndex);
					if(filterIndex > -1)
					{
						appData->AudioCodec->getFilterByINDEX(filterIndex);

						DialogBox(ghInstApp, MAKEINTRESOURCE(IDD_AUDIO_COMP_CAP), hwnd, 
							(DLGPROC)CompCapabilityProc);
					}

					break;
				}

			case IDC_AUDIO_DEVICE:
				{

					if(gcap.pACap != NULL)
					{
						gcap.audioMix = new CAudioMixer(gcap.pACap);

						DialogBox(ghInstApp, MAKEINTRESOURCE(IDD_AUDIO_INPUT), hwnd, 
							(DLGPROC)DIALOG_AUDIO_INPUT);

						delete gcap.audioMix;
					}
/*
                    ISpecifyPropertyPages *pSpec;
                    CAUUID cauuid;

					if(gcap.pACap == NULL)
						break;

                    HRESULT hr = gcap.pACap->QueryInterface(IID_ISpecifyPropertyPages,
                        (void **)&pSpec);
                    if(hr == S_OK)
					{
                        hr = pSpec->GetPages(&cauuid);
                        hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
                            (IUnknown **)&gcap.pACap, cauuid.cElems,
                            (GUID *)cauuid.pElems, 0, 0, NULL);
                        CoTaskMemFree(cauuid.pElems);
                        pSpec->Release();
                    }
*/

					break;
				}
			case IDC_AUDIO_TV_DEV:
				{
					IAMTVAudio *pTVA;

					if(gcap.pBuilder == NULL || gcap.pACap == NULL)
						break;

					HRESULT hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Audio, gcap.pACap, IID_IAMTVAudio, (void **)&pTVA);
					if (hr != S_OK)
						hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, gcap.pVCap, IID_IAMTVAudio, (void **)&pTVA);

					if (hr != S_OK)
					{
						ErrMsg(TEXT("No TV Tuner Audio Device Found"));
						break;
					}

					ISpecifyPropertyPages *pSpec;
					CAUUID cauuid;
					hr = pTVA->QueryInterface(IID_ISpecifyPropertyPages,
						(void **)&pSpec);
					if(hr == S_OK)
					{
						hr = pSpec->GetPages(&cauuid);
						hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
							(IUnknown **)&pTVA, cauuid.cElems,
							(GUID *)cauuid.pElems, 0, 0, NULL);
						CoTaskMemFree(cauuid.pElems);
						pSpec->Release();
					}
					pTVA->Release();

					break;
				}

            }
            break;

        default:
            return FALSE;
    }
    return TRUE;
}

int FAR PASCAL PROP_DEVICES_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{
				//
				// Populate the Video Device List
				//
				HWND hwndCombo = GetDlgItem(hwnd, IDC_VIDEO_DEVICE_LIST);
				SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);

				char name[256];
				IPropertyBag *pPropBag = NULL;
				for(int x = 0; x < 10; x++)
				{
					if(!gcap.rgpmVideoMenu[x]) break;

					gcap.rgpmVideoMenu[x]->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);
					if(pPropBag)
					{
						VARIANT varName;
						VariantInit(&varName);
						varName.vt = VT_BSTR;
						pPropBag->Read(L"FriendlyName", &varName, 0);
						WideCharToMultiByte(CP_ACP, 0, varName.bstrVal, -1, name, 128, NULL, NULL);
						SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)name);
						VariantClear(&varName);
					}
				}
				//Select the current video device
				SendMessage(hwndCombo, CB_SETCURSEL, 
					(WPARAM)appData->VideoDeviceIndex, (LPARAM)0);

				//
				// Populate the Audio Device List
				//
				hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_DEVICE_LIST);
				SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
				for(int y = 0; y < 10; y++)
				{
					if(!gcap.rgpmAudioMenu[y]) break;

					gcap.rgpmAudioMenu[y]->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pPropBag);
					if(pPropBag)
					{
						VARIANT varName;
						VariantInit(&varName);
						varName.vt = VT_BSTR;
						pPropBag->Read(L"FriendlyName", &varName, 0);
						WideCharToMultiByte(CP_ACP, 0, varName.bstrVal, -1, name, 128, NULL, NULL);
						SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)name);
						VariantClear(&varName);
					}
				}

				if(pPropBag)
					pPropBag->Release();
				pPropBag = NULL;

				//Select the current audio device
				SendMessage(hwndCombo, CB_SETCURSEL, 
					(WPARAM)appData->AudioDeviceIndex, (LPARAM)0);

				//
				// Populate the Source Drop Down
				//
				hwndCombo = GetDlgItem(hwnd, IDC_VIDEO_SOURCE);
				SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
				if(gcap.pCrossbar)
				{
					LONG currentIndex = 0;
					LONG totalInputs = 0;
					gcap.pCrossbar->GetInputIndex(&currentIndex);
					gcap.pCrossbar->GetInputCount(&totalInputs);

					TCHAR inputName[128];
					for(int z = 0; z < totalInputs; z++)
					{
						gcap.pCrossbar->GetInputName(z, inputName, 128);
						SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)inputName);
					}
					if(currentIndex < 0) currentIndex = 0;
					SendMessage(hwndCombo, CB_SETCURSEL, 
						(WPARAM)currentIndex, (LPARAM)0);
				}

				//
				// Now populate the Device Interface List
				//
				hwndCombo = GetDlgItem(hwnd, IDC_DEVICE_FILTER_LIST);
				PopulateDeviceDropDownBox(hwndCombo);

            break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{

			case PSN_APPLY:
				{
					int selected = -1;
					HWND hwndCombo = GetDlgItem(hwnd, IDC_VIDEO_SOURCE);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);
					if(gcap.pCrossbar)
					{
						appData->iVideoInputIndex = selected;
						gcap.pCrossbar->SetInputIndex(appData->iVideoInputIndex);
					}

					break;
				}
			break;
			}

		case WM_COMMAND:
            switch(wParam)
			{

			case IDC_FILTER_PROPERTIES:
				{
					int selected = -1;
					int filter = -1;
					HWND hwndCombo = GetDlgItem(hwnd, IDC_DEVICE_FILTER_LIST);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);

					filter = SendMessage(hwndCombo, CB_GETITEMDATA, selected, 0);
					//MsgBox(TEXT("Prop of %d list index %d"), filter, selected);
					DoPropertiesDialog(filter);
					break;
				}
			case IDC_VIDEO_SELECT:
				{
					int selected = -1;
					HWND hwndCombo = GetDlgItem(hwnd, IDC_VIDEO_DEVICE_LIST);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);

					ChooseDevices(gcap.rgpmVideoMenu[selected], gcap.pmAudio);
					appData->VideoDeviceIndex = selected;

					hwndCombo = GetDlgItem(hwnd, IDC_DEVICE_FILTER_LIST);
					PopulateDeviceDropDownBox(hwndCombo);

					break;
				}
			case IDC_AUDIO_SELECT:
				{
					int selected = -1;
					HWND hwndCombo = GetDlgItem(hwnd, IDC_AUDIO_DEVICE_LIST);
					selected = SendMessage(hwndCombo, CB_GETCURSEL, 0, 0);

					ChooseDevices(gcap.pmVideo, gcap.rgpmAudioMenu[selected]);
					appData->AudioDeviceIndex = selected;

					hwndCombo = GetDlgItem(hwnd, IDC_DEVICE_FILTER_LIST);
					PopulateDeviceDropDownBox(hwndCombo);

					break;
				}
            }
            break;

        default:
            return FALSE;
    }
    return TRUE;
}

BOOL PopulateDeviceDropDownBox(HWND hwndCombo)
{
	//Reset the content of the combo first
	SendMessage(hwndCombo, CB_RESETCONTENT, 0, 0);
	HRESULT hr = 0;

	// Kepp track of the count
	int ListCount = 0;

    // If this device supports the old legacy UI dialogs, offer them
    if(gcap.pDlg && !gcap.pDlg->HasDialog(VfwCaptureDialog_Format))
	{
		SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"(VFW) Video Format");
		SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 1 );
    }
    if(gcap.pDlg && !gcap.pDlg->HasDialog(VfwCaptureDialog_Source))
	{
		SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"(VFW) Video Source");
		SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 2 );
    }
    if(gcap.pDlg && !gcap.pDlg->HasDialog(VfwCaptureDialog_Display))
	{
		SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"(VFW) Video Display");
		SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 3 );
    }

    // Also check the audio capture filter at this point, since even non wdm devices
    // may support an IAMAudioInputMixer property page (we'll also get any wdm filter
    // properties here as well). We'll get any audio capture pin property pages just
    // a bit later.
    if(gcap.pACap != NULL)
	{
        ISpecifyPropertyPages *pSpec;
        CAUUID cauuid;

        hr = gcap.pACap->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
        if(hr == S_OK)
		{
            hr = pSpec->GetPages(&cauuid);
            if(hr == S_OK && cauuid.cElems > 0)
			{
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Audio Capture Filter");
				SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 4 );
                CoTaskMemFree(cauuid.pElems);
            }
            pSpec->Release();
        }
    }

    // don't bother looking for new property pages if the old ones are supported
    // or if we don't have a capture filter
    if(gcap.pVCap == NULL || ListCount > 1)
        return true;

    // New WDM devices support new UI and new interfaces.
    // Your app can use some default property
    // pages for UI if you'd like (like we do here) or if you don't like our
    // dialog boxes, feel free to make your own and programmatically set 
    // the capture options through interfaces like IAMCrossbar, IAMCameraControl
    // etc.

    // There are 9 objects that might support property pages.  Let's go through
    // them.

    ISpecifyPropertyPages *pSpec;
    CAUUID cauuid;

    // 1. the video capture filter itself

    hr = gcap.pVCap->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
    if(hr == S_OK)
	{
        hr = pSpec->GetPages(&cauuid);
        if(hr == S_OK && cauuid.cElems > 0)
		{
			SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Video Capture Filter");
			SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 5 );
            CoTaskMemFree(cauuid.pElems);
        }
        pSpec->Release();
    }

    // 2.  The video capture capture pin
	IAMStreamConfig *pSC;
	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap, IID_IAMStreamConfig, (void **)&pSC);

    if(hr == S_OK)
	{
        hr = pSC->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
        if(hr == S_OK)
		{
            hr = pSpec->GetPages(&cauuid);
            if(hr == S_OK && cauuid.cElems > 0)
			{
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Video Capture Pin");
				SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 6 );
                CoTaskMemFree(cauuid.pElems);
            }
            pSpec->Release();
        }
        pSC->Release();
    }

    // 3.  The video capture preview pin.
    // This basically sets the format being previewed.  Typically, you
    // want to capture and preview using the SAME format, instead of having to
    // enter the same value in 2 dialog boxes.  For a discussion on this, see
    // the comment above the MakePreviewGraph function.

	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_PREVIEW,
		&MEDIATYPE_Video, gcap.pVCap, IID_IAMStreamConfig, (void **)&pSC);
    if(hr == S_OK)
	{
        hr = pSC->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
        if(hr == S_OK)
		{
            hr = pSpec->GetPages(&cauuid);
            if(hr == S_OK && cauuid.cElems > 0)
			{
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Video Preview Pin");
				SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 7 );
                CoTaskMemFree(cauuid.pElems);
            }
            pSpec->Release();
        }
        pSC->Release();
    }


    // 4 & 5.  The video crossbar, and a possible second crossbar

    IAMCrossbar *pX, *pX2;
    IBaseFilter *pXF;
	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap, IID_IAMCrossbar, (void **)&pX);

    if(hr == S_OK)
	{
        hr = pX->QueryInterface(IID_IBaseFilter, (void **)&pXF);
        if(hr == S_OK)
		{
            hr = pX->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
            if(hr == S_OK)
			{
                hr = pSpec->GetPages(&cauuid);
                if(hr == S_OK && cauuid.cElems > 0)
				{
					SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Video Crossbar");
					SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 8 );
                    CoTaskMemFree(cauuid.pElems);
                }
                pSpec->Release();
            }
            hr = gcap.pBuilder->FindInterface(&LOOK_UPSTREAM_ONLY, NULL, pXF,
                IID_IAMCrossbar, (void **)&pX2);
            if(hr == S_OK)
			{
                hr = pX2->QueryInterface(IID_ISpecifyPropertyPages,
                    (void **)&pSpec);
                if(hr == S_OK)
				{
                    hr = pSpec->GetPages(&cauuid);
                    if(hr == S_OK && cauuid.cElems > 0)
					{
						SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Second Video Crossbar");
						SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 9 );
                        CoTaskMemFree(cauuid.pElems);
                    }
                    pSpec->Release();
                }
                pX2->Release();
            }
            pXF->Release();
        }
        pX->Release();
    }

    // 6.  The TVTuner

    IAMTVTuner *pTV;

	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap, IID_IAMTVTuner, (void **)&pTV);
    if(hr == S_OK)
	{
        hr = pTV->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
        if(hr == S_OK)
		{
            hr = pSpec->GetPages(&cauuid);
            if(hr == S_OK && cauuid.cElems > 0)
			{
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"TV Tuner");
				SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 10 );
                CoTaskMemFree(cauuid.pElems);
            }
            pSpec->Release();
        }

        pTV->Release();
    }

    // no audio capture, then skip audio stuff
    if(gcap.pACap == NULL)
        goto FinishedAudio;

    // 7.  The Audio capture filter itself... Thanks anyway, but we got these already

    // 8.  The Audio capture pin

    hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
        &MEDIATYPE_Audio, gcap.pACap,
        IID_IAMStreamConfig, (void **)&pSC);
    if(hr == S_OK)
	{
        hr = pSC->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
        if(hr == S_OK)
		{
            hr = pSpec->GetPages(&cauuid);
            if(hr == S_OK && cauuid.cElems > 0)
			{
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"Audio Capture Pin");
				SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 11 );
                CoTaskMemFree(cauuid.pElems);
            }
            pSpec->Release();
        }
        pSC->Release();
    }

    // 9.  The TV Audio filter
    IAMTVAudio *pTVA;
    hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Audio, gcap.pACap, IID_IAMTVAudio, (void **)&pTVA);

	if (hr != S_OK)
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap, IID_IAMTVAudio, (void **)&pTVA);

    if(hr == S_OK)
	{
        hr = pTVA->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
        if(hr == S_OK)
		{
            hr = pSpec->GetPages(&cauuid);
            if(hr == S_OK && cauuid.cElems > 0)
			{
				SendMessage(hwndCombo, CB_ADDSTRING, 0, (LPARAM) (LPSTR)"TV Audio");
				SendMessage(hwndCombo, CB_SETITEMDATA, ListCount++, (LPARAM) 12 );
                CoTaskMemFree(cauuid.pElems);
            }
            pSpec->Release();
        }
        pTVA->Release();
    }

FinishedAudio:

	SendMessage(hwndCombo, CB_SETCURSEL, 0, 0);

	return true;
}

BOOL DoPropertiesDialog(int index)
{
	HRESULT hr = 0;

	// they want the VfW format dialog
	if(index == 1)
	{
		HRESULT hrD;
		hrD = gcap.pDlg->ShowDialog(VfwCaptureDialog_Format, ghwndApp);
		// Oh uh!  Sometimes bringing up the FORMAT dialog can result
		// in changing to a capture format that the current graph 
		// can't handle.  It looks like that has happened and we'll
		// have to rebuild the graph.
		if(hrD == VFW_E_CANNOT_CONNECT)
		{
			DbgLog((LOG_TRACE,1,TEXT("DIALOG CORRUPTED GRAPH!")));
			TearDownGraph();    // now we need to rebuild
			// !!! This won't work if we've left a stranded h/w codec
		}
	}
	else if(index == 2)
	{
		// this dialog will not work while previewing
		gcap.pDlg->ShowDialog(VfwCaptureDialog_Source, ghwndApp);
	}
	else if(index == 3)
	{
		// this dialog will not work while previewing
		gcap.pDlg->ShowDialog(VfwCaptureDialog_Display, ghwndApp);
	}
	else if(index == 4)
	{
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = gcap.pACap->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&gcap.pACap, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
	}
	else if(index == 5)
	{
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = gcap.pVCap->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&gcap.pVCap, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
	}
	else if(index == 6)
	{
		IAMStreamConfig *pSC;
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
			&MEDIATYPE_Video, gcap.pVCap,
			IID_IAMStreamConfig, (void **)&pSC);
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = pSC->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&pSC, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
		pSC->Release();

		// Set the capture Video Format
		//if(SetVideoFormat() == FALSE)
			//return FALSE;
	}
	else if(index == 7)
	{
		
		IAMStreamConfig *pSC;
		
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_PREVIEW,
			&MEDIATYPE_Video, gcap.pVCap,
			IID_IAMStreamConfig, (void **)&pSC);
		
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = pSC->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&pSC, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
		pSC->Release();
	}
	else if(index == 8)
	{
		IAMCrossbar *pX;
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
			&MEDIATYPE_Video, gcap.pVCap,
			IID_IAMCrossbar, (void **)&pX);
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = pX->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&pX, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
		pX->Release();
	}
	else if(index == 9)
	{
		IAMCrossbar *pX, *pX2;
		IBaseFilter *pXF;
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
			&MEDIATYPE_Video, gcap.pVCap,
			IID_IAMCrossbar, (void **)&pX);

		hr = pX->QueryInterface(IID_IBaseFilter, (void **)&pXF);
		hr = gcap.pBuilder->FindInterface(&LOOK_UPSTREAM_ONLY, NULL,
			pXF, IID_IAMCrossbar, (void **)&pX2);
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = pX2->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&pX2, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
		pX2->Release();
		pXF->Release();
		pX->Release();
	}
	else if(index == 10)
	{
		IAMTVTuner *pTV;
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
			&MEDIATYPE_Video, gcap.pVCap,
			IID_IAMTVTuner, (void **)&pTV);
		
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = pTV->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&pTV, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
		
		pTV->Release();
	}
	else if(index == 11)
	{
		IAMStreamConfig *pSC;
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
			&MEDIATYPE_Audio, gcap.pACap,
			IID_IAMStreamConfig, (void **)&pSC);
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = pSC->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&pSC, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
		pSC->Release();
//		if(appData->fWantPreview)
//			StartPreview();
	}
	else if(index == 12)
	{
		IAMTVAudio *pTVA;
		hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
			&MEDIATYPE_Audio, gcap.pACap,
			IID_IAMTVAudio, (void **)&pTVA);

		if (hr != S_OK)
			hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
				&MEDIATYPE_Video, gcap.pVCap,
				IID_IAMTVAudio, (void **)&pTVA);
		
		ISpecifyPropertyPages *pSpec;
		CAUUID cauuid;
		hr = pTVA->QueryInterface(IID_ISpecifyPropertyPages,
			(void **)&pSpec);
		if(hr == S_OK)
		{
			hr = pSpec->GetPages(&cauuid);
			hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
				(IUnknown **)&pTVA, cauuid.cElems,
				(GUID *)cauuid.pElems, 0, 0, NULL);
			CoTaskMemFree(cauuid.pElems);
			pSpec->Release();
		}
		pTVA->Release();
	}

	return true;
}

int FAR PASCAL PROP_TUNER_PROC(HWND hwnd, UINT msg, UINT wParam, LONG lParam) 
{

    switch(msg) {
		case WM_HELP:
		{
			HELPINFO *helpData;
			helpData = (LPHELPINFO)lParam;

			if(helpData->iCtrlId > -1)
			{
				POINT mouseLoc = helpData->MousePos;
				CHAR *helpText = new CHAR[1024];
				int charCount = GetHelpText(helpData->iCtrlId, helpText, 1024);

				ShowHelpPOPup(hwnd, helpText, mouseLoc.x, mouseLoc.y, charCount);
					
				delete [] helpText;
			}
			
			break;
		}
        case WM_INITDIALOG:
			{
				TCHAR *TunerNote = "Note:\r\n" \
					"You will need to set the TV tuner country code and input type " \
					"before doing a scan, select the device settings button, do " \
					"this before do starting the scan.\r\n\r\n" \
					"You can also use the number keys " \
					"and enter to change channels in the main window\r\ni.e.\r\n" \
					"1 0 <enter> \r\nThis will change to channel 10";

				SetDlgItemText(hwnd, IDC_TUNER_NOTE, TunerNote);

//				TCHAR *tach = new TCHAR[64];
//				wsprintf(tach, TEXT("%d"), appData->TunerChannel);
//				SetDlgItemText(hwnd, IDC_CHANNEL, tach);
//				delete []tach;

//				SendMessage(GetDlgItem( hwnd, IDC_CHANNEL_SPIN ),
//					UDM_SETRANGE, 0, MAKELPARAM(200, 0));

				// Set up the Channels Data

				CHAR channelTEXT[256];

				HWND hwndList = GetDlgItem(hwnd, IDC_CHANNELS);
				SendMessage(hwndList, LB_RESETCONTENT, 0, 0);

				for(int x = 0; x < 256; x++)
				{
					if(appData->channels[x] != NULL)
					{
						appData->channels[x]->getChannelTEXT(channelTEXT, 255);
						SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) channelTEXT);
					}
					else
						break;
				}

				break;
			}

		case WM_NOTIFY:
			switch(((NMHDR FAR *) lParam)->code)
			{
			case PSN_APPLY:
				{
					//int newChannel = 
					//	GetDlgItemInt(hwnd, IDC_CHANNEL, NULL, FALSE);
					//if(newChannel > -1 && newChannel < 1000)
					//{	
					//	appData->TunerChannel = newChannel;
					//	changeChannel(appData->TunerChannel);
					//}

					break;
				}
			case PSN_QUERYCANCEL:
				{
					//changeChannel(appData->TunerChannel);
					break;
				}

			break;
			}


		case WM_COMMAND:
			switch(HIWORD(wParam))
			{
	  			case EN_CHANGE:
				{
//	    			switch( LOWORD(wParam) )
//	     			{
//					case IDC_CHANNEL:
//						{

//							break;
//						}
//					}
//					break;
				}

	   			case CBN_SELCHANGE:
				{
	    			switch( LOWORD(wParam) )
	     			{
					case IDC_CHANNELS:
						{
							char channelTEXT[256];

							int selected = SendDlgItemMessage(hwnd, IDC_CHANNELS, LB_GETCURSEL, 0, 0);
							if(selected > -1 && appData->channels[selected] != NULL)
							{
								appData->channels[selected]->getChannelTEXT(channelTEXT, 256);

								SetDlgItemText(hwnd, IDC_CHANNELSHOWN, channelTEXT);
								SetDlgItemText(hwnd, IDC_CHANNELNAME, appData->channels[selected]->getChannelName());

								appData->TunerChannel = appData->channels[selected]->getChannel();
								changeChannel(appData->TunerChannel);
							}
						}
					}
					break;
				}

	   			case BN_CLICKED:
	    			switch( LOWORD(wParam) )
	     			{
					case IDC_UPDATECHANNEL:
						{
							char channelName[256];
							int selected = SendDlgItemMessage(hwnd, IDC_CHANNELS, LB_GETCURSEL, 0, 0);
							if(selected < 0)
								break;

							int newChannelNumber = 
								GetDlgItemInt(hwnd, IDC_CHANNELSHOWN, NULL, FALSE);

							if(newChannelNumber < -1 || newChannelNumber > 999)
								break;

							GetDlgItemText(hwnd, IDC_CHANNELNAME, channelName, 255);

							if(appData->channels[selected] != NULL)
							{
								appData->channels[selected]->setChannelMAP(newChannelNumber);
								appData->channels[selected]->setChannelName(channelName);
							}

							// Set up the Channels Data
							CHAR channelTEXT[256];

							HWND hwndList = GetDlgItem(hwnd, IDC_CHANNELS);
							SendMessage(hwndList, LB_RESETCONTENT, 0, 0);

							for(int x = 0; x < 256; x++)
							{
								if(appData->channels[x] != NULL)
								{
									appData->channels[x]->getChannelTEXT(channelTEXT, 255);
									SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) channelTEXT);
								}
								else
									break;
							}


							break;
						}
					case IDC_DELETECHANNEL:
						{
							int selected = SendDlgItemMessage(hwnd, IDC_CHANNELS, LB_GETCURSEL, 0, 0);
							if(selected < 0)
								break;
							
							if(appData->channels[selected] != NULL)
							{
								delete appData->channels[selected];
								appData->channels[selected] = NULL;
							}
							
							selected++;
							while(appData->channels[selected] != NULL)
							{
								appData->channels[selected - 1] = appData->channels[selected];
								appData->channels[selected] = NULL;
								selected++;
							}

							// Set up the Channels Data
							CHAR channelTEXT[256];

							HWND hwndList = GetDlgItem(hwnd, IDC_CHANNELS);
							SendMessage(hwndList, LB_RESETCONTENT, 0, 0);

							for(int x = 0; x < 256; x++)
							{
								if(appData->channels[x] != NULL)
								{
									appData->channels[x]->getChannelTEXT(channelTEXT, 255);
									SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) channelTEXT);
								}
								else
									break;
							}
							SetDlgItemText(hwnd, IDC_CHANNELSHOWN, "");
							SetDlgItemText(hwnd, IDC_CHANNELNAME, "");

							break;
						}

					case IDC_SCANFORCHANNELS:
						{
							int cont = MessageBox(
								ghwndApp,
								"Doing a scan for TV channels will erase any current channel data!\r\nAre you sure you want to continue?",
								"Are You Sure",
								MB_YESNO|MB_ICONINFORMATION|MB_TASKMODAL);

							if(cont != IDYES)
								break;

							DoDialog(hwnd, IDD_CHANNELSCAN,(DLGPROC) ScanProgProc, 0);

							// Set up the Channels Data
							CHAR channelTEXT[256];

							HWND hwndList = GetDlgItem(hwnd, IDC_CHANNELS);
							SendMessage(hwndList, LB_RESETCONTENT, 0, 0);

							for(int x = 0; x < 256; x++)
							{
								if(appData->channels[x] != NULL)
								{
									appData->channels[x]->getChannelTEXT(channelTEXT, 255);
									SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) channelTEXT);
								}
								else
									break;
							}
							SetDlgItemText(hwnd, IDC_CHANNELSHOWN, "");
							SetDlgItemText(hwnd, IDC_CHANNELNAME, "");

							break;
						}

					case IDC_TUNER_PROP:
						{
							HRESULT hr = 0;
							IAMTVTuner *pTV = NULL;

							if(gcap.pBuilder == NULL || gcap.pVCap == NULL)
								break;

							hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
								&MEDIATYPE_Video, gcap.pVCap,
								IID_IAMTVTuner, (void **)&pTV);
							
							if(!pTV)
							{
								ErrMsg(TEXT("No TV Tuner Device Found"));
								break;
							}


							ISpecifyPropertyPages *pSpec;
							CAUUID cauuid;
							hr = pTV->QueryInterface(IID_ISpecifyPropertyPages,
								(void **)&pSpec);
							if(hr == S_OK)
							{
								hr = pSpec->GetPages(&cauuid);
								hr = OleCreatePropertyFrame(ghwndApp, 30, 30, NULL, 1,
									(IUnknown **)&pTV, cauuid.cElems,
									(GUID *)cauuid.pElems, 0, 0, NULL);
								CoTaskMemFree(cauuid.pElems);
								pSpec->Release();
							}
							
							pTV->Release();
							break;
						}
					}
			}


        default:
            return FALSE;
	}
    return TRUE;
}

void toggleFullScreen()
{
	IVideoWindow *videoPreviewInterface = NULL;
	gcap.pFg->QueryInterface(IID_IVideoWindow, (void **)&videoPreviewInterface);

    if(videoPreviewInterface == NULL)
	{
        ErrMsg(TEXT("Could Not Get Video Window Interface"));
		return;
	}

	videoPreviewInterface->put_MessageDrain((OAHWND)ghwndApp);

	long fullScreen = 0;
	videoPreviewInterface->get_FullScreenMode(&fullScreen);

//	MsgBox(TEXT("Changing to full screen mode %d"), fullScreen);

	if(fullScreen == 0)
		videoPreviewInterface->put_FullScreenMode(OATRUE);
	else
		videoPreviewInterface->put_FullScreenMode(OAFALSE);

	videoPreviewInterface->Release();
	videoPreviewInterface = NULL;

}

int hrToText(HRESULT hr, CHAR *error)
{
				
	if(hr == S_OK)
		strcpy(error, "S_OK");
	else if(hr == E_FAIL)
		strcpy(error, "E_FAIL");
	else if(hr == S_FALSE)
		strcpy(error, "S_FALSE");
	else if(hr == E_OUTOFMEMORY)
		strcpy(error, "E_OUTOFMEMORY");
	else if(hr == E_POINTER)
		strcpy(error, "E_POINTER");
	else if(hr == VFW_E_INVALIDMEDIATYPE)
		strcpy(error, "VFW_E_INVALIDMEDIATYPE");
	else if(hr == VFW_E_NOT_CONNECTED)
		strcpy(error, "VFW_E_NOT_CONNECTED");
	else if(hr == VFW_E_NOT_STOPPED)
		strcpy(error, "VFW_E_NOT_STOPPED");
	else if(hr == VFW_E_WRONG_STATE)
		strcpy(error, "VFW_E_WRONG_STATE");
	else if(hr == VFW_E_NOT_FOUND)
		strcpy(error, "VFW_E_NOT_FOUND");
	else
		strcpy(error, "Code Not Known");
				
	return 0;
}

int getFilterVersion(char *ver, GUID filterGUID)
{
	int result = 0;

	if(ver == NULL)
		return -1;

	IBaseFilter *filter = NULL;

	CoCreateInstance(
		filterGUID,
		NULL,
		CLSCTX_INPROC, 
		IID_IBaseFilter, 
		(void **)&filter);
	
	if(filter != NULL)
	{
		IFilterProperties *filterProp = NULL;
		
		filter->QueryInterface(IID_IFilterProperties,
			(void **)&filterProp);
		
		if(filterProp)
		{
			filterProp->get_Version(ver);
			filterProp->Release();
		}
		else
		{
			strcpy(ver, "N/A");
			result = -2;
		}
		
		filter->Release();
	}
	else
	{
		strcpy(ver, "N/A");
		result = -3;
	}

	return result;
}

// CreateAToolBar creates a toolbar and adds a set of buttons to it.
// The function returns the handle to the toolbar if successful, 
// or NULL otherwise. 
// hwndParent is the handle to the toolbar's parent window. 

HWND CreateToolBar(HWND hwndParent) 
{ 
	HWND hwndTB = NULL; 
	int NUM_BUTTONS = 12;
	TBBUTTON tbb[12]; 
	INITCOMMONCONTROLSEX icex;
	int lResult = 0;
	
	// Ensure that the common control DLL is loaded. 
	icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
	icex.dwICC  = ICC_BAR_CLASSES;
	InitCommonControlsEx(&icex);
	
	// Create a toolbar. 
	hwndTB = CreateWindowEx(0, TOOLBARCLASSNAME, (LPSTR) NULL, 
        WS_CHILD | CCS_ADJUSTABLE | TBSTYLE_TOOLTIPS, 0, 0, 0, 0, hwndParent, 
        (HMENU) ID_TOOLBAR, ghInstApp, NULL); 

	// Set the Button Size
	lResult = SendMessage(hwndTB,
		TB_SETBUTTONSIZE,         // message ID
        (WPARAM) 0,                 // = 0; not used, must be zero 
        (LPARAM) MAKELONG (20, 20)
        );	

	// Set the Bitmap size
	lResult = SendMessage(hwndTB,
		TB_SETBITMAPSIZE,
		(WPARAM) 0,
		(LPARAM) MAKELONG (20, 20)
		);

	SendMessage(hwndTB,
		TB_SETMAXTEXTROWS,
		(WPARAM) 0,      // = (WPARAM) (INT) iMaxRows;
		(LPARAM) 0
		);  

	// Send the TB_BUTTONSTRUCTSIZE message, which is required for 
	// backward compatibility. 
	SendMessage(hwndTB, TB_BUTTONSTRUCTSIZE, (WPARAM) sizeof(TBBUTTON), 0); 

	TBADDBITMAP bitmap;
	bitmap.hInst = ghInstApp;
	bitmap.nID = IDB_TOOLBAR;

	lResult = SendMessage(hwndTB,
		TB_ADDBITMAP,
		(WPARAM) 4,
		(LPARAM) (LPTBADDBITMAP) &bitmap);  

	// Fill the TBBUTTON array with button information, and add the 
	// buttons to the toolbar. The buttons on this toolbar have text 
	// but do not have bitmap images. 

	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Settings [C]"); //0
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Save Current Settings"); //1 
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Load from File"); //2
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Save to File"); //3
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Start Capture [F5]"); //4
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Stop Capture [ESC]"); //5
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Toggle Preview [P]"); //6 
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Toggle Stats [S]"); //7
	SendMessage(hwndTB, TB_ADDSTRING, 0, (LPARAM) (LPSTR) "Show Dropped Frames"); //8

	tbb[0].iBitmap = 3;
	tbb[0].idCommand = MENU_SETTINGS;
	tbb[0].fsState = TBSTATE_ENABLED;
	tbb[0].fsStyle = TBSTYLE_BUTTON;
	tbb[0].dwData = 0;
	tbb[0].iString = 0;

	tbb[1].fsStyle   = TBSTYLE_BUTTON|TBSTYLE_SEP; 
	tbb[1].iBitmap   = 8;   // Width of separator 
	tbb[1].idCommand = NULL;
	tbb[1].fsState   = TBSTATE_ENABLED; 
	tbb[1].dwData    = 0; 
	tbb[1].iString   = NULL; 

	tbb[2].iBitmap = 0; 
	tbb[2].idCommand = MENU_SAVESETTINGS; 
	tbb[2].fsState = TBSTATE_ENABLED; 
	tbb[2].fsStyle = TBSTYLE_BUTTON; 
	tbb[2].dwData = 0; 
	tbb[2].iString = 1; 
	
	tbb[3].iBitmap = 1; 
	tbb[3].idCommand = MENU_LOADFROMFILE; 
	tbb[3].fsState = TBSTATE_ENABLED; 
	tbb[3].fsStyle = TBSTYLE_BUTTON; 
	tbb[3].dwData = 0; 
	tbb[3].iString = 2; 

	tbb[4].iBitmap = 2; 
	tbb[4].idCommand = MENU_SAVETOFILE; 
	tbb[4].fsState = TBSTATE_ENABLED; 
	tbb[4].fsStyle = TBSTYLE_BUTTON; 
	tbb[4].dwData = 0; 
	tbb[4].iString = 3; 

	tbb[5].fsStyle   = TBSTYLE_BUTTON|TBSTYLE_SEP; 
	tbb[5].iBitmap   = 8;   // Width of separator 
	tbb[5].idCommand = NULL; 
	tbb[5].fsState   = TBSTATE_ENABLED; 
	tbb[5].dwData    = 0; 
	tbb[5].iString   = NULL; 

	tbb[6].iBitmap = 4; 
	tbb[6].idCommand = MENU_START_CAP; 
	tbb[6].fsState = TBSTATE_ENABLED; 
	tbb[6].fsStyle = TBSTYLE_BUTTON; 
	tbb[6].dwData = 0; 
	tbb[6].iString = 4; 

	tbb[7].iBitmap = 5; 
	tbb[7].idCommand = MENU_STOP_CAP; 
	tbb[7].fsState = TBSTATE_ENABLED | TBSTATE_HIDDEN; 
	tbb[7].fsStyle = TBSTYLE_BUTTON; 
	tbb[7].dwData = 0; 
	tbb[7].iString = 5; 

	tbb[8].fsStyle   = TBSTYLE_BUTTON|TBSTYLE_SEP; 
	tbb[8].iBitmap   = 8;   // Width of separator 
	tbb[8].idCommand = NULL; 
	tbb[8].fsState   = TBSTATE_ENABLED; 
	tbb[8].dwData    = 0; 
	tbb[8].iString   = NULL; 

	tbb[9].iBitmap = 6; 
	tbb[9].idCommand = MENU_PREVIEW; 
	tbb[9].fsState = TBSTATE_ENABLED; 
	tbb[9].fsStyle = TBSTYLE_BUTTON; 
	tbb[9].dwData = 0; 
	tbb[9].iString = 6; 

	tbb[10].iBitmap = 7; 
	tbb[10].idCommand = MENU_SHOWSTATS; 
	tbb[10].fsState = TBSTATE_ENABLED; 
	tbb[10].fsStyle = TBSTYLE_BUTTON;
	tbb[10].dwData = 0; 
	tbb[10].iString = 7; 

	tbb[11].iBitmap = 8; 
	tbb[11].idCommand = MENU_SHOW_DROPPED_FRAMES; 
	tbb[11].fsState = TBSTATE_ENABLED; 
	tbb[11].fsStyle = TBSTYLE_BUTTON;
	tbb[11].dwData = 0;
	tbb[11].iString = 8;

	SendMessage(hwndTB, TB_ADDBUTTONS, (WPARAM) NUM_BUTTONS, 
        (LPARAM) (LPTBBUTTON) &tbb); 
	
	SendMessage(hwndTB, TB_AUTOSIZE, 0, 0);
	
	ShowWindow(hwndTB, SW_SHOW); 

	SIZE toolBarSize;
	lResult = SendMessage(hwndTB,
		TB_GETMAXSIZE,
		(WPARAM) 0,
		(LPARAM) (LPSIZE) &toolBarSize);
	
	gcap.toolBarHeight = toolBarSize.cy + 5;

	//MsgBox(TEXT("%d %d"), toolBarSize.cx, toolBarSize.cy); 

	return hwndTB; 
}

// This is the thread called to do the channel scanning
void scannForTVChannels( void *mode )
{
	CHAR channelUpdate[256];
	int agressive = (int) mode;

	if(gcap.log) logAction(TEXT("scannForTVChannels(%d)"), agressive);

	gcap.channelScanning = TRUE;

	int numberCount = 0;
	appData->deleteChannelData();

	HRESULT hr;
	IAMTVTuner *pTV = NULL;

	if(gcap.pBuilder == NULL || gcap.pVCap == NULL)
	{
		wsprintf(channelUpdate, "Scan Finished, Found %d Channels", 
			numberCount);

		if(gcap.channelScanningStatus)
		{
			SendMessage(GetDlgItem(gcap.channelScanningStatus, IDC_SCANPROG),
				WM_SETTEXT,
				0, 
				(LPARAM)"No Video Capture Deice Found");

			SendMessage(GetDlgItem(gcap.channelScanningStatus, IDC_STOPSCAN),
				WM_SETTEXT,
				0, 
				(LPARAM)"CLOSE");
		}
		return;
	}

	hr = gcap.pBuilder->FindInterface(&PIN_CATEGORY_CAPTURE,
		&MEDIATYPE_Video, gcap.pVCap,
		IID_IAMTVTuner, (void **)&pTV);

	if(!pTV)
	{
		wsprintf(channelUpdate, "Scan Finished, Found %d Channels", 
			numberCount);

		if(gcap.channelScanningStatus)
		{
			SendMessage(GetDlgItem(gcap.channelScanningStatus, IDC_SCANPROG),
				WM_SETTEXT,
				0, 
				(LPARAM)"No Tuner Deice Found");

			SendMessage(GetDlgItem(gcap.channelScanningStatus, IDC_STOPSCAN),
				WM_SETTEXT,
				0, 
				(LPARAM)"CLOSE");
		}
		return;
	}

	long minChannel = 0;
	long maxChannel = 0;
	long signal = 0;

	pTV->ChannelMinMax(&minChannel, &maxChannel);

	for(long channel = minChannel; channel <= maxChannel; channel++)
	{
		if(gcap.channelScanning == FALSE)
			break;

		HRESULT hr = pTV->put_Channel(channel, AMTUNER_SUBCHAN_DEFAULT, AMTUNER_SUBCHAN_DEFAULT);
		pTV->SignalPresent(&signal);

		wsprintf(channelUpdate, "Scanning : %d of %d Found : %d", 
			(channel-minChannel)+1, 
			(maxChannel-minChannel)+1,
			numberCount);

		if(gcap.channelScanningStatus)
		{
			SendMessage(GetDlgItem(gcap.channelScanningStatus, IDC_SCANPROG),
				WM_SETTEXT,
				0, 
				(LPARAM)channelUpdate);
		}

		if(gcap.log)
		{
			logAction(
				TEXT("Scanning Result for : %d = (signal=%d return=%d)"),
				channel,
				signal,
				hr);
		}

		if((signal != AMTUNER_NOSIGNAL) && (hr == S_OK || agressive == 1))
		{
			appData->channels[numberCount] = new CTVstation();
			appData->channels[numberCount]->setChannel(channel);
			appData->channels[numberCount]->setChannelMAP(channel);

			numberCount++;
		}

	}

	pTV->StoreAutoTune();
	pTV->put_TuningSpace(0);
	pTV->Release();

	wsprintf(channelUpdate, "Scan Finished, Found %d Channels", 
		numberCount);

	if(gcap.channelScanningStatus)
	{
		SendMessage(GetDlgItem(gcap.channelScanningStatus, IDC_SCANPROG),
			WM_SETTEXT,
			0, 
			(LPARAM)channelUpdate);

		SendMessage(GetDlgItem(gcap.channelScanningStatus, IDC_STOPSCAN),
			WM_SETTEXT,
			0, 
			(LPARAM)"CLOSE");
	}

//	return numberCount;
}

int ChannelChange(int amount)
{

	int currentINDEX = -1;
	int maxINDEX = -1;
	char status[256];
	const char *name;

	for(int x = 0; x < 256; x++)
	{
		if(appData->channels[x] != NULL)
		{
			if(appData->channels[x]->getChannel() == appData->TunerChannel)
			{
				currentINDEX = x;
			}
			maxINDEX = x;
		}
		else
			break;
	}

	if(currentINDEX < 0 || currentINDEX > 255)
		currentINDEX = 0;

	int newChannel = -1;
	int newChannelMAP = -1;
	int newINDEX = currentINDEX + amount;

	if(newINDEX < 0) newINDEX = maxINDEX;
	if(newINDEX > maxINDEX) newINDEX = 0;

	if(appData->channels[newINDEX] != NULL)
	{
		newChannel = appData->channels[newINDEX]->getChannel();
		name = appData->channels[newINDEX]->getChannelName();
		newChannelMAP = appData->channels[newINDEX]->getChannelMAP();
	}

	if(newChannel > -1)
	{
		appData->TunerChannel = newChannel;
		changeChannel(appData->TunerChannel);

		wsprintf(status, TEXT("Channel : %d - %s"), newChannelMAP, name);
		statusUpdateStatus(ghwndStatus, status, 1);
	}
	
	return 0;
}